/**
 * Abbreviates a number
 * Note: consider big numbers with https://github.com/Patashu/break_infinity.js
 * @moduleName Abbreviate
 * @snippet Abbreviate|constructor
Abbreviate($1)
 */
bento.define('abbreviate', [
    'bento',
    'bento/utils'
], function (
    Bento,
    Utils
) {
    'use strict';
    var notations = ['', 'k', 'm', 'b', 't']; // after this comes aa, ab, ac etc.
    var log1000 = Math.log(1000);
    var l = notations.length;
    var Abbreviate = function (number) {
        if (number <= 999) {
            return number.toString();
        }
        // todo: memoize results?
        var n = Math.floor(Math.log(number) / log1000);
        var m = number / Math.pow(1000, n);
        var p = n - l;
        var abbreviated = (Math.floor(m * 100) / 100).toFixed(2);

        if (abbreviated.endsWith('.00')) {
            // trim zeros
            abbreviated = abbreviated.replace('.00', '');
        } else if (abbreviated.endsWith('0')) {
            // trim zeros
            abbreviated = abbreviated.slice(0, -1);
        } else if (abbreviated.length === 5) {
            // the whole text is too long 23.45X -> 23.4X
            abbreviated = abbreviated.slice(0, -1);
        } else if (abbreviated.length === 6) {
            // the whole text is too long 123.45X -> 123X
            abbreviated = abbreviated.slice(0, -3);
        }

        if (n < l) {
            return abbreviated + notations[n];
        } else {
            return abbreviated + String.fromCharCode(97 + Math.floor(p / 26),97 + (p % 26));
        }
    };
    // debugging
    if (Bento.isDev()) {
        window.Abbreviate = Abbreviate;
    }
    return Abbreviate;
});