/**
 * 3D Healthbar above enemies
 * @moduleName HealthBar3D
 * @snippet HealthBar3D|Constructor
HealthBar3D({
    
})
 */
bento.define('components/healthbar3d', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'onigiri/spriteplane',
    'onigiri/onigiri',
    'onigiri/entity3d'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    SpritePlane,
    Onigiri,
    Entity3D
) {
    'use strict';
    return function (settings) {
        var entity;
        var height = settings.height || 36 * Onigiri.pxToThree;
        var width = settings.width || 20 * Onigiri.pxToThree;
        var targetEntity = settings.targetEntity;
        var percent = 1;
        var originalHealth = 1;
        var bar = new SpritePlane({
            width: width,
            imageName: 'gui/battery-inner',
        });
        var back = new SpritePlane({
            width: width,
            position: new THREE.Vector3(0, 0, 0.1),
            imageName: 'gui/battery',
        });
        var front = new SpritePlane({
            width: width,
            position: new THREE.Vector3(0, 0, 0.1),
            imageName: 'gui/battery-red',
        });
        var clippingPlane = new THREE.Plane(new THREE.Vector3(-1, 0, 0), 0);
        var container = new Entity3D({
            name: 'container',
            position: new THREE.Vector3(0, height, 0),
            components: [
                back,
                bar,
                front
            ]
        });

        var component = new Object({
            name: 'healthbar3D',
            start: function (data) {
                Bento.objects.attach(container);

                // requires renderer.localClippingEnabled = true 
                data.renderer.three.renderer.localClippingEnabled = true;

                bar.sprite.mesh.material.clippingPlanes = [clippingPlane];

                originalHealth = targetEntity.health;
                container.visible = false;
            },
            destroy: function (data) {
                Bento.objects.remove(container);
            },
            update: function (data) {
                container.position.copy(targetEntity.position);
                container.position.y += height * 1.5;
                container.position.z = 5;

                percent = Utils.approach(percent, targetEntity.health / originalHealth, 1 / 10);

                if (percent < 0.10) {
                    front.sprite.visible = (Math.round(container.timer / 30) % 2) === 0;
                    front.sprite.object3D.visible = front.sprite.visible;
                } else {
                    front.sprite.visible = false;
                }
            },
            draw: function (data) {
                clippingPlane.constant = container.position.x - (width * 0.5) + (width * percent);
            },
            attached: function (data) {
                entity = data.entity;
            },
            setPercent: function (value) {
                percent = value;
            },
            setVisible: function (value) {
                container.visible = value;
            }
        });

        return component;
    };
});