/**
 * Draws a line
 * @moduleName Line
 * @snippet Line|Constructor
Line({})
 */
bento.define('components/line', [
    'bento',
    'bento/utils',
    'onigiri/entity3d',
    'onigiri/spriteplane',
    'onigiri/onigiri',
    'bento/math/vector2',
    'globals',
    'modules/tapticengine',
    'onigiri/primitive'
], function (
    Bento,
    Utils,
    Entity3D,
    SpritePlane,
    Onigiri,
    Vector2,
    Globals,
    TapticEngine,
    Primitive
) {
    'use strict';
    var cacheMaterial;
    var Line = function (settings) {
        var component;
        var container = new Entity3D({
            name: 'line',
            components: [

            ]
        });
        var distanceBetweenDots = 1;
        var totalLength = 0;
        var previousPoint;
        var previousDot;
        var positions = [];
        var sprites = [];
        var hapticCooldown = 0;

        component = new Object({
            name: 'line',
            start: function () {
                Bento.objects.attach(container);
            },
            destroy: function () {
                Bento.objects.remove(container);
            },
            update: function () {
                if (hapticCooldown) {
                    hapticCooldown -= 1;
                }
            },
            setLine: function (vec3Array, lastAngle) {
                /*positions = [];
                totalLength = 0;
                Utils.forEach(vec3Array, function (vec3, i, l) {
                    var distance;
                    positions.push(vec3.x, vec3.y, vec3.z || 0);

                    if (previousPoint) {
                        distance = previousPoint.distanceTo(vec3);
                        totalLength += distance;

                        dotDistance += distance;

                        while (dotDistance > distanceBetweenDots) {
                            dotDistance -= distanceBetweenDots;
                            addDot();
                        }
                    }
                    previousPoint = vec3;
                });*/
                // append points
                var i, l;
                for (i = positions.length - 1, l = vec3Array.length; i < l; ++i) {
                    var vec3 = vec3Array[i];
                    component.addPoint(vec3);
                }
            },
            addPoint: function (vec3) {
                var distance, angle;
                positions.push(vec3);

                if (previousPoint) {
                    distance = previousPoint.distanceTo(vec3);
                    totalLength += distance;


                    if (!previousDot) {
                        angle = Math.atan2(vec3.y - previousPoint.y, vec3.x - previousPoint.x);
                        addDot(vec3.clone(), angle, totalLength);
                    } else {
                        var dist = previousDot.position.distanceTo(vec3);
                        var dotIndex = 0,
                            pos;
                        while (dist >= distanceBetweenDots) {
                            // dist -= distanceBetweenDots;
                            pos = previousDot.position.clone();
                            // re-calculate angle to be sure
                            angle = Math.atan2(vec3.y - previousDot.position.y, vec3.x - previousDot.position.x);
                            pos.x += distanceBetweenDots * Math.cos(angle);
                            pos.y += distanceBetweenDots * Math.sin(angle);
                            addDot(pos, angle, totalLength);
                            dist = previousDot.position.distanceTo(vec3);
                            dotIndex += 1;
                            if (dotIndex > 50) {
                                break;
                            }
                        }
                    }
                }
                previousPoint = vec3;
            },
            visibleFrom: function (distance) {
                Utils.forEach(sprites, function (sprite) {
                    if (sprite.distanceFromStart < distance) {
                        sprite.visible = false;
                    }
                });
            },
            reset: function () {
                Utils.forEach(sprites, function (sprite) {
                    container.remove(sprite);
                });
                totalLength = 0;
                previousDot = null;
                previousPoint = null;
                sprites = [];
                positions = [];
            }
        });
        // use a single material for all sprites
        var addDot = function (position, rotation, distanceFromStart) {
            // TODO: make this more efficient by reusing materials
            // var sprite3d = new SpritePlane({
            //     width: 1,
            //     imageName: 'gui/dash',
            //     position: position,
            //     originRelative: new Vector2(0.5, 0.5),
            //     color: Globals.getLineColor()
            // });
            var sprite3d = new Primitive({
                name: 'container',
                shape: 'plane',
                parameters: [
                    1, // width
                    1 // height
                ],
                position: position,
                material: cacheMaterial,
                disposeGeometry: true,
                disposeMaterial: false,
                // castShadow: true,
                // receiveShadow: true
            });
            sprite3d.euler.z = rotation;
            sprite3d.distanceFromStart = distanceFromStart;
            container.attach(sprite3d);
            sprites.push(sprite3d);
            previousDot = sprite3d;

            if (hapticCooldown === 0) {
                TapticEngine.impact({
                    type: "light"
                });
                hapticCooldown = 4;
            }
        };

        if (!cacheMaterial) {
            cacheMaterial = new THREE.MeshBasicMaterial({
                color: Globals.getLineColor(),
                map: Onigiri.getTexture('gui/dash'),
                transparent: true
            });
        }

        return component;
    };
    return Line;
});