/**
 * Directional light from above, add as a component to be lightweight. We don't need entities for Three
 * @moduleName Sun
 */
bento.define('components/sun', [
    'bento',
    'onigiri/onigiri',
    'onigiri/light',
    'onigiri/entity3d'
], function (
    Bento,
    Onigiri,
    Light,
    Entity3D
) {
    'use strict';
    return function (settings) {
        //directional sun
        var directionalLight = new Light({
            type: THREE.PointLight,
            position: settings.position || new THREE.Vector3(0, 0, 0),
            targetPosition: settings.targetPosition || new THREE.Vector3(0, 0, 0),
            color: settings.color || '#ffffff',
            intensity: settings.directionalIntensity || 0.5
        });

        if (settings.shadow || settings.castShadow) {
            //extra setup for shadows
            directionalLight.lightObject.castShadow = true;
            directionalLight.lightObject.shadow.radius = 8;
            // directionalLight.lightObject.shadow.bias = -0.001;
            directionalLight.lightObject.shadow.mapSize.width = 256;
            directionalLight.lightObject.shadow.mapSize.height = 256;
            directionalLight.lightObject.shadow.camera.left = -2 * settings.shadowSize;
            directionalLight.lightObject.shadow.camera.right = 2 * settings.shadowSize;
            directionalLight.lightObject.shadow.camera.top = 3 * settings.shadowSize;
            directionalLight.lightObject.shadow.camera.bottom = -3 * settings.shadowSize;
            directionalLight.lightObject.shadow.camera.near = 0.5 * settings.shadowSize;
            directionalLight.lightObject.shadow.camera.far = 25 * settings.shadowSize;

            if (settings.debug) {
                var shadowHelper = new THREE.CameraHelper(directionalLight.lightObject.shadow.camera);
                Onigiri.scene.add(shadowHelper);
            }
        }

        //ambient sun
        var ambientLight = new Light({
            type: THREE.AmbientLight,
            color: settings.color || '#ffffff',
            intensity: settings.ambientIntensity || 0.5
        });

        var light = new Entity3D({
            name: 'sun',
            position: new THREE.Vector3(0, 0, 0),
            components: [
                directionalLight,
                ambientLight
            ]
        });
        return light;
    };
});