/**
 * Set mesh to target size
 * @moduleName TargetSize
 * @snippet TargetSize|Constructor
TargetSize({
    width: ${1:5},
    height: ${2:null},
    depth: ${3:null},
    mesh: ${4:null} // takes parent's object3D if null 
})
 */
bento.define('components/targetsize', [
    'bento',
    'bento/utils',
    'onigiri/onigiri'
], function (
    Bento,
    Utils,
    Onigiri
) {
    'use strict';
    var TargetSize = function (settings) {
        var smallest = settings.smallest || null;
        var largest = settings.largest || null;
        var width = settings.width || null;
        var height = settings.height || null;
        var depth = settings.depth || null;
        var mesh = settings.mesh || null;
        var align = settings.alignOffset || null;
        var entity;
        var component = new Object({
            name: 'behavior',
            start: function (data) {
                if (!mesh) {
                    mesh = entity.object3D;
                }
                if (!mesh) {
                    // nothing to do
                    console.warn('Warning: no mesh found');
                    return;
                }
                var size = Onigiri.getMeshSize(mesh);

                if (largest) {
                    // determine which dimension is the largest
                    if (size.x >= size.y && size.x >= size.z) {
                        width = settings.largest;
                    } else if (size.y >= size.x && size.y >= size.z) {
                        height = settings.largest;
                    } else if (size.z >= size.x && size.z >= size.y) {
                        depth = settings.largest;
                    }
                } else if (smallest) {
                    // determine which dimension is the smallest
                    if (size.x <= size.y && size.x <= size.z) {
                        width = settings.largest;
                    } else if (size.y <= size.x && size.y <= size.z) {
                        height = settings.largest;
                    } else if (size.z <= size.x && size.z <= size.y) {
                        depth = settings.largest;
                    }
                }

                if (width) {
                    mesh.scale.x = width / size.x;
                }
                if (height) {
                    mesh.scale.y = height / size.y;
                }
                if (depth) {
                    mesh.scale.z = depth / size.z;
                }
                if (!width) {
                    mesh.scale.x = height ? mesh.scale.y : mesh.scale.z;
                }
                if (!height) {
                    mesh.scale.y = width ? mesh.scale.x : mesh.scale.z;
                }
                if (!depth) {
                    mesh.scale.z = width ? mesh.scale.x : mesh.scale.y;
                }

                if (align) {
                    mesh.position.x = align.x * (width || mesh.scale.x * size.x);
                    mesh.position.y = align.y * (height || mesh.scale.y * size.y);
                    mesh.position.z = align.z * (depth || mesh.scale.z * size.z);
                }
            },
            attached: function (data) {
                entity = data.entity;
            }
        });
        return component;
    };
    return TargetSize;
});