/**
 * 3D Trail using TrailRendererJS
 * @moduleName Trail
 * @snippet Trail|constructor
Trail({
    mesh: null
})
 */
bento.define('components/trail', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'onigiri/onigiri'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Onigiri
) {
    'use strict';
    return function (settings) {
        var mesh = settings.mesh;
        var isActivated = false;
        var entity;
        var component = new Object({
            name: 'trail',
            start: function (data) {
                component.activate();
            },
            destroy: function (data) {
                if (isActivated) {
                    isActivated = false;
                    trail.reset();
                    trail.deactivate();
                    Onigiri.scene.remove(trail.mesh);
                    if (trail.mesh && trail.mesh.geometry && trail.mesh.geometry.dispose) {
                        trail.mesh.geometry.dispose();
                    }
                    if (trail.mesh.dispose) {
                        trail.mesh.dispose();
                    }
                    trail.mesh = null;
                }
                if (trailMaterial.dispose) {
                    trailMaterial.dispose();
                }
            },
            update: function (data) {
                if (isActivated) {
                    trail.advance();
                    trail.updateHead();
                    if (component.hideTrailTimer) {
                        // workaround: hide trail showing at start of level 
                        trail.currentEnd = -1;
                        component.hideTrailTimer -= 1;
                    }
                } else {
                    component.activate();
                }
            },
            draw: function (data) {},
            attached: function (data) {
                entity = data.entity;
            },
            activate: function () {
                if (isActivated || !mesh) {
                    return;
                }

                isActivated = true;

                // initialize the trail
                trail.initialize(trailMaterial, trailLength, false, 0, trailHeadGeometry, mesh);
                trail.activate();
            },
            hideTrailTimer: 0,
            snap: function () {
                // workaround: hide trail showing at start of level 
                trail.currentEnd = trail.length;
                trail.currentLength = trail.length;
                // must wait until it goes into update
                component.hideTrailTimer = 2;
            }
        });
        // create the trail renderer object
        var trail = new THREE.TrailRenderer(Onigiri.scene, false);
        var trailHeadGeometry = [];
        var r = settings.size || 1;
        var segments = 8;
        var trailLength = settings.trailLength || 8;
        Utils.repeat(segments, function (i, l) {
            trailHeadGeometry.push(
                new THREE.Vector3(r * Math.cos(2 * Math.PI / l * i), r * Math.sin(2 * Math.PI / l * i), 0)
            );
        });
        // close
        trailHeadGeometry.push(
            new THREE.Vector3(r, 0, 0)
        );
        // trailHeadGeometry.push(
        //     new THREE.Vector3(0, 0, 0),
        //     new THREE.Vector3(r, 0, 0),
        //     new THREE.Vector3(r, r, 0),
        //     new THREE.Vector3(0, r, 0)
        // );
        // create material for the trail renderer
        var trailMaterial = THREE.TrailRenderer.createBaseMaterial();
        trailMaterial.uniforms.headColor.value.set(1, 1, 1, 0.25);
        trailMaterial.uniforms.tailColor.value.set(1, 1, 1, 0.25);

        // specify length of trail

        return component;
    };
});