/**
 * Component that can be added to ad IconButtons to indicate that rewarded videos are loading
 * @moduleName WaitingForAd
 * @snippet WaitingForAd|Constructor
WaitingForAd({
    radius: 12,
    offset: new Vector2(0, 0)
})
 */
bento.define('components/waitingforad', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'modules/watchad',
    'bento/components/eventlistener',
    'gui/widgets/loadingspinner'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    WatchAd,
    EventListener,
    LoadingSpinner
) {
    'use strict';
    return function (settings) {
        var entity;
        var canShow = false;
        var loadingSpinner = new LoadingSpinner({
            name: 'loadingSpinner',
            radius: settings.radius || 12,
            fadeSpeed: 1 / 60,
            ballsToSpawn: 8,
            rotateBalls: false,
            imageName: 'gui/dot',
            position: settings.offset,
            components: [] 
        });
        var eventListener = new EventListener({
            name: 'adListener',
            eventName: 'rewardedVideoHasChangedAvailability',
            ignorePause: true,
            onEvent: function (data) {
                if (WatchAd.canShowRewarded() || Bento.isDev()) {
                    canShow = true;
                } else {
                    canShow = false;
                }
            }
        });
        var component = new Object({
            name: 'waitingForAd',
            start: function (data) {
                canShow = WatchAd.canShowRewarded() || Bento.isDev();
                entity.attach(eventListener);
                entity.attach(loadingSpinner);
            },
            destroy: function (data) {
                entity.remove(loadingSpinner);
                entity.remove(eventListener);
            },
            update: function (data) {
                // check every second
                if (Bento.objects.ticker % 60 === 0) {
                    canShow = WatchAd.canShowRewarded() || Bento.isDev();
                }

                loadingSpinner.visible = !canShow;
                if (entity.icon) {
                    entity.icon.visible = canShow;
                }
            },
            draw: function (data) {},
            attached: function (data) {
                entity = data.entity;
            }
        });
        return component;
    };
});