/**
 * Cheat overlay
 * @moduleName CheatOverlay
 * @snippet CheatOverlay|constructor
CheatOverlay({})
 */
bento.define('debug/cheat', [
    'bento',
    'bento/utils',
    'debug/html/html',
    'debug/html/accordion',
    'bento/eventsystem',
    'globals',
    'managers/savevalues',
    'modules/localization'
], function (
    Bento,
    Utils,
    Html,
    Accordion,
    EventSystem,
    Globals,
    SaveValues,
    Localization
) {
    'use strict';
    var addedCss = false;
    var CheatOverlay = function (settings) {
        this.container = null;
        this.shouldRefresh = false;
        this.totalLevel = false;
        Bento.objects.pause();
    };
    // helper functions
    /**
     * @snippet createSaveInput()|HTML
    createSaveInput('${1:saveKey}', '${2:label}', '${3:int}', function (key, newValue) {$4})
     */
    var createSaveInput = function (saveKey, label, type, onEdit) {
        // creates input that can edit save value
        var inputLabel = new Html('<p>' + label + '</p>');
        var input = new Html('input', {
            class: 'input',
            value: SaveValues[saveKey],
            onchange: function (evt) {
                if (type === 'int') {
                    SaveValues[saveKey] = parseInt(evt.target.value);
                } else if (type === 'float') {
                    SaveValues[saveKey] = parseFloat(evt.target.value);
                } else {
                    SaveValues[saveKey] = evt.target.value;
                }
                if (onEdit) {
                    onEdit(saveKey, evt.target.value);
                }
            }
        });
        var container = new Html('div', {
            class: 'container',
            children: [
                inputLabel,
                input
            ]
        });
        return container;

    };
    /**
     * @snippet createCheckbox()|HTML
    createCheckbox('${1:saveKey}', '${2:label}', function (checked) {$3})
     */
    var createCheckbox = function (saveKey, label, onEdit, checkedValue) {
        var id = saveKey + Date.now();
        var checkBoxContainer = new Html('<div class="field"></div>');
        var checkBox = new Html('input', {
            id: id,
            name: id,
            type: 'checkbox',
            class: 'switch is-rounded is-info',
            onchange: function (evt) {
                var value = evt.target.checked;
                Bento.saveState.save('cheat-' + saveKey, value);
                onEdit(value);
            }
        });
        var span = new Html('label', {
            for: id,
            innerHtml: label
        });

        checkBoxContainer.attach(checkBox);
        checkBoxContainer.attach(span);

        if (checkedValue !== undefined) {
            checkBox.checked = checkedValue;
            onEdit(checkedValue);
        } else if (Bento.saveState.load('cheat-' + saveKey, false)) {
            checkBox.checked = true;
            onEdit(true);
        }

        return checkBoxContainer;
    };

    /**
     * Show/hide cheat overlay
     */
    CheatOverlay.prototype.hide = function () {
        this.container.removeSelf();
        if (this.onHide) {
            this.onHide(this.shouldRefresh);
        }
        if (this.shouldRefresh) {
            // refresh screen
            Bento.screens.show(Bento.screens.getCurrentScreen().name);
        }
        // focus canvas
        var canvas = Html.getById('canvas');
        if (canvas) {
            canvas.focus();
        }
        Bento.objects.resume();

    };
    CheatOverlay.prototype.show = function (onHide) {
        var self = this;
        self.onHide = onHide;
        var nav = Html('<nav class="navbar" role="navigation" aria-label="main navigation" />').attach(
            Html('<div class="navbar-brand" />').attach([
                Html('<a id="close" class="navbar-item" href="#"><i class="fas fa-times-circle"></i></a>', {
                    onclick: function () {
                        self.hide();
                    }
                }),
                Html('<h1 id="frontpage-h1" class="title navbar-item">Cheats</h1>')
            ])
        );
        var cheatsDiv = new Html('div', {
            style: 'padding: 32px 16px;'
        });
        var container = Html('div', {
            id: 'cheatContainer',
            style: {
                display: 'block',
                position: 'absolute',
                top: '0px',
                left: '0px',
                width: '100%',
                height: '100%',
                '-webkit-overflow-scrolling': 'touch',
                overflow: 'auto',
                'min-width': '100%',
                'min-height': '100%',
                'background-color': 'rgba(255, 255, 255, 0.9)',
            },
            children: [
                nav,
                cheatsDiv
            ]
        });
        Html.appendToBody(container);
        this.container = container;

        // load bulma.io style
        if (!addedCss) {
            document.head.insertAdjacentHTML('beforeend', '<link rel=stylesheet href=https://cdnjs.cloudflare.com/ajax/libs/bulma/0.7.5/css/bulma.min.css>');
            document.head.insertAdjacentHTML('beforeend', '<link rel="stylesheet" type="text/css" href="https://use.fontawesome.com/releases/v5.8.1/css/all.css" integrity="sha384-50oBUHEmvpQ+1lW4y57PTFmhCaXp0ML5d60M1M7uH2+nqUivzIebhndOJK28anvf" crossorigin="anonymous">');
            document.head.insertAdjacentHTML('beforeend', '<link rel="stylesheet" type="text/css" href="https://cdn.jsdelivr.net/npm/bulma-switch@2.0.0/dist/css/bulma-switch.min.css" crossorigin="anonymous">');
            // document.head.insertAdjacentHTML('beforeend', '<script src="https://cdnjs.cloudflare.com/ajax/libs/jscolor/2.0.4/jscolor.js">');
            var jsColorScript = document.createElement('script');
            jsColorScript.src = "https://cdnjs.cloudflare.com/ajax/libs/jscolor/2.0.4/jscolor.js";
            document.head.append(jsColorScript);
            addedCss = true;
        }

        // Utils.forEach(modules, function (module) {
        //     container.attach(module.html.container);
        // });
        // todo: append all cheats

        Utils.forEach(CheatOverlay.prototype, function (fn, name) {
            var level;
            if (name === 'show' || name === 'hide') {
                return;
            }
            level = new Html('nav', {
                class: 'level'
            });
            cheatsDiv.attach(level);
            self[name](level);
        });
    };

    CheatOverlay.prototype.version = function (container) {
        container.attach(new Html('div', {
            class: 'level-item',
            innerHtml: 'v' + window.VERSION + '-' + window.BUILD + '.' + (window.PATCHVERSION || 0) + (Bento.isDev() ? 'DEV' : '')
        }))
    };

    /**
     * Shows debug overlay
     */
    CheatOverlay.prototype.closeCheats = function (container) {
        var self = this;
        var button = new Html('a', {
            class: 'level button is-primary',
            margin: '4px',
            innerHtml: 'Close',
            onclick: function () {
                // container.removeSelf();
                self.hide();
            }
        });
        container.attach(new Html('div', {
            class: 'level-item',
            children: [
                button
            ]
        }));
    };
    /**
     * Refresh self
     */
    CheatOverlay.prototype.refresh = function (container) {
        var self = this;
        var button = new Html('a', {
            class: 'level button is-primary',
            margin: '4px',
            innerHtml: 'Refresh (debug)',
            onclick: function () {
                self.hide();

                Bento.reload();
                window.setTimeout(function () {
                    // bento.require(['debug/cheat'], function (Cheat) {
                    //     var cheatOverlay = new Cheat({});
                    //     cheatOverlay.show();
                    // });
                    var loadCheats = function () {
                        bento.require(['debug/cheat'], function (Cheat) {
                            var cheatOverlay = new Cheat({});
                            cheatOverlay.show();
                        });
                    };
                    // bento.require(['https://lucky-kat.s3.amazonaws.com/ufo/cheat.js'], loadCheats, loadCheats);
                    loadCheats();
                }, 500);
            }
        });
        container.attach(new Html('div', {
            class: 'level-item',
            children: [
                button
            ]
        }));
    };

    /**
     * Shows debug overlay
     */
    CheatOverlay.prototype.showDebug = function (container) {
        var button = new Html('a', {
            class: 'level button is-primary',
            margin: '4px',
            innerHtml: 'Open plugin tests',
            onclick: function () {
                // container.removeSelf();
                bento.require(['debug/overlay'], function (DebugOverlay) {
                    new DebugOverlay({
                        showOverlay: true,
                        params: {
                            'debug/tests/tenjin': null,
                            'debug/tests/ironsrc': null,
                            // 'debug/tests/iap': {
                            //     testProduct: '${1:PRODUCT_ID}'
                            // },
                            'debug/tests/facebook': null,
                            'debug/tests/systemdata': null,
                            'debug/tests/gameanalytics': null,
                            'debug/tests/notifications': null,
                        }
                    });
                });
            }
        });
        container.attach(new Html('div', {
            class: 'level-item',
            children: [
                button
            ]
        }));
    };

    /**
     * Reset entire save file
     */
    CheatOverlay.prototype.resetSaveFile = function (container) {
        var self = this;
        var button = new Html('a', {
            class: 'level button is-primary',
            margin: '4px',
            innerHtml: 'Reset save file',
            onclick: function () {
                SaveValues.clear();
                // re-set isDev
                Bento.saveState.save('isDev', true);

                // reset screen
                self.shouldRefresh = true;
                self.hide();
            }
        });
        container.attach(new Html('div', {
            class: 'level-item',
            children: [
                button
            ]
        }));
    };

    /**
     * Move camera
     */
    CheatOverlay.prototype.moveCamera = function (container) {
        var self = this;
        var button = new Html('a', {
            class: 'level button is-primary',
            margin: '4px',
            innerHtml: 'Move camera',
            onclick: function () {
                self.hide();

                bento.require(['onigiri/orbitcontrols'], function (OrbitControls) {
                    var orbitControls = new OrbitControls({
                        target: new THREE.Vector3(0, 5, 0)
                    });
                    Bento.objects.pause();
                    Bento.objects.attach(orbitControls);
                    orbitControls.updateWhenPaused = Bento.objects.isPaused();

                    var buttonDone = Html('button', {
                        id: 'buttonDone',
                        class: 'button button-primary',
                        style: {
                            display: 'block',
                            position: 'absolute',
                            top: '64px',
                            left: '128px',
                        },
                        innerHtml: 'DONE',
                        onclick: function () {
                            buttonDone.removeSelf();
                            Bento.objects.remove(orbitControls);
                            Bento.objects.resume();
                        }
                    });
                    Html.appendToBody(buttonDone);
                });
            }
        });
        container.attach(new Html('div', {
            class: 'level-item',
            children: [
                button
            ]
        }));
    };
    
    /**
     * Test rating
     */
    CheatOverlay.prototype.testRating = function (container) {
        var self = this;
        var button = new Html('a', {
            class: 'level button is-primary',
            margin: '4px',
            innerHtml: 'Test Rating Popup',
            onclick: function () {
                // re-set isDev
                Bento.saveState.save('cheatTestRating', true);
                // reset screen
                self.shouldRefresh = true;
                self.hide();
            }
        });
        container.attach(new Html('div', {
            class: 'level-item',
            children: [
                button
            ]
        }));
    };
    CheatOverlay.prototype.unlockAll = function (container) {
        var self = this;
        var button = new Html('a', {
            class: 'level button is-primary',
            margin: '4px',
            innerHtml: 'Unlock everything',
            onclick: function () {
                Bento.saveState.save('cheat-shouldUnlockAll', true);
                self.shouldRefresh = true;
                self.hide();
            }
        });
        container.attach(new Html('div', {
            class: 'level-item',
            children: [
                button
            ]
        }));
    };
    /**
     * Schedule test notification
     */
    CheatOverlay.prototype.testNotification = function (container) {
        var button = new Html('a', {
            class: 'level button is-primary',
            margin: '4px',
            innerHtml: 'Test Notification',
            onclick: function () {
                EventSystem.fire('scheduleNotification', {
                    id: 'test',
                    title: Localization.getText('notifications-title'),
                    body: Localization.getText('notifications-body' + Utils.getRandom(3)),
                    time: 60 * 1000
                });
                window.alert("test notification scheduled after 60 seconds");
            }
        });
        container.attach(new Html('div', {
            class: 'level-item',
            children: [
                button
            ]
        }));
    };
    /**
     * Show bonus
     */
    CheatOverlay.prototype.bonusLevel = function (container) {
        var self = this;
        var button = new Html('a', {
            class: 'level button is-primary',
            margin: '4px',
            innerHtml: 'Trigger Bonus Level',
            onclick: function () {
                SaveValues.levelsUntilBonus = 0;
                self.shouldRefresh = true;
                self.hide();
            }
        });
        container.attach(new Html('div', {
            class: 'level-item',
            children: [
                button
            ]
        }));
    };

    /**
     * Daily Gift/Bonus
     */
    CheatOverlay.prototype.dailygift = function (container) {
        var button = new Html('a', {
            class: 'level button is-primary',
            margin: '4px',
            innerHtml: 'Get Daily gift',
            onclick: function () {
                SaveValues.dailyBonusKey = 0;
            }
        });
        container.attach(new Html('div', {
            class: 'level-item',
            children: [
                button
            ]
        }));
    };

    CheatOverlay.prototype.hideOverlay = function (container) {
        var self = this;
        var checkBoxContainer = createCheckbox('hideUi', 'Hide UI', function (checked) {
            EventSystem.fire('hideUi');
            self.shouldRefresh = true;
        });

        container.attach(new Html('div', {
            class: 'level-item',
            children: [
                checkBoxContainer
            ]
        }));
    };
    CheatOverlay.prototype.autoAim = function (container) {
        var checkBoxContainer = createCheckbox('noAutoAim', 'Disable Auto-aim', function (checked) {});
        container.attach(new Html('div', {
            class: 'level-item',
            children: [
                checkBoxContainer
            ]
        }));
    };
    CheatOverlay.prototype.failFast = function (container) {
        var checkBoxContainer = createCheckbox('failFast', 'Fail fast', function (checked) {});

        container.attach(new Html('div', {
            class: 'level-item',
            children: [
                checkBoxContainer
            ]
        }));
    };
    CheatOverlay.prototype.showUaHand = function (container) {
        var checkBoxContainer = createCheckbox('showUaHand', 'Show UA hand', function (checked) {});

        container.attach(new Html('div', {
            class: 'level-item',
            children: [
                checkBoxContainer
            ]
        }));
    };
    CheatOverlay.prototype.lowerCamera = function (container) {
        var self = this;
        var checkBoxContainer = createCheckbox('lowerCamera', 'Lower camera', function (checked) {
            self.shouldRefresh = true;
        });

        container.attach(new Html('div', {
            class: 'level-item',
            children: [
                checkBoxContainer
            ]
        }));
    };
    CheatOverlay.prototype.skipInterstitials = function (container) {
        var checkBoxContainer = createCheckbox('skipInterstitials', 'Skip interstitials', function (checked) {});

        container.attach(new Html('div', {
            class: 'level-item',
            children: [
                checkBoxContainer
            ]
        }));
    };
    CheatOverlay.prototype.skipTutorials = function (container) {
        var self = this;
        var checkBoxContainer = createCheckbox('skipTutorials', 'Skip tutorials', function (checked) {
            self.shouldRefresh = true;
        });

        container.attach(new Html('div', {
            class: 'level-item',
            children: [
                checkBoxContainer
            ]
        }));
    };
    CheatOverlay.prototype.bannerActive = function (container) {
        var self = this;
        var checkBoxContainer = createCheckbox('noBanner', 'Hide banner', function (checked) {
            self.shouldRefresh = true;
        });

        container.attach(new Html('div', {
            class: 'level-item',
            children: [
                checkBoxContainer
            ]
        }));
    };
    CheatOverlay.prototype.tapAndHold = function (container) {
        var checkBoxContainer = createCheckbox('tapStyle', 'Alternate stop method: tap', function (checked) {
            Globals.tapStyle = checked ? 'tap' : 'tapAndHold';
        });

        container.attach(new Html('div', {
            class: 'level-item',
            children: [
                checkBoxContainer
            ]
        }));
    };

    CheatOverlay.prototype.editLevel = function (container) {
        var pC = new Html('<p>Level: </p>');
        var self = this;
        var inputCoins = new Html('input', {
            class: 'input',
            value: SaveValues.currentLevel,
            onchange: function (evt) {
                SaveValues.currentLevel = parseInt(evt.target.value);
                self.shouldRefresh = true;
                self.totalLevel = true;
            }
        });
        container.attach(new Html('div', {
            class: 'level-item',
            children: [
                pC,
                inputCoins,
            ]
        }));
    };

    /**
     * Sets current amount of coins/gems
     */
    CheatOverlay.prototype.editCurrency = function (container) {
        var pC = new Html('<p>Coins:</p>');
        var inputCoins = new Html('input', {
            class: 'input',
            value: SaveValues.coins,
            onchange: function (evt) {
                SaveValues.coins = parseInt(evt.target.value) || 0;
            }
        });
        container.attach(new Html('div', {
            class: 'level-item',
            children: [
                pC,
                inputCoins,
            ]
        }));
    };

    CheatOverlay.prototype.backgrounds = function (container) {
        var p = new Html('<h2 class="subtitle  has-text-weight-bold">Background</h2>');
        var self = this;
        var div = new Html.make('div', "control");
        var makeRadioButton = function (radioLabel, key) {
            var label = new Html.make('label', 'radio', '', {});
            var input = new Html.make('input', '', '', {
                type: "radio",
                name: "Backgrounds",
                onchange: function () {
                    self.shouldRefresh = true;
                    Bento.saveState.save('cheat-background', key);
                }
            });
            var namespan = new Html.make('span', '', '', {
                innerHtml: radioLabel
            });
            var background = Bento.saveState.load('cheat-background', 'default');
            label.attach(input);
            label.attach(namespan);
            div.attach(label);

            if (key === undefined) {
                key = radioLabel;
            }

            if (background === key) {
                input.checked = true;
            }
        };
        container.attach(new Html('div', {
            children: [
                p,
                div,
            ]
        }));

        makeRadioButton('default');
        makeRadioButton('bg');
        makeRadioButton('bg-bright');
        makeRadioButton('bg-city');
        makeRadioButton('bg-hills');
        makeRadioButton('bg-gradient1');
        makeRadioButton('bg-gradient2');
        makeRadioButton('bg-scifi');
        makeRadioButton('bg-trees');
        makeRadioButton('bg-beach');
        makeRadioButton('bg-desert');
        makeRadioButton('bg-egypt');
        makeRadioButton('Gradient...');
    };

    CheatOverlay.prototype.gradients = function (container) {
        var self = this;
        var color1 = new Html('div', {
            innerHtml: 'Color 1: '
        });
        var input1 = new Html('input', {
            type: 'color',
            // class: 'jscolor',
            value: Bento.saveState.load('cheat-gradient1', '#da8bcf'),
            onchange: function (evt) {
                var value = evt.target.value;
                if (!value.startsWith('#')) {
                    value = '#' + value;
                }
                Bento.saveState.save('cheat-gradient1', value);
                self.shouldRefresh = true;
            }
        });
        var color2 = new Html('div', {
            innerHtml: 'Color 2: '
        });
        var input2 = new Html('input', {
            type: 'color',
            // class: 'jscolor',
            value: Bento.saveState.load('cheat-gradient2', '#8bcbf5'),
            onchange: function (evt) {
                var value = evt.target.value;
                if (!value.startsWith('#')) {
                    value = '#' + value;
                }
                Bento.saveState.save('cheat-gradient2', value);
                self.shouldRefresh = true;
            }
        });

        container.attach(new Html('div', {
            class: 'level-item',
            children: [
                color1.attach(input1),
                color2.attach(input2),
            ]
        }));
    };
    CheatOverlay.prototype.wallColor = function (container) {
        var p = new Html('<h2 class="subtitle has-text-weight-bold">Wall color</h2>');
        var self = this;
        var color1 = new Html('div', {
            innerHtml: 'Custom Wall Color: '
        });
        var input1 = new Html('input', {
            type: 'color',
            // class: 'jscolor',
            value: Bento.saveState.load('cheat-wallcolor', '#fde38f'),
            onchange: function (evt) {
                var value = evt.target.value;
                if (!value.startsWith('#')) {
                    value = '#' + value;
                }
                Bento.saveState.save('cheat-wallcolor', value);
                self.shouldRefresh = true;
            }
        });
        var div = new Html.make('div', "control");
        var makeRadioButton = function (radioLabel, key) {
            var label = new Html.make('label', 'radio', '', {});
            var input = new Html.make('input', '', '', {
                type: "radio",
                name: "WallColor",
                onchange: function () {
                    self.shouldRefresh = true;
                    Bento.saveState.save('cheat-setWallColor', key);
                }
            });
            var namespan = new Html.make('span', '', '', {
                innerHtml: radioLabel
            });
            var saveKey = Bento.saveState.load('cheat-setWallColor', 'default');
            label.attach(input);
            label.attach(namespan);
            div.attach(label);

            if (key === undefined) {
                key = radioLabel;
            }

            if (saveKey === key) {
                input.checked = true;
            }
        };
        makeRadioButton('default', 'default');
        makeRadioButton('Other...', 'other');

        container.attach(new Html('div', {
            children: [
                p,
                div,
                color1.attach(input1)
            ]
        }));
    };

    CheatOverlay.prototype.solidColorPlayers = function (container) {
        var p = new Html('<h2 class="subtitle  has-text-weight-bold">Player skin</h2>');
        var self = this;
        var div = new Html.make('div', "control");
        var makeRadioButton = function (radioLabel, key) {
            var label = new Html.make('label', 'radio', '', {});
            var input = new Html.make('input', '', '', {
                type: "radio",
                name: "Players",
                onchange: function () {
                    self.shouldRefresh = true;
                    Bento.saveState.save('cheat-playerskin', key);
                }
            });
            var namespan = new Html.make('span', '', '', {
                innerHtml: radioLabel
            });
            var background = Bento.saveState.load('cheat-playerskin', 'default');
            label.attach(input);
            label.attach(namespan);
            div.attach(label);

            if (key === undefined) {
                key = radioLabel;
            }

            if (background === key) {
                input.checked = true;
            }
        };
        container.attach(new Html('div', {
            children: [
                p,
                div,
            ]
        }));

        makeRadioButton('default');
        makeRadioButton('alternate');
        makeRadioButton('green/red');
    };

    // CheatOverlay.prototype.ballSkin = function (container) {
    //     var p = new Html('<h2 class="subtitle  has-text-weight-bold">Ball skin</h2>');
    //     var self = this;
    //     var div = new Html.make('div', "control");
    //     var makeRadioButton = function (radioLabel, key) {
    //         var label = new Html.make('label', 'radio', '', {});
    //         var input = new Html.make('input', '', '', {
    //             type: "radio",
    //             name: "Balls",
    //             onchange: function () {
    //                 self.shouldRefresh = true;
    //                 SaveValues.currentBall = key;
    //             }
    //         });
    //         var namespan = new Html.make('span', '', '', {
    //             innerHtml: radioLabel
    //         });
    //         var background = SaveValues.currentBall;
    //         label.attach(input);
    //         label.attach(namespan);
    //         div.attach(label);

    //         if (key === undefined) {
    //             key = radioLabel;
    //         }

    //         if (background === key) {
    //             input.checked = true;
    //         }
    //     };
    //     container.attach(new Html('div', {
    //         children: [
    //             p,
    //             div,
    //         ]
    //     }));

    //     // makeRadioButton('default');
    //     var assets = Bento.assets.getAssets();
    //         Utils.forEach(assets.meshes, function (item, key, l, breakLoop) {
    //             if (key.startsWith('balls/ball-')) {
    //                 makeRadioButton(key.replace('balls/ball-', '').replace('.gltf', ''));
    //             }
    //         });
    // };

    return CheatOverlay;
});