/**
 * IronSrc debug test
 * @moduleName IronSrcTest
 */
bento.define('debug/tests/ironsrc', [
    'bento',
    'bento/utils',
    'modules/ironsource',
    'debug/html/html'
], function (
    Bento,
    Utils,
    IronSource,
    Html
) {
    'use strict';
    var Test = function (settings) {
        return [{
            // simplest: global variable needs to exists
            name: 'Plugin exists',
            run: function (htmlContainer, onSuccess, onFail) {
                if (window.plugins && window.plugins.ironsource) {
                    onSuccess('PASS');
                } else {
                    onFail('FAIL: Missing global variable window.plugins.ironsource');
                }
            }
        }, {
            // can view a rewarded ad
            name: 'Can view a rewarded ad',
            run: function (htmlContainer, onSuccess, onFail) {
                // append a button so the user can wait until a rewarded video is available 
                var button = Html('<a class="level button is-primary">Loading video...<i class="fas fa-circle-notch fa-spin"></i></a>', {
                    onclick: function () {
                        if (hasRewarded) {
                            IronSource.showRewarded({
                                tag: 'debug',
                                muteAudio: true,
                                onComplete: function (e) {
                                    var msg;
                                    if (e) {
                                        try {
                                            msg = JSON.stringify(e);
                                        } catch (err) {
                                            console.log(err);
                                            msg = '';
                                        }
                                        onFail('FAIL: could not watch video ' + msg);
                                        button.removeSelf();
                                        return;

                                    }
                                    onSuccess('PASS: video watched');
                                    button.removeSelf();
                                },
                                onReward: function () {
                                    // window.alert('Reward callback');
                                    console.log('Reward callback');
                                },
                            });
                        } else {
                            window.alert('No video has been loaded (yet).');
                        }
                    }
                });
                var hasRewarded = false;
                var checkRewarded = function () {
                    // occasionally check for a rewarded video
                    if (IronSource.canShowRewarded()) {
                        hasRewarded = true;
                        button.innerHtml = 'Watch video';
                    } else {
                        window.setTimeout(checkRewarded, 1000);
                    }
                };
                htmlContainer.append(button);
                checkRewarded();
            }
        }, {
            // can view a rewarded ad
            name: 'Can view an interstitial ad',
            run: function (htmlContainer, onSuccess, onFail) {
                // append a button so the user can wait until a rewarded video is available 
                var button = Html('<a class="level button is-primary">Loading video...<i class="fas fa-circle-notch fa-spin"></i></a>', {
                    onclick: function () {
                        if (hasInterstitial) {
                            IronSource.showInterstitial({
                                tag: 'debug',
                                muteAudio: true,
                                onComplete: function (e) {
                                    if (e) {
                                        onFail('FAIL: could not watch video ' + e);
                                        button.removeSelf();
                                        return;
                                    }
                                    onSuccess('PASS: video watched');
                                    button.removeSelf();
                                }
                            });
                        } else {
                            window.alert('No video has been loaded (yet).');
                        }
                    }
                });
                var hasInterstitial = false;
                var checkInterstitial = function () {
                    // occasionally check for a rewarded video
                    if (IronSource.canShowInterstitial()) {
                        hasInterstitial = true;
                        button.innerHtml = 'Watch video';
                    } else {
                        window.setTimeout(checkInterstitial, 1000);
                    }
                };
                htmlContainer.append(button);
                checkInterstitial();
            }
        }];
    };
    return Test;
});