/**
 * Notifications debug test: Check if module is present
 * @moduleName Notifications
 */
bento.define('debug/tests/notifications', [
    'bento',
    'bento/utils',
    'debug/html/html',
    'bento/eventsystem',
    'modules/notifications',
    'modules/localization'
], function (
    Bento,
    Utils,
    Html,
    EventSystem,
    Notifications,
    Localization
) {
    'use strict';
    var Test = function (settings) {
        return [{
            name: 'Plugin exists',
            run: function (htmlContainer, onSuccess, onFail) {
                if (!window.Cocoon || !window.Cocoon.Notification || !window.Cocoon.Notification.Local) {
                    onFail('FAIL: missing global variable window.Cocoon.Notification');
                } else {
                    onSuccess('PASS');
                }
            }
        }, {
            // send a notification
            name: 'Send notification',
            run: function (htmlContainer, onSuccess, onFail) {
                var active = true;
                var button = Html('<a class="level button is-primary">Schedule Notification</a>', {
                    onclick: function () {
                        if (active) {
                            doNotification();
                        } else {
                            window.alert('Notification already scheduled...');
                        }
                    }
                });
                htmlContainer.append(button);

                var doNotification = function () {
                    var getNotification = function (prms) {
                        prms = prms || {};
                        var time = prms.time || 24 * 60 * 60 * 1000;
                        var notificationLocalization = prms.Localization;
                        if (!notificationLocalization) {
                            Utils.log('ERROR: pass Localization as parameter');
                            return {};
                        }

                        return {
                            title: notificationLocalization.getText('title'),
                            text: notificationLocalization.getText(prms.type || 'notificationInactive1'),
                            deltaTime: time
                        };
                    };

                    var scheduleNotification = function () {
                        // reset badge
                        Notifications.resetBadgeNumber();

                        // cancel old notification if needed
                        var removeOldNotification = function (notification) {
                            var oldNotification = Bento.saveState.load(notification, null);
                            if (oldNotification) {
                                Notifications.cancel(oldNotification);
                            }
                        };
                        removeOldNotification('notificationTest');

                        // schedule notifications
                        var notification;
                        notification = {
                            title: Localization.getText('title'),
                            text: 'This is a test!',
                            deltaTime: 1000 * 30
                        };
                        Notifications.send(notification, function (id) {
                            // save id so it can be cancelled if needed
                            Bento.saveState.save('notificationTest', id);
                        });

                        button.innerHtml = 'Notification scheduled...';
                        active = false;
                    };

                    if (!Bento.saveState.load('notificationsAllowed', false)) {
                        Notifications.init(function (initialized) {
                            if (initialized) {
                                Bento.saveState.save('notificationsAllowed', true);
                                EventSystem.fire('allowedNotifications');
                                scheduleNotification();
                            }
                        });
                    } else {
                        scheduleNotification();
                    }
                };
            }
        }];
    };
    return Test;
});