/**
 * Sets up 2D image behind the 3D scene
 * @moduleName BackgroundImage
 * @snippet BackgroundImage|Constructor
BackgroundImage({})
 */
bento.define('entities/backgroundimage', [
    'bento',
    'bento/utils',
    'bento/entity',
    'bento/components/spritecontainer',
    'bento/math/vector2',
    'bento/components/anchor',
    'onigiri/onigiri'
], function (
    Bento,
    Utils,
    Entity,
    SpriteContainer,
    Vector2,
    Anchor,
    Onigiri
) {
    'use strict';
    var BackgroundImage = function (settings) {
        var imageName = settings.imageName || 'bg';
        
        var scene, camera;
        var viewport = Bento.getViewport();
        var setupScene = function () {
            var width = viewport.width;
            var height = viewport.height;
            if (!scene) {
                scene = new THREE.Scene();
                scene.background = Onigiri.getTexture(imageName, {
                    flipY: true
                });
            }
            if (camera) {
                scene.remove(camera);
            }
            camera = new THREE.OrthographicCamera(
                0, // left
                width, // right
                height, // top
                0, // bottom
                -10000, // near
                10000 // far
            );
            // rotate camera in such a way that x axis is right and y axis is down
            camera.lookAt(new THREE.Vector3(0, 0, 1));
            camera.rotation.z = 0;
            camera.position.x = 0;
            camera.position.y = height;

        };
        var behavior = new Object({
            name: 'behavior',
            start: function (data) {
                setupScene();
                // spriteContainer.sprite.targetScene = scene;
                data.renderer.three.sceneList.unshift({
                    scene: scene,
                    camera: camera
                });
            },
            destroy: function (data) {
                data.renderer.three.sceneList.shift();
            },
            update: function (data) {},
            draw: function (data) {}
        });
        // var spriteContainer = new SpriteContainer({
        //     width: viewport.width + 8,
        //     imageName: imageName,
        //     originRelative: new Vector2(0.5, 0.5),
        //     alphaTest: 1
        // });

        var anchor = new Anchor({
            relative: new Vector2(0.5, 0.5),
            offset: new Vector2(0, 0),
            onResize: function (isPortrait) {}
        });
        var entity = new Entity({
            name: 'bgImage',
            components: [
                behavior,
                // spriteContainer,
                anchor,
            ]
        });
        return entity;
    };
    return BackgroundImage;
});