/**
 * Basket hoop target
 * @moduleName BasketHoop
 * @snippet BasketHoop|Constructor
BasketHoop({})
 */
bento.define('entities/baskethoop', [
    'bento',
    'bento/utils',
    'bento/math/vector2',
    'bento/math/rectangle',
    'onigiri/entity3d',
    'onigiri/primitive',
    'modules/box2d',
    'bento/eventsystem',
    'entities/confetti3d',
    'bento/tween',
    'onigiri/onigiri',
    'components/materialswapper',
    'components/textureswapper',
    'onigiri/animationmixer',
    'components/targetsize',
    'components/manualupdate',
    'modules/tapticengine',
    'bento/components/eventlistener',
    'onigiri/spriteplane'
], function (
    Bento,
    Utils,
    Vector2,
    Rectangle,
    Entity3D,
    Primitive,
    Box2D,
    EventSystem,
    Confetti3D,
    Tween,
    Onigiri,
    MaterialSwapper,
    TextureSwapper,
    AnimationMixer,
    TargetSize,
    ManualUpdate,
    TapticEngine,
    EventListener,
    SpritePlane
) {
    'use strict';
    var StarEffect = function (settings) {
        var sprite3d = new SpritePlane({
            width: settings.size || 5,
            imageName: 'fx/star-small',
            originRelative: new Vector2(0.5, 0.5),
        });
        var behavior = new Object({
            name: 'behavior',
            start: function (data) {
                new Tween({
                    from: entity.position.y,
                    to: entity.position.y + 5,
                    in: 40,
                    onUpdate: function (v, t) {
                        entity.position.y = v;
                    },
                    onComplete: function () {
                        // entity.removeSelf();
                    }
                });
                new Tween({
                    from: 0,
                    to: 1,
                    in: 40,
                    ease: 'elastic',
                    decay: 5,
                    oscillations: 3,
                    onUpdate: function (v, t) {
                        entity.scale.y = 1 + v;
                        entity.scale.x = 1 - v;
                    },
                    onComplete: function () {
                        // entity.removeSelf();
                    }
                });
                new Tween({
                    from: 1,
                    to: 0,
                    in: 40,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        sprite3d.sprite.material.opacity = v;
                    }
                });
            },
            destroy: function (data) {},
            update: function (data) {},
        });
        var entity = new Entity3D({
            name: 'starEffect',
            position: settings.position,
            components: [
                sprite3d,
                behavior
            ]
        });

        return entity;
    };
    var BasketHoop = function (settings) {
        var position = settings.position;
        var align = settings.align || 'center'; // 'left' or 'right'
        var ringScale = settings.ringScale || 1;
        var ballRadius = 1 * ringScale;
        var innerRadius = ballRadius + 2 / 6;
        var tubeRadius = 1 / 6;
        var plateWidth = 0.5;
        var plateHeight = 6;
        var ringMesh = new Primitive({
            shape: 'torus',
            rotation: new THREE.Euler(Math.PI / 2, 0, 0),
            castShadow: true,
            parameters: [
                innerRadius + tubeRadius, // radius - Radius of the torus, from the center of the torus to the center of the tube. Default is 1.
                tubeRadius, // tube - Radius of the tube. Default is 0.4.
                16, // radialSegments
                16 // tubularSegments
                //true, // optional: arc - Central angle. Default is Math.PI * 2.
            ],
            material: new THREE.MeshPhongMaterial({
                color: '#b95b01',
                // roughness: 0.5,
                // metalness: 0,
                specular: 0xffffff,
                shininess: 5,
                reflectivity: 0.5,
            }),
            disposeGeometry: true,
            disposeMaterial: true,
            components: [new ManualUpdate({})]
        });
        /*var backPlate = new Primitive({
            shape: 'cube',
            position: new THREE.Vector3(0, plateHeight / 3, 0),
            rotation: new THREE.Euler(0, 0, 0),
            scale: new THREE.Vector3(1, 1, 1),
            castShadow: true,
            receiveShadow: true,
            parameters: [
                plateWidth, // width
                plateHeight, // height
                plateHeight * 1.5, // depth
            ],
            material: new THREE.MeshStandardMaterial({
                color: '#ffffff',
                roughness: 0.5,
                metalness: 0,
            }),
            disposeGeometry: true,
            disposeMaterial: true
        });*/
        var plateMesh = Onigiri.getMesh('back-board.gltf');
        var backPlate = new Entity3D({
            name: 'backPlate',
            position: new THREE.Vector3(0, -plateHeight / 6, 0),
            object3D: plateMesh, // THREE.object3D reference
            components: [
                new TargetSize({
                    width: plateWidth,
                    height: plateHeight,
                    depth: plateHeight * 1.5,
                }),
                new MaterialSwapper({
                    id: 'backPlate', // unique id for refcounting
                    deepSwap: true,
                    type: 'MeshStandardMaterial',
                    materialSettings: {
                        map: Onigiri.getTexture('textures/backboard', {
                            flipY: false,
                            wrapS: THREE.RepeatWrapping,
                            wrapT: THREE.RepeatWrapping
                        }),
                        color: '#ffffff',
                        roughness: 0.5,
                        metalness: 0,
                    }
                }),
                new ManualUpdate({})
            ]
        });
        var didBallEnterFromTop = null;
        var didLose = false;
        var sensor = new Box2D.Body({
            type: 'static',
            fixture: {
                isSensor: true,
                shape: new Box2D.CircleShape(innerRadius - 1.5, new Vector2(0, 0)),
                friction: 0,
                restitution: 0,
                density: 1,
                onBeginContact: function (contact) {
                    if (contact && contact.entity && contact.entity.name === 'ball') {
                        if (didBallEnterFromTop === null && contact.entity.position.y > entity.position.y) {
                            didBallEnterFromTop = true;
                            animationMixer.stop('idle', 0);
                            animationMixer.stop('ball-in', 0);
                            animationMixer.play('idle');
                        }
                        // console.log('begin', contact);
                    }
                },
                onEndContact: function (contact) {
                    if (contact && contact.entity && contact.entity.name === 'ball' && !didLose) {
                        if (didBallEnterFromTop === true && contact.entity.position.y < entity.position.y) {
                            console.log('DUNKED IT');
                            EventSystem.fire('dunkedBall');

                            TapticEngine.impact({
                                type: "heavy"
                            });
                            // Bento.setGameSpeed(0.25);
                            // new Tween({
                            //     ignoreGameSpeed: true,
                            //     from: 0.25,
                            //     to: 1,
                            //     in: 60,
                            //     delay: 30,
                            //     onUpdate: function (v, t) {
                            //         Bento.setGameSpeed(v);
                            //     }
                            // });

                            contact.entity.getComponent('body', function (body) {
                                var velocity = body.getLinearVelocity();
                                velocity.x /= 2;
                                body.setLinearVelocity(velocity);
                            });

                            Bento.objects.attach(new Confetti3D({
                                position: entity.position.clone().sub(new THREE.Vector3(0, 2, 0)),
                                direction: new THREE.Vector3(0, 0.5, 0),
                                size: new Vector2(0.4, 0.8),
                                gravity: 0.02,
                                lifeTime: 180
                            }));

                            animationMixer.stop('idle', 0);
                            animationMixer.stop('ball-in', 0);
                            animationMixer.play('ball-in');

                            // get rid of sensor
                            entity.remove(sensor);

                            dunkEffect();
                        }
                        // console.log('end', contact);
                    }

                    // reset
                    didBallEnterFromTop = null;
                }

            }
        });
        var dunkEffect = function () {
            var ringEffectMesh = new Primitive({
                shape: 'torus',
                parameters: [
                    innerRadius + tubeRadius, // radius - Radius of the torus, from the center of the torus to the center of the tube. Default is 1.
                    tubeRadius, // tube - Radius of the tube. Default is 0.4.
                    16, // radialSegments
                    16 // tubularSegments
                    //true, // optional: arc - Central angle. Default is Math.PI * 2.
                ],
                material: new THREE.MeshBasicMaterial({
                    color: '#ffffff',
                    transparent: true
                }),
                components: []
            });
            ringMesh.attach(ringEffectMesh);

            new Tween({
                from: 1,
                to: 2,
                in: 30,
                ease: 'exponential',
                decay: 5,
                onUpdate: function (v, t) {
                    ringEffectMesh.scale.x = v;
                    ringEffectMesh.scale.y = v;
                    ringEffectMesh.scale.z = v;
                },
                onComplete: function () {
                    ringEffectMesh.removeSelf();
                }
            });
            new Tween({
                from: 1,
                to: 0,
                in: 30,
                onUpdate: function (v, t) {
                    ringEffectMesh.object3D.material.opacity = v;
                }
            });



            Bento.objects.attach(new StarEffect({
                position: entity.position.clone().add(new THREE.Vector3(0, 0, -1)),
            }));
            Utils.timeout(5, function () {
                Bento.objects.attach(new StarEffect({
                    size: 3,
                    position: entity.position.clone().add(new THREE.Vector3(0.75, -0.75, -2)),
                }));
            });
            Utils.timeout(15, function () {
                Bento.objects.attach(new StarEffect({
                    size: 2,
                    position: entity.position.clone().add(new THREE.Vector3(-0.4, -1, -3)),
                }));
            });
        };
        var bodyLeft = new Box2D.Body({
            type: 'static',
            fixture: {
                shape: new Box2D.CircleShape(tubeRadius, new Vector2(-innerRadius - tubeRadius, 0)),
                friction: 0.8,
                restitution: 0.15,
                density: 0.3,
            }
        });
        var bodyRight = new Box2D.Body({
            type: 'static',
            fixture: {
                shape: new Box2D.CircleShape(tubeRadius, new Vector2(innerRadius + tubeRadius, 0)),
                friction: 0.8,
                restitution: 0.15,
                density: 0.3,
            }
        });

        var bodyPlate = new Box2D.Body({
            type: 'static',
            fixture: {
                shape: new Box2D.BoxShape(
                    plateWidth / 2,
                    plateHeight / 2,
                    new Vector2(
                        (tubeRadius * 2 + innerRadius + plateWidth / 2) * (align === 'right' ? 1 : -1),
                        plateHeight / 3
                    )
                ),
                friction: 0.8,
                restitution: 0.15,
                density: 0.3,
            }
        });
        var behavior = new Object({
            name: 'behavior',
            start: function (data) {
                if (align === 'left') {
                    entity.position.x += tubeRadius * 2 + innerRadius + plateWidth;
                    backPlate.position.x -= tubeRadius * 2 + innerRadius + plateWidth / 2;
                } else if (align === 'right') {
                    entity.position.x -= tubeRadius * 2 + innerRadius + plateWidth;
                    backPlate.position.x += tubeRadius * 2 + innerRadius + plateWidth / 2;
                } else {

                }
            }
        });
        var netMesh = Onigiri.getMesh('net.gltf');
        var animationMixer = new AnimationMixer({
            object3D: netMesh,
            defaultAnimation: 'idle',
            defaultAnimationWeight: 1,
            loopAnimations: false
        });
        var net = new Entity3D({
            name: 'net',
            object3D: netMesh,
            castShadow: true,
            components: [
                animationMixer,
                new Object({
                    name: 'behavior',
                    start: function (data) {
                        var size = Onigiri.getMeshSize(netMesh);
                        netMesh.scale.x = (innerRadius + tubeRadius) * 2 / size.x;
                        netMesh.scale.y = (innerRadius + tubeRadius) * 2 / size.x;
                        netMesh.scale.z = (innerRadius + tubeRadius) * 2 / size.x;
                        animationMixer.stop('idle', 0);
                        animationMixer.stop('ball-in', 0);
                    }
                }),
                new MaterialSwapper({
                    id: 'net', // unique id for refcounting
                    object3D: netMesh,
                    deepSwap: true,
                    type: 'MeshBasicMaterial',
                    materialSettings: {
                        map: Onigiri.getTexture('textures/net'),
                        transparent: true,
                        skinning: true,

                        // specular: 0xffffff,
                        // shininess: 10,
                        // reflectivity: 0.5,
                        // metalness: 0,
                        // side: THREE.DoubleSide
                    }
                })
            ]
        });
        var pulseBehavior = new Object({
            name: 'behavior',
            shouldPulse: true,
            pulseTimer: 0,
            update: function (data) {
                pulseBehavior.pulseTimer += data.speed;
                if (pulseBehavior.shouldPulse && pulseBehavior.pulseTimer > 60) {
                    pulseBehavior.pulseTimer = 0;
                    pulseBehavior.pulseHoop();
                }
            },
            pulseHoop: function () {
                var ringEffect = new Primitive({
                    shape: 'torus',
                    rotation: new THREE.Euler(Math.PI / 2, 0, 0),
                    castShadow: true,
                    parameters: [
                        innerRadius + tubeRadius,
                        tubeRadius,
                        16,
                        16
                    ],
                    material: new THREE.MeshBasicMaterial({
                        color: '#b95b01',
                        transparent: true,
                        side: THREE.DoubleSide
                    }),
                    disposeGeometry: true,
                    disposeMaterial: true
                });
                var effect = new Entity3D({
                    name: 'pulseEffect',
                    position: entity.position.clone(),
                    // scale: new THREE.Vector3(0.75, 0.75, 0.75),
                    components: [
                        ringEffect
                    ]
                });
                Bento.objects.attach(effect);

                new Tween({
                    from: effect.position.y,
                    to: effect.position.y + 1,
                    in: 60,
                    ease: 'squareroot',
                    decay: 2,
                    oscillations: 1,
                    onUpdate: function (v, t) {
                        effect.position.y = v;
                    },
                    onComplete: function () {
                        effect.removeSelf();
                    }
                });
                new Tween({
                    from: 1,
                    to: 0,
                    in: 30,
                    onUpdate: function (v, t) {
                        console.log(ringEffect);
                        ringEffect.object3D.material.opacity = v;
                    }
                });
            }
        });
        // not the best solution: when the gameover starts, no more chance of winning
        var loseListener = new EventListener({
            name: 'loseListener',
            eventName: 'gameOver',
            ignorePause: true,
            onEvent: function (data) {
                didLose = true;
            }
        });
        var winListener = new EventListener({
            name: 'winListener',
            eventName: 'dunkedBall',
            ignorePause: true,
            onEvent: function (data) {
                // this event can only happen during bonus levels, where you cant lose
                // get rid of sensor
                entity.remove(sensor);
            }
        });
        var entity = new Entity3D({
            name: 'basketHoop',
            family: ['hoops', 'ballTargets'],
            position: position,
            components: [
                behavior,
                ringMesh,
                backPlate,
                bodyLeft,
                bodyRight,
                bodyPlate,
                sensor,
                net,
                loseListener,
                winListener
            ]
        });

        return entity;
    };
    return BasketHoop;
});