/**
 * spawn confetti in 3d
 * @moduleName Confetti3D
 */
bento.define('entities/confetti3d', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'onigiri/onigiri',
    'onigiri/entity3d'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Onigiri,
    Entity3D
) {
    'use strict';
    var materials = [
        new THREE.MeshBasicMaterial({
            color: 0xFF0000,
            side: THREE.DoubleSide
        }),
        new THREE.MeshBasicMaterial({
            color: 0xFFEF00,
            side: THREE.DoubleSide
        }),
        new THREE.MeshBasicMaterial({
            color: 0x008FFF,
            side: THREE.DoubleSide
        })
    ];
    return function (settings) {
        var position = settings.position;
        var direction = settings.direction;
        var size = settings.size || new Vector2(0.2, 0.4);
        var gravity = settings.gravity || 0.005;
        var lifeTime = settings.lifeTime || 180;
        var confettis = [];

        var behavior = {
            name: 'behavior',
            start: function (data) {
                var paperGeometry = new THREE.PlaneGeometry(size.x, size.y);
                var paper;
                var paperEntity;
                var rot = 0.12;
                var i = 20;
                while (i--) {
                    paper = new THREE.Mesh(paperGeometry.clone(), materials[Utils.getRandom(3)]);
                    paper.force = direction.clone(new THREE.Vector3());
                    paper.force.x += Utils.getRandomRangeFloat(-0.15, 0.15);
                    paper.force.y += Utils.getRandomRangeFloat(-0.05, 0.2);
                    paper.force.z += Utils.getRandomRangeFloat(-0.15, 0.15);
                    paper.torque = new THREE.Vector3(Utils.getRandomRangeFloat(-rot, rot), Utils.getRandomRangeFloat(-rot, rot), Utils.getRandomRangeFloat(-rot, rot));
                    confettis.push(paper);
                    
                    paperEntity = new Entity3D({
                        name: 'confetti-' + i,
                        object3D: paper,
                    });
                    entity.attach(paperEntity);
                }

            },
            destroy: function (data) {
                confettis.forEach(function (p) {
                    p.geometry.dispose();
                });
            },
            update: function (data) {
                var active = false;
                confettis.forEach(function (p) {
                    p.position.x += p.force.x * data.speed;
                    p.position.y += p.force.y * data.speed;
                    p.position.z += p.force.z * data.speed;

                    p.force.x -= p.force.x * 0.03 * data.speed;
                    p.force.y -= p.force.y * 0.01 * data.speed;
                    p.force.z -= p.force.z * 0.03 * data.speed;

                    p.force.y -= gravity * data.speed;
                    if (p.force.y < -0.1) {
                        p.force.y = -0.1;
                    }

                    p.rotation.x += p.torque.x * data.speed;
                    p.rotation.y += p.torque.y * data.speed;
                    p.rotation.z += p.torque.z * data.speed;

                    if (p.position.y > -0.1) {
                        active = true;
                    }

                    if (lifeTime && entity.timer > lifeTime) {
                        lifeTime = null;
                        behavior.end();
                    }
                });
            },
            end: function () {
                new Tween({
                    from: 1,
                    to: 0,
                    in: 30,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        Utils.forEach(confettis, function (p) {
                            p.scale.x = v;
                            p.scale.y = v;
                            p.scale.z = v;
                        });
                    },
                    onComplete: function () {
                        entity.removeSelf();
                    }
                });
            }
        };
        var entity = new Entity3D({
            name: 'confetti',
            position: position,
            components: [
                behavior
            ]
        });
        return entity;
    };
});