/**
 * Module description
 * @moduleName ConfettiBurst
 * @snippet ConfettiBurst|snippet
ConfettiBurst({
    z: 0,
    position: new Vector2(0, 0)
})
 */
bento.define('entities/effects/confettiburst', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'color'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    Color
) {
    'use strict';
    var ColoredPlane = function (settings) {
        var geometry = new THREE.PlaneGeometry(96, 128, 1, 1);
        var material = new THREE.MeshBasicMaterial(settings.material);
        var mesh = new THREE.Mesh(geometry, material);
        var object3D = new THREE.Object3D().add(mesh);
        var component = new Object({
            name: 'coloredPlane',
            start: function (data) {
                data.renderer.three.scene.add(object3D);
                // rotate inner mesh like the confetti image
                mesh.rotation.z = Math.PI / 6;
            },
            destroy: function (data) {
                data.renderer.three.scene.remove(object3D);
                // clean memory
                material.dispose();
                geometry.dispose();
            },
            draw: function (data) {
                data.renderer.render(object3D, material);
            }
        });
        return component;
    };

    return function (settings) {
        // --- PARAMETERS ---
        var position = settings.position || new Vector2(0, 0);
        var velocity = settings.velocity || new Vector2(0, 0);
        var friction = settings.friction || 1;
        var gravity = settings.gravity || 0.05;
        var life = settings.life || Utils.getRandomRangeFloat(50, 80);
        var masterScale = new Vector2(settings.particleScale || 1, settings.particleScale || 1);
        var isRemoving = false;
        var spinRate = settings.spinRate || Utils.getRandomRangeFloat(-0.1, 0.1);

        // --- ENTITY ---
        var confetti = new Entity({
            z: settings.z || 0,
            name: 'confetti',
            position: position.clone(),
            scale: masterScale.clone(),
            float: settings.float,
            updateWhenPaused: settings.updateWhenPaused,
            components: [
                /*new Sprite({
                    imageName: settings.imageName || 'confetti',
                    originRelative: new Vector2(0.5, 0.5),
                    frameCountX: 3,
                    animations: {
                        'default': {
                            speed: 0,
                            frames: [Utils.getRandom(3)]
                        }
                    }
                })*/
                new ColoredPlane({
                    material: {
                        color: new THREE.Color(Utils.pickRandomFrom([new Color(255, 59, 59), new Color(255, 224, 111), new Color(59, 212, 255)]).toInt())
                    }
                })
            ]
        }).attach({
            name: 'confettiBehaviour',
            speedX: 0,
            speedY: settings.speedY || 1,
            rotationSpeed: Utils.getRandom(5),
            start: function () {
                confetti.timer = Utils.getRandomFloat(1000);
            },
            update: function (data) {
                confetti.rotation += spinRate;
                if (!isRemoving) {
                    life -= data.speed;
                    if (life <= 0) {
                        isRemoving = true;
                        new Tween({
                            from: masterScale,
                            to: new Vector2(0, 0),
                            in: 15,
                            onUpdate: function (v, t) {
                                masterScale = v;
                            },
                            onComplete: function () {
                                confetti.removeSelf();
                            }
                        });
                    }
                }

                // move
                velocity.y += gravity * data.speed;
                confetti.position.addTo(velocity.scalarMultiply(data.speed));
                velocity.scalarMultiplyWith(1 - ((1 - friction) * data.speed));

                // flip
                confetti.scale.y = masterScale.y;
                confetti.scale.x = Math.sin(confetti.timer / (this.rotationSpeed + 5)) * masterScale.x;
            }
        });
        Bento.objects.attach(confetti);
        return confetti;
    };
});