/**
 * Enemy basketballer
 * @moduleName Enemy
 * @snippet Enemy()|Constructor
Enemy({})
 */
bento.define('entities/enemy', [
    'bento',
    'bento/utils',
    'onigiri/entity3d',
    'onigiri/onigiri',
    'components/materialswapper',
    'onigiri/primitive',
    'bento/math/vector2',
    'modules/box2d',
    'onigiri/animator',
    'globals',
    'bento/components/eventlistener'
], function (
    Bento,
    Utils,
    Entity3D,
    Onigiri,
    MaterialSwapper,
    Primitive,
    Vector2,
    Box2D,
    Animator,
    Globals,
    EventListener
) {
    'use strict';
    var Enemy = function (settings) {
        var position = settings.position;
        var direction = settings.direction || 1;
        var balls = Bento.objects.getByFamily('balls');
        var solidColor = Bento.saveState.load('cheat-playerskin', 'default') === 'green/red';
        var isReadyToJump = false;
        var jumped = false;
        var playerWon = false;
        var lastVelY = 0;
        var mode = 'jumpPeriodically'; // jumpPeriodically / jumpInVicinity
        var behavior = new Object({
            name: 'behavior',
            jumpTimer: 0,
            start: function (data) {
                if (direction === -1) {
                    entity.euler.y += Math.PI;
                }
            },
            isGrounded: function () {
                // using a raycast from just above the feet to below the feet
                var feetPosition = Box2D.toB2Vec2(entity.position).Copy();
                var feetLower = feetPosition.Copy();
                var isGrounded = false;

                feetPosition.y += 0.1;
                feetLower.y -= 0.1;

                Box2D.box2DWorld.RayCast(function () {
                    isGrounded = true;
                }, feetPosition, feetLower);
                return isGrounded && !isReadyToJump;
            },
            update: function (data) {
                var isGrounded = behavior.isGrounded();
                // check vicinity of ball
                if (mode === 'jumpInVicinity') {
                    Utils.forEach(balls, function (ball) {
                        if (
                            // ball.isFollowingPath &&
                            isGrounded &&
                            ball.position.distanceToSquared(entity.position) < 10 * 10 &&
                            Math.abs(body.getLinearVelocity().y) < 1
                        ) {
                            entity.jump();
                        }
                    });
                } else if (mode === 'jumpPeriodically') {
                    if (isGrounded) {
                        behavior.jumpTimer += 1;
                        if (behavior.jumpTimer > 60) {
                            behavior.jumpTimer = 0;
                            entity.jump();
                            isGrounded = false;
                        }
                    }
                }

                if (jumped && isGrounded && lastVelY < 0) {
                    animator.stop();
                    animator.play('guardjumpland-' + (direction === 1 ? 'faceright' : 'faceleft'), {
                        loop: false
                    }, false);
                    jumped = false;
                    isGrounded = true;
                }

                // keep track of y velocity
                lastVelY = body.getLinearVelocity().y;
            }
        });
        var mesh = Onigiri.getMesh('players/baller-default.gltf');
        var animator = new Animator({
            object3D: mesh,
            defaultAnimation: 'guard-' + (direction === 1 ? 'faceright' : 'faceleft'),
            onFinish: function (evt) {
                var name = evt.action.getClip().name;
                if (name.startsWith('guardjumpland')) {
                    animator.stop();
                    animator.play('guard-' + (direction === 1 ? 'faceright' : 'faceleft'), {
                        loop: true
                    }, false);

                }
            }
        });
        var meshContainer = new Entity3D({
            name: 'mesh',
            castShadow: true,
            object3D: mesh,
            components: [
                new MaterialSwapper({
                    id: 'enemy', // unique id for refcounting
                    deepSwap: true,
                    type: 'MeshStandardMaterial',
                    materialSettings: {
                        color: solidColor ? 0xff2200 : 0xffffff,
                        map: solidColor ? undefined : Onigiri.getTexture(Globals.getEnemyTexture(), {}),
                        skinning: true,
                        metalness: 0
                    }
                }),
                animator
            ]
        });

        var width = 1.5;
        var height = 6;
        var body = new Box2D.Body({
            type: 'dynamic',
            fixedRotation: true,
            fixture: {
                // BoxShape is halfW, halfH
                shape: new Box2D.BoxShape(width / 2, height / 2, new Vector2(0, height / 2)),
                friction: 1,
                restitution: 0,
                density: 10000,
            }
        });
        var winListener = new EventListener({
            name: 'winListener',
            eventName: 'dunkedBall',
            ignorePause: false,
            onEvent: function (data) {
                playerWon = true;
            }
        });
        var entity = new Entity3D({
            name: 'enemy',
            position: position,
            euler: new THREE.Euler(0, Math.PI / 2, 0),
            scale: new THREE.Vector3(2.4, 2.4, 2.4),
            family: ['enemies'],
            components: [
                behavior,
                meshContainer,
                body,
                winListener
            ]
        }).extend({
            jump: function (velY) {
                if (playerWon) {
                    return;
                }

                velY = velY || 32;
                // console.log('jump');
                isReadyToJump = true;
                Utils.timeout(10, function () {
                    body.setAwake(true);
                    body.setLinearVelocity(new Vector2(0, velY));
                    isReadyToJump = false;
                    jumped = true;
                });

                animator.stop();
                animator.play('guardjumpup-' + (direction === 1 ? 'faceright' : 'faceleft'), {
                    loop: false
                }, false);
            }
        });


        // var debugMesh = new Primitive({
        //     shape: 'cube',
        //     position: new THREE.Vector3(0, height / 2, 0),
        //     parameters: [
        //         width, // width
        //         height, // height
        //         3, // depth
        //         //1, // optional: widthSegments
        //         //1, // optional: heightSegments
        //         //1, // optional: depthSegments
        //     ],
        //     // material: undefined,
        //     disposeGeometry: true,
        //     disposeMaterial: true
        // });
        // entity.attach(debugMesh);


        return entity;
    };
    return Enemy;
});