/**
 * Floor box
 * @moduleName Floor
 * @snippet Floor|Constructor
Floor({})
 */
bento.define('entities/floor', [
    'bento',
    'bento/utils',
    'onigiri/primitive',
    'modules/box2d',
    'bento/math/vector2',
    'globals',
    'onigiri/entity3d',
    'onigiri/onigiri',
    'components/materialswapper',
    'components/targetsize'
], function (
    Bento,
    Utils,
    Primitive,
    Box2D,
    Vector2,
    Globals,
    Entity3D,
    Onigiri,
    MaterialSwapper,
    TargetSize
) {
    'use strict';
    var Floor = function (settings) {
        // var depth = settings.depth;
        // big floor for Box2D
        var floorWidth = 1000000;
        var floorHeight = 10;
        // var floorMesh = new Primitive({
        //     shape: 'plane',
        //     position: new THREE.Vector3(0, floorHeight / 2, 0),
        //     euler: new THREE.Euler(-Math.PI * 0.5, 0, 0),
        //     material: new THREE.MeshPhongMaterial({
        //         color: 0xe0f9ff,
        //         shininess: 100,
        //         side: THREE.FrontSide,
        //         blending: THREE.NormalBlending
        //     }),
        //     parameters: [
        //         10000,
        //         depth
        //     ],
        //     receiveShadow: true,
        // });
        var addFloorMesh = function (offset, receiveShadow, scaleY) {
            var courtMesh = new Onigiri.getMesh('ground.gltf');
            var floorMesh = new Entity3D({
                name: 'floor',
                object3D: courtMesh, // THREE.object3D reference
                receiveShadow: receiveShadow,
                components: [
                    new TargetSize({
                        width: 40,
                        height: null,
                        depth: 30,
                        alignOffset: new THREE.Vector3(0, -0.5, 0)
                    }),
                    new MaterialSwapper({
                        id: 'floor', // unique id for refcounting
                        type: 'MeshPhongMaterial',
                        deepSwap: true,
                        materialSettings: {
                            map: Onigiri.getTexture('textures/court', {
                                flipY: false,
                                wrapS: THREE.RepeatWrapping,
                                wrapT: THREE.RepeatWrapping
                            }),
                            // color: 0x9d6468,
                            shininess: 10,
                            side: THREE.FrontSide,
                            blending: THREE.NormalBlending
                        }
                    }),
                    new Object({
                        name: 'behavior',
                        start: function (data) {
                            floorMesh.position.y += floorHeight / 1 + 0.5 + (offset || 0);
                            floorMesh.scale.y *= scaleY || 1;
                        },
                    })
                ]
            });
            return floorMesh;
        };
        var floorBody = new Box2D.Body({
            type: 'static',
            fixture: {
                shape: new Box2D.BoxShape(floorWidth, floorHeight, new Vector2(floorWidth / 2, -floorHeight / 2)),
                friction: Globals.wallFriction,
                restitution: Globals.wallRestitution,
                density: 1
            }
        });
        var floor = new Entity3D({
            name: 'floor',
            position: new THREE.Vector3(0, -floorHeight / 2, 0),
            components: [
                addFloorMesh(0, true),
                floorBody,
                addFloorMesh(-16, false, -2),
                /*new Primitive({
                    shape: 'cube',
                    parameters: [
                        floorWidth, // width
                        floorHeight, // height
                        levelSpawner.depth, // depth
                    ]
                })*/
            ]
        });
        return floor;
    };
    return Floor;
});