/**
 * Key
 * @moduleName Key
 * @snippet Key|Constructor
Key({})
 */
bento.define('entities/key', [
    'bento',
    'bento/utils',
    'onigiri/entity3d',
    'onigiri/onigiri',
    'components/materialswapper',
    'globals',
    'modules/box2d',
    'bento/math/vector2',
    'bento/tween',
    'onigiri/spriteplane',
    'managers/savevalues',
    'modules/tapticengine',
    'components/targetsize',
    'components/pathfollower'
], function (
    Bento,
    Utils,
    Entity3D,
    Onigiri,
    MaterialSwapper,
    Globals,
    Box2D,
    Vector2,
    Tween,
    SpritePlane,
    SaveValues,
    TapticEngine,
    TargetSize,
    PathFollower
) {
    'use strict';
    var Coin = function (settings) {
        var mesh = Onigiri.getMesh('key.gltf');
        var entity = new Entity3D({
            name: 'coin',
            position: settings.position,
            object3D: mesh,
            components: [
                new TargetSize({
                    width: 3,
                }),
                new MaterialSwapper({
                    id: 'key', // unique id for refcounting
                    object3D: mesh, // the target object, otherwise the parent's object3d
                    type: 'MeshPhongMaterial',
                    deepSwap: true,
                    materialSettings: {
                        color: 0xf4d73b,
                        specular: 0xad5f29,
                        shininess: 50,
                        reflectivity: 0.1,
                    }
                }),
                new Box2D.Body({
                    type: 'static',
                    ignoreRotation: true,
                    fixture: {
                        isSensor: true,
                        shape: new Box2D.CircleShape(1, new Vector2(0, 0)),
                        friction: 0,
                        restitution: 0,
                        density: 1,
                        onBeginContact: function (contact) {
                            if (contact && contact.entity && contact.entity.name === 'ball') {
                                new Tween({
                                    from: entity.scale.x,
                                    to: 0.2,
                                    in: 10,
                                    ease: 'easeInBack',
                                    onUpdate: function (v, t) {
                                        entity.scale.x = v;
                                        entity.scale.y = v;
                                        entity.scale.z = v;
                                    },
                                    onComplete: function () {
                                        entity.removeSelf();
                                        // do not collect until end of screen (gamecontroller.js)
                                        Globals.didGetKey = true;
                                        TapticEngine.impact({
                                            type: "medium"
                                        });
                                    }
                                });

                                var haloSprite = new SpritePlane({
                                    width: 2,
                                    imageName: 'fx/bloom-halo',
                                    originRelative: new Vector2(0.5, 0.5)
                                });
                                var bloomEffect = new Entity3D({
                                    name: 'bloomEffect',
                                    position: entity.position.clone().add(new THREE.Vector3(0, 0, 0.1)),
                                    components: [
                                        haloSprite
                                    ]
                                });
                                var haloSprite2 = new SpritePlane({
                                    width: 8,
                                    imageName: 'fx/shine',
                                    originRelative: new Vector2(0.5, 0.5)
                                });
                                var haloSprite3 = new SpritePlane({
                                    width: 3,
                                    imageName: 'fx/bloom-soft',
                                    originRelative: new Vector2(0.5, 0.5)
                                });
                                var bloomHardEffect = new Entity3D({
                                    name: 'bloomEffect',
                                    position: entity.position.clone().add(new THREE.Vector3(0, 0, 0.1)),
                                    components: [
                                        haloSprite2,
                                        haloSprite3
                                    ]
                                });

                                new Tween({
                                    from: 1,
                                    to: 0,
                                    in: 10,
                                    onStart: function () {
                                        Bento.objects.attach(bloomHardEffect);
                                    },
                                    onUpdate: function (v, t) {
                                        haloSprite2.sprite.material.opacity = v;
                                        haloSprite3.sprite.material.opacity = v;
                                    },
                                    onComplete: function () {
                                        Bento.objects.remove(bloomHardEffect);
                                    }
                                });
                                // new Tween({
                                //     from: 0,
                                //     to: 4,
                                //     in: 5,
                                //     ease: 'linear',
                                //     onStart: function () {
                                //         Bento.objects.attach(bloomHardEffect);
                                //     },
                                //     onUpdate: function (v, t) {
                                //         bloomHardEffect.scale.x = v;
                                //         bloomHardEffect.scale.y = v;
                                //     }
                                // });
                                // new Tween({
                                //     from: 4,
                                //     to: 0.2,
                                //     in: 20,
                                //     delay: 5,
                                //     ease: 'easeOutBack',
                                //     onUpdate: function (v, t) {
                                //         bloomHardEffect.scale.x = v;
                                //         bloomHardEffect.scale.y = v;
                                //     },
                                //     onComplete: function () {
                                //         Bento.objects.remove(bloomHardEffect);
                                //     }
                                // });

                                new Tween({
                                    from: 1,
                                    to: 2.5,
                                    in: 20,
                                    delay: 10,
                                    ease: 'exponential',
                                    decay: 5,
                                    oscillations: 1,
                                    onStart: function () {
                                        Bento.objects.attach(bloomEffect);
                                        // spawn sparkles
                                        addSparkle({
                                            position: new THREE.Vector3(0, 0.5, 0.1),
                                            scale: 1,
                                            delay: 0,
                                            offset: 0.1,
                                        });
                                        addSparkle({
                                            position: new THREE.Vector3(0.8, -0.3, 0.1),
                                            scale: 0.75,
                                            delay: 5,
                                            offset: 0.1,
                                        });
                                        addSparkle({
                                            position: new THREE.Vector3(-0.75, -0.6, 0.1),
                                            scale: 0.5,
                                            delay: 10,
                                            offset: 0.1,
                                        });
                                    },
                                    onUpdate: function (v, t) {
                                        bloomEffect.scale.x = v;
                                        bloomEffect.scale.y = v;
                                        bloomEffect.scale.z = v;
                                    },
                                    onComplete: function () {
                                        bloomEffect.removeSelf();
                                    }
                                });

                                new Tween({
                                    from: 1,
                                    delay: 10,
                                    to: 0,
                                    in: 20,
                                    onUpdate: function (v, t) {
                                        haloSprite.sprite.material.opacity = v;
                                    },
                                    onComplete: function () {}
                                });
                            }
                        }
                    }
                }),
                new Object({
                    name: 'behavior',
                    start: function (data) {
                        if (SaveValues.keys >= 3 && !Utils.getHashParameter('level', '')) {
                            // safeguard: cannot get more than 3 keys
                            entity.removeSelf();
                        }

                        mesh.rotation.z = -Math.PI * 0.4;
                        
                        if(settings.properties && settings.properties.follow) {
                            var follow = new PathFollower({
                                pathName: settings.properties.follow,
                            });
                            entity.attach(follow);
                        }
                    },
                    update: function (data) {
                        entity.euler.y += 0.05 * data.speed;
                    },
                    destroy: function (data) {}
                }),
            ]
        });
        var addSparkle = function (data) {
            var sparkleSprite = new SpritePlane({
                width: 2,
                imageName: 'fx/sparkle',
                originRelative: new Vector2(0.5, 0.5)
            });
            var sparkle = new Entity3D({
                name: 'sparkle',
                position: entity.position.clone().add(data.position),
                components: [
                    sparkleSprite
                ]
            });

            new Tween({
                from: 0,
                to: 1,
                in: 10,
                ease: 'linear',
                delay: data.delay,
                onStart: function () {
                    Bento.objects.attach(sparkle);

                    new Tween({
                        from: 0,
                        to: data.scale,
                        in: 20,
                        ease: 'easeOutExpo',
                        onUpdate: function (v, t) {
                            sparkle.scale.x = v;
                            sparkle.scale.y = v;
                        },
                        onComplete: function () {
                            new Tween({
                                from: data.scale,
                                to: 0,
                                in: 50,
                                ease: 'linear',
                                onUpdate: function (v, t) {
                                    sparkle.scale.x = v;
                                    sparkle.scale.y = v;
                                }
                            });
                        }
                    });
                    // new Tween({
                    //     from: sparkle.position.y,
                    //     to: sparkle.position.y + data.offset || 0,
                    //     in: 50,
                    //     ease: 'linear',
                    //     onUpdate: function (v, t) {
                    //         sparkle.position.y = v;
                    //     }
                    // });
                },
                onUpdate: function (v, t) {
                    sparkleSprite.sprite.material.opacity = v;
                },
                onComplete: function () {
                    new Tween({
                        from: 1,
                        to: 0,
                        in: 50,
                        ease: 'linear',
                        onStart: function () {},
                        onUpdate: function (v, t) {
                            sparkleSprite.sprite.material.opacity = v;
                        },
                        onComplete: function () {
                            sparkle.removeSelf();
                        }
                    });
                }
            });


            return sparkle;
        };
        return entity;
    };
    return Coin;
});