/**
 * Polygon wall for Box2d
 * @moduleName PolyWall
 */
bento.define('entities/polywall', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'modules/box2d',
    'globals',
    'onigiri/onigiri',
    'onigiri/entity3d'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Box2D,
    Globals,
    Onigiri,
    Entity3D
) {
    'use strict';
    return function (settings) {
        var position = settings.position.clone() || new Vector2(0, 0);
        var path = settings.path || [new Vector2(0, 0)];
        var levelScale = settings.levelScale || 1;
        var flipOffset = settings.flipOffset || 0;
        var addOnigiriShape = function (vertices) {
            // this is for debugging
            var shape = new THREE.Shape();
            Utils.forEach(vertices, function (vertex, i, l, breakLoop) {
                if (i === 0) {
                    shape.moveTo(vertex.x, vertex.y);
                }
                shape.lineTo(vertex.x, vertex.y);

            });
            var extrudeSettings = {
                steps: 2,
                depth: 2 * levelScale,
                bevelEnabled: false,
                bevelThickness: 64 * Onigiri.pxToThree,
                bevelSize: 0 * Onigiri.pxToThree,
                bevelOffset: 0,
                bevelSegments: 1
            };
            var geometry = new THREE.ExtrudeGeometry(shape, extrudeSettings);
            var material = new THREE.MeshBasicMaterial({
                color: 0xFF00FF,
            });
            var mesh = new THREE.Mesh(geometry, material);

            Bento.objects.attach(new Entity3D({
                name: 'debugMesh',
                position: new THREE.Vector3(position.x * Onigiri.pxToThree * levelScale, -position.y * Onigiri.pxToThree * levelScale, 0),
                object3D: mesh,
            }));
        };
        var fixConcavePath = function (p) {
            var decomp = window.decomp;

            var concavePoly = p.map(function (v) {
                return [
                    v.x * Onigiri.pxToThree * levelScale,
                    v.y * -1 * Onigiri.pxToThree * levelScale + flipOffset
                ];
            });

            decomp.makeCCW(concavePoly);
            var convexPolys = decomp.quickDecomp(concavePoly);

            convexPolys.forEach(function (poly) {
                decomp.removeCollinearPoints(poly);
            });

            var fixtures = convexPolys.map(function (poly) {
                var vertices = poly.map(function (v) {
                    return new Vector2(v[0], v[1]);
                });

                // debugging
                if (false) {
                    addOnigiriShape(vertices);
                }

                return {
                    shape: new Box2D.PolygonShape(vertices),
                    // filter,
                    // friction: friction,
                    friction: Globals.wallFriction,
                    restitution: Globals.wallRestitution,
                    // density,
                    // onBeginContact,
                    // etc.
                };
            });
            return fixtures;
            // now you can pass fixtures to new Box2D.Body
        };


        // var box2dPath = fixConcavePath(copyPath(path));
        var body = new Box2D.Body({
            type: 'static',
            fixture: fixConcavePath(path)
        });
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {},
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        };
        var entity = new Entity({
            z: 2,
            name: 'polyWall',
            position: new Vector2(position.x * Onigiri.pxToThree * levelScale, position.y * Onigiri.pxToThree * levelScale),
            components: [
                behavior,
                body
            ]
        });
        return entity;
    };
});