/**
 * Axis aligned Box2D wall
 * @moduleName Wall
 */
bento.define('entities/wall', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'modules/box2d',
    'globals',
    'onigiri/onigiri'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Box2D,
    Globals,
    Onigiri
) {
    'use strict';
    return function (settings) {
        var position = settings.position.clone() || new Vector2(0, 0);
        var levelScale = settings.levelScale || 1;
        var dimension = settings.dimension.clone();
        var body = new Box2D.Body({
            type: 'static',
            fixture: {
                shape: new Box2D.BoxShape(
                    dimension.width / 2 * Onigiri.pxToThree * levelScale,
                    dimension.height / 2 * Onigiri.pxToThree * levelScale,
                    new Vector2(
                        (dimension.width / 2 + dimension.x) * Onigiri.pxToThree * levelScale,
                        (dimension.height / 2 + dimension.y) * Onigiri.pxToThree * -levelScale
                    )
                ),
                friction: Globals.wallFriction,
                restitution: Globals.wallRestitution,
                density: 0.3,
            }
        });
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {},
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        };
        var entity = new Entity({
            z: 2,
            name: 'wall',
            position: new Vector2(
                position.x * Onigiri.pxToThree * levelScale,
                position.y * Onigiri.pxToThree * levelScale
            ),
            components: [
                behavior,
                // fill,
                body
            ]
        });
        return entity;
    };
});