/**
 * No Ads IAP button
 * @moduleName NoAdsButton
 * @snippet NoAdsButton|Constructor
NoAdsButton({})
 */
bento.define('gui/buttons/noadsbutton', [
    'bento',
    'bento/utils',
    'gui/iconbutton',
    'gui/hdbuttonsprite',
    'bento/tween',
    'bento/eventsystem',
    'bento/math/vector2',
    'bento/components/spritecontainer',
    'bento/entity',
    'bento/components/eventlistener',
    'managers/savevalues',
    'gui/popups/skinselectpopup',
    'gui/iapoverlay',
    'globals',
    'modules/banner',
    'bento/components/anchor'
], function (
    Bento,
    Utils,
    IconButton,
    HdButtonSprite,
    Tween,
    EventSystem,
    Vector2,
    SpriteContainer,
    Entity,
    EventListener,
    SaveValues,
    SkinSelectPopup,
    IapOverlay,
    Globals,
    BannerAd,
    Anchor
) {
    'use strict';
    var NoAdsButton = function (settings) {
        /*var iconBackSprite = new SpriteContainer({
            width: 50,
            imageName: 'gui/icons/icon-norv-back',
            position: new Vector2(32, -2),
            originRelative: new Vector2(0.5, 0.5),
        });
        var iconAds = new SpriteContainer({
            width: 24,
            imageName: 'gui/icons/ad',
            position: new Vector2(32, -2),
            originRelative: new Vector2(0.5, 0.5),
        });
        var iconFrontSprite = new SpriteContainer({
            width: 40,
            imageName: 'gui/icons/icon-norv-front',
            position: new Vector2(32, -2),
            originRelative: new Vector2(0.5, 0.5),
        });*/

        var iconAds = new SpriteContainer({
            width: 40,
            imageName: 'gui/icons/icon-norv',
            position: new Vector2(32, -2),
            originRelative: new Vector2(0.5, 0.5),
        });
        var icon = new Entity({
            name: 'buttonIcon',
            components: [
                // iconBackSprite,
                // iconFrontSprite,
                // iconAds,
                iconAds,
            ]
        });
        var hideListener = new EventListener({
            name: 'eventListener',
            eventName: 'pathDrawingStarted',
            ignorePause: false,
            onEvent: function (data) {
                entity.remove(hideListener);
                entity.hide();
            }
        });
        var behavior = new Object({
            name: 'behavior',
            start: function (data) {
                // don't show at lv 1
                if (SaveValues.currentLevel === 1) {
                    entity.removeSelf();
                }
                // remove self if purchased
                if (Bento.saveState.load('noAds', false)) {
                    entity.removeSelf();
                }
            }
        });
        var entity = new IconButton({
            sprite: HdButtonSprite({
                imageName: 'gui/buttons/blue',
                height: 64
            }),
            // icon options
            icon: icon,
            // offset of icon
            offset: new Vector2(0, 0),
            offsetWhenPressed: 2, // push pixels down when onButtonDown
            // icon transparency when inactive
            iconAlpha: 0,
            onButtonDown: function () {
                EventSystem.fire('cancelThrow');
            },
            onClick: function () {
                var viewport = Bento.getViewport();
                IapOverlay.purchase({
                    z: Globals.Layers.IAP,
                    id: 'doodledunk_noads',
                    isConsumable: false,
                    resolveAutomatically: true,
                    fontSettings: {
                        font: 'omnes',
                        fontSize: 24
                    },
                    onPurchaseComplete: function (success, id, purchaseInfo) {
                        if (success && id === 'doodledunk_noads') {
                            // remove self
                            entity.removeSelf();
                        }
                    }
                })
                // attach working fill
                .attach(new Entity({
                    name: 'fill',
                    components: [
                        new Object({
                            name: 'behavior',
                            start: function (data) {
                                // move self to top
                                data.entity.parent.moveComponentTo(data.entity, 0);
                            }
                        }),
                        new SpriteContainer({
                            width: viewport.width + 16,
                            alpha: 0.7,
                            imageName: 'gui/bg-dark',
                            originRelative: new Vector2(0.5, 0.5),
                        }),
                        new Anchor({
                            relative: new Vector2(0.5, 0.5),
                        })
                    ]
                }));
            },
            components: [
                behavior,
                hideListener
            ].concat(settings.components || [])
        }).extend({
            hide: function () {
                new Tween({
                    from: entity.position.x,
                    to: entity.position.x - 128,
                    in: 30,
                    ease: 'easeInBack',
                    onUpdate: function (v, t) {
                        entity.position.x = v;
                    },
                    onComplete: function () {
                        entity.active = false;
                    }
                });
            }
        });
        return entity;
    };
    return NoAdsButton;
});