/**
 * Skins shop button
 * @moduleName SkinsShopButton
 * @snippet SkinsShopButton|Constructor
SkinsShopButton({})
 */
bento.define('gui/buttons/skinshopbutton', [
    'bento',
    'bento/utils',
    'gui/iconbutton',
    'gui/hdbuttonsprite',
    'bento/tween',
    'bento/eventsystem',
    'bento/math/vector2',
    'bento/components/spritecontainer',
    'bento/entity',
    'bento/components/eventlistener',
    'managers/savevalues',
    'gui/popups/skinselectpopup'
], function (
    Bento,
    Utils,
    IconButton,
    HdButtonSprite,
    Tween,
    EventSystem,
    Vector2,
    SpriteContainer,
    Entity,
    EventListener,
    SaveValues,
    SkinSelectPopup
) {
    'use strict';
    var SkinsShopButton = function (settings) {
        var iconSprite = new SpriteContainer({
            width: 40,
            imageName: 'gui/icons/icon-skins',
            position: new Vector2(-32, -2),
            originRelative: new Vector2(0.5, 0.5),
        });
        var icon = new Entity({
            name: 'buttonIcon',
            components: [
                iconSprite
            ]
        });
        var hideListener = new EventListener({
            name: 'eventListener',
            eventName: 'pathDrawingStarted',
            ignorePause: false,
            onEvent: function (data) {
                entity.remove(hideListener);
                entity.hide();
            }
        });
        var behavior = new Object({
            name: 'behavior',
            start: function (data) {
                // don't show at lv 1
                if (SaveValues.currentLevel === 1) {
                    entity.removeSelf();
                }
            }
        });
        var entity = new IconButton({
            sprite: HdButtonSprite({
                imageName: 'gui/buttons/blue',
                height: 64
            }),

            // icon options
            icon: icon,

            // offset of icon
            offset: new Vector2(0, 0),
            offsetWhenPressed: 2, // push pixels down when onButtonDown

            // icon transparency when inactive
            iconAlpha: 0,

            onButtonDown: function () {
                EventSystem.fire('cancelThrow');
            },
            onClick: function () {
                EventSystem.fire('GameAnalytics-addDesignEvent', {
                    eventId: 'UI:homeScreen:skinShopButton'
                });
                Bento.objects.attach(new SkinSelectPopup({}));
            },
            components: [
                behavior,
                hideListener
            ].concat(settings.components || [])
        }).extend({
            hide: function () {
                new Tween({
                    from: entity.position.x,
                    to: entity.position.x + 128,
                    in: 30,
                    ease: 'easeInBack',
                    onUpdate: function (v, t) {
                        entity.position.x = v;
                    },
                    onComplete: function () {
                        entity.active = false;
                    }
                });
            }
        });
        return entity;
    };
    return SkinsShopButton;
});