/**
 * Button to watch RV for skipping levels
 * @moduleName SkipLevelButton
 * @snippet SkipLevelButton|Constructor
SkipLevelButton({})
 */
bento.define('gui/buttons/skiplevelbutton', [
    'bento',
    'bento/utils',
    'bento/entity',
    'bento/math/vector2',
    'globals',
    'bento/components/anchor',
    'gui/iconbutton',
    'gui/hdbuttonsprite',
    'bento/gui/text',
    'modules/watchad',
    'bento/components/eventlistener',
    'bento/tween',
    'bento/components/spritecontainer',
    'managers/savevalues',
    'bento/eventsystem',
    'modules/localization'
], function (
    Bento,
    Utils,
    Entity,
    Vector2,
    Globals,
    Anchor,
    IconButton,
    HdButtonSprite,
    Text,
    WatchAd,
    EventListener,
    Tween,
    SpriteContainer,
    SaveValues,
    EventSystem,
    Localization
) {
    'use strict';
    var SkipLevelButton = function (settings) {
        var text = new Text({
            position: new Vector2(0, 14),
            text: Localization.getText('skipLv'),
            font: 'omnes',
            fontSize: 12,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            strokeStyle: '#000000',
            innerStroke: false,
            pixelStroke: true, // workaround for Cocoon bug
            antiAlias: true, // Cocoon only
            maxWidth: 64,
            drawDebug: true,
            components: []
        });
        var icon = new Entity({
            name: 'buttonIcon',
            components: [
                text,
            ]
        });
        var hideListener = new EventListener({
            name: 'eventListener',
            eventName: 'pathDrawingStarted',
            ignorePause: false,
            onEvent: function (data) {
                entity.remove(hideListener);
                entity.hide();
            }
        });
        var behavior = new Object({
            name: 'behavior',
            start: function (data) {
                // don't show at lv 1
                if (SaveValues.currentLevel === 1) {
                    entity.removeSelf();
                }
            }
        });
        var entity = new IconButton({
            name: 'skipLvButton',
            sprite: HdButtonSprite({
                imageName: 'gui/buttons/skip-rv',
                height: 56
            }),

            // icon options
            icon: icon,

            // offset of icon
            offset: new Vector2(0, 0),
            offsetWhenPressed: 2, // push pixels down when onButtonDown

            // icon transparency when inactive
            iconAlpha: 0,

            onButtonDown: function () {
                EventSystem.fire('cancelThrow');
            },

            onClick: function () {
                EventSystem.fire('GameAnalytics-addDesignEvent', {
                    eventId: 'UI:homeScreen:skipLevelButton'
                });
                if (WatchAd.canShowRewarded() || Bento.isDev()) {
                    EventSystem.fire('GameAnalytics-addDesignEvent', {
                        eventId: 'Ads:rv:skipLvAd'
                    });
                    WatchAd.showRewarded({
                        tag: 'skipLv',
                        onReward: function () {
                            Bento.objects.get('gameController', function (gameController) {
                                gameController.winLevel({
                                    skipInterstitial: true
                                });
                            });
                            EventSystem.fire('GameAnalytics-addDesignEvent', {
                                eventId: 'Ads:rv:skipLvAdComplete'
                            });
                        }
                    });
                }
            },
            components: [
                behavior,
                hideListener
            ].concat(settings.components || [])
        }).extend({
            hide: function () {
                new Tween({
                    from: entity.position.x,
                    to: entity.position.x + 128,
                    in: 30,
                    ease: 'easeInBack',
                    onUpdate: function (v, t) {
                        entity.position.x = v;
                    },
                    onComplete: function () {
                        entity.active = false;
                    }
                });
            }
        });
        return entity;
    };
    return SkipLevelButton;
});