/**
 * Coinbox on top of screen
 * @moduleName CoinBox
 * @snippet CoinBox|Constructor
CoinBox({})
 */
bento.define('gui/counters/coinbox', [
    'bento',
    'bento/utils',
    'bento/entity',
    'globals',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/spritecontainer',
    'bento/gui/text',
    'managers/savevalues',
    'bento/components/eventlistener',
    'bento/tween',
    'modules/tapticengine',
    'bento/gui/centeredlist'
], function (
    Bento,
    Utils,
    Entity,
    Globals,
    Vector2,
    Rectangle,
    SpriteContainer,
    Text,
    SaveValues,
    EventListener,
    Tween,
    TapticEngine,
    CenteredList
) {
    'use strict';
    var CoinBox = function (settings) {
        var coinText = new Text({
            position: new Vector2(8, 1),
            text: SaveValues.coins.toString(),
            font: 'omnes',
            fontSize: 14,
            fontColor: Globals.Colors.darkblue,
            align: 'right',
            textBaseline: 'middle',
            maxWidth: 32,
            linebreaks: false,
            drawDebug: false,
            components: []
        });
        var coolDown = 0;
        var box = new SpriteContainer({
            height: 32,
            imageName: 'gui/top/currency-box',
        });
        var holdUpdate = false;
        var coinListener = new EventListener({
            name: 'coinListener',
            eventName: 'updateCoins',
            ignorePause: true,
            onEvent: function (data) {
                if (holdUpdate) {
                    holdUpdate = false;
                    return;
                }
                coinText.setText(SaveValues.coins.toString());
            }
        });
        var holdListener = new EventListener({
            name: 'holdListener',
            eventName: 'holdUpdate',
            ignorePause: true,
            onEvent: function (coins) {
                holdUpdate = true;
            }
        });
        var fakeListener = new EventListener({
            name: 'fakeListener',
            eventName: 'fakeCoins',
            ignorePause: true,
            onEvent: function (coins) {
                coinText.setText(coins.toString());
                // show fake amount of coins (useful for tweens and such, before actually updating the coin amount)
            }
        });
        var notEnoughListener = new EventListener({
            name: 'notEnoughListener',
            eventName: 'notEnoughCoins',
            ignorePause: true,
            onEvent: function (coins) {
                entity.shake();
            }
        });
        var keysList = new CenteredList({
            spacing: 16, // in pixels
            useItemDimensions: false, // note: uses last item's dimension
            direction: 'x',
            position: new Vector2(0, -24),
            items: (function () {
                var items = [];
                Utils.repeat(3, function (i, l) {
                    items.push(new Entity({
                        name: 'key-' + i,
                        position: new Vector2(0, -2),
                        components: [
                            new SpriteContainer({
                                height: 16,
                                imageName: 'gui/chests/key',
                            }),
                            new Object({
                                name: 'behavior',
                                update: function (data) {
                                    data.entity.visible = (SaveValues.keys > i);
                                },
                            })
                        ]
                    }));
                });
                return items;
            })()
        });
        var entity = new Entity({
            z: Globals.Layers.COINS,
            name: 'coinBox',
            visible: !Bento.saveState.load('cheat-hideUi', false),
            components: [
                box,
                keysList,
                coinText,
                coinListener,
                fakeListener,
                holdListener,
                notEnoughListener,
                new Object({
                    name: 'behavior',
                    update: function (data) {
                        if (coolDown) {
                            coolDown -= 1;
                        }
                    }
                })
            ].concat(settings.components || [])
        }).extend({
            shake: function () {
                new Tween({
                    from: 4,
                    to: 0,
                    in: 40,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        entity.transform.x = Math.sin(t) * v;
                    },
                    onComplete: function () {
                        entity.transform.x = 0;
                    }
                });
            },
            bounce: function () {
                if (coolDown) {
                    return;
                }
                coolDown = 4;
                TapticEngine.impact({
                    type: "light"
                });
                new Tween({
                    from: 1.1,
                    to: 1,
                    in: 60,
                    ease: 'elastic',
                    decay: 5,
                    oscillations: 2,
                    onUpdate: function (v, t) {
                        entity.scale.x = v;
                        entity.scale.y = v;
                    },
                    onComplete: function () {

                    }
                });
            }
        });
        return entity;
    };
    return CoinBox;
});