/**
 * Coin counter
 * @moduleName CoinCounter
 * @snippet CoinCounter|constructor
CoinCounter({
    imageName: '',
    color: '#4da2ff',
    position: new Vector2(0, 0),
    alignment: 'center',
})
 */
bento.define('gui/counters/coincounter', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'abbreviate',
    'bento/components/spritecontainer'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Abbreviate,
    SpriteContainer
) {
    'use strict';
    return function (settings) {
        var position = settings.position || new Vector2(0, 0);
        var abbreviated = settings.abbreviated;
        var scale = settings.scale;
        var color = settings.color || '#ffffff';
        var fontSize = settings.fontSize || 14;
        var coinSize = settings.coinSize || fontSize;
        var maxWidth = settings.maxWidth || 96;
        var coins = settings.coins || settings.value || 0;
        var alignment = settings.alignment || 'center';
        var coinPos = settings.coinPos || 'left';
        var coinOffset = settings.coinOffset;
        // using text to replace with abbreviations, don't update too many times!
        var coinText = new Text({
            position: new Vector2(0, 1),
            text: abbreviated ? Abbreviate(coins) : coins.toString(),
            font: 'omnes',
            fontSize: fontSize,
            fontColor: color,
            align: alignment,
            textBaseline: 'middle',
            maxWidth: maxWidth,
            linebreaks: false,
            lineWidth: settings.outline || settings.lineWidth,
            strokeStyle: settings.strokeStyle,
            strokeOffset: settings.strokeOffset,
            components: [],
        });
        var coinSprite = new Entity({
            name: 'coinSprite',
            components: [
                new SpriteContainer({
                    imageName: settings.imageName || 'gui/top/currency-coins',
                    width: coinSize,
                    position: coinOffset,
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });
        var centerBehavior = new Object({
            name: 'behavior',
            update: function (data) {
                coinSprite.position.x = (coinSprite.dimension.width + coinText.dimension.width + 2) / 2 * (coinPos === 'left' ? -1 : 1);
                iconContainer.position.x = coinSprite.dimension.width / 2 * (coinPos === 'left' ? 1 : -1);
            }
        });
        var iconContainer = new Entity({
            name: 'iconContainer',
            components: [
                coinSprite,
                coinText
            ]
        });
        var coin = new Entity({
            name: 'coinCounter',
            position: position,
            scale: scale,
            components: [
                iconContainer
            ]
        }).extend({
            getWidth: function () {
                return coinSprite.dimension.width + coinText.dimension.width * coinText.scale.x;
            },
            setValue: function (value) {
                coins = value;
                coinText.setText(abbreviated ? Abbreviate(coins) : coins.toString());
            },
            getValue: function () {
                return coins;
            }
        });

        if (alignment === 'center') {
            coin.attach(centerBehavior);
        }

        return coin;
    };
});