/**
 * Popup background
 * @moduleName BaseDialog
 * @snippet BaseDialog|constructor
BaseDialog({
    text: '',
    title: '',
    components: [],
    onClose: null
})
 */
bento.define('gui/dialogs/basedialog', [
    'bento',
    'bento/utils',
    'bento/entity',
    'bento/math/rectangle',
    'bento/math/vector2',
    'bento/gui/text',
    'gui/iconbutton',
    'gui/hdbuttonsprite',
    'bento/components/anchor',
    'bento/components/fill',
    'globals',
    'bento/components/modal',
    'bento/components/spritecontainer'
], function (
    Bento,
    Utils,
    Entity,
    Rectangle,
    Vector2,
    Text,
    IconButton,
    HdButtonSprite,
    Anchor,
    Fill,
    Globals,
    Modal,
    SpriteContainer
) {
    'use strict';
    var BaseDialog = function (settings) {
        var safeZone = Anchor.getSafeZone();
        var viewport = Bento.getViewport();
        var components = settings.components || [];
        var canClose = Utils.getDefault(settings.canClose, true);
        var isModal = Utils.getDefault(settings.modal, true);
        var z = settings.z || Globals.Layers.GUI;
        var text = settings.text || '';
        var title = settings.title || '';
        var onComplete = settings.onComplete || function () {};
        var onClose = settings.onClose || function () {
            baseDialog.removeSelf();
        };

        /**
         * Close button
         */
        var closeButton = new IconButton({
            sort: true,
            position: new Vector2(64, -72),
            // sfx: 'sfx_button',
            sprite: new HdButtonSprite({
                imageName: 'gui/buttons/button-close',
                frameCountX: 1,
                frameCountY: 3,
                width: 28
            }),
            onClick: function () {
                onClose();
                onComplete();
            }
        });
        var background = new SpriteContainer({
            width: 160,
            imageName: 'gui/popup/popup-back',
            originRelative: new Vector2(0.5, 0.5),
        });
        
        var fill = new Entity({
            name: 'fill',
            components: [
                new SpriteContainer({
                    width: viewport.width + 16,
                    imageName: 'gui/bg-back',
                    alpha: 0.9,
                    originRelative: new Vector2(0.5, 0.5),
                })
            ]
        });

        var mainText = new Text({
            name: 'mainText',
            position: new Vector2(0, 0),
            text: text,
            font: 'omnes',
            fontSize: 12,
            fontColor: '#217eff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 128,
            maxHeight: 64,
            linebreaks: true,
            // drawDebug: true,
            components: []
        });

        var titleText = new Text({
            name: 'titleText',
            position: new Vector2(0, -62),
            text: title,
            font: 'panton',
            fontSize: 18,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 140,
            linebreaks: false,
            drawDebug: false,
            strokeStyle: ['#27895c', '#27895c'],
            lineWidth: [2, 2],
            strokeOffset: [new Vector2(0, 0), new Vector2(0, 2)],

            components: []
        });
        var modal = new Modal({});
        var baseDialog = new Entity({
            z: z,
            name: settings.name || 'baseDialog',
            position: safeZone.getRelative(0.5, 0.5),
            components: [
                isModal ? modal : {
                    name: 'dummy'
                },
                fill,
                background,
                titleText,
                mainText,
            ].concat(components)
        }).extend({
            dismiss: function () {
                baseDialog.removeSelf();
                onComplete();
            }
        });

        if (canClose) {
            baseDialog.attach(closeButton);
        }
        return baseDialog;
    };
    return BaseDialog;
});