/**
 * HD buttons
 * @moduleName HdButtonSprite
 * @snippet HdButtonSprite|constructor
HdButtonSprite({
    imageName: '$1',
    width: ${2:32},
    height: ${3:null}
})
 */
bento.define('gui/hdbuttonsprite', [
    'bento',
    'bento/utils',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/spritecontainer'
], function (
    Bento,
    Utils,
    Vector2,
    Rectangle,
    SpriteContainer
) {
    'use strict';

    // fix for SpriteContainer if missing
    var HdButtons = function (settings) {
        var imageName = settings.imageName;
        var height = settings.height || 0;
        var width = settings.width || 0;
        var frameCountX = settings.frameCountX || 1;
        var frameCountY = settings.frameCountY || 3;
        var framesTotal = frameCountX * frameCountY;
        var spriteContainer = new SpriteContainer({
            width: width,
            height: height,
            imageName: imageName,
            frameCountX: frameCountX,
            frameCountY: frameCountY,
            originRelative: new Vector2(0.5, 0.5),
            animations: {
                'up': {
                    speed: 0,
                    frames: [0]
                },
                'down': {
                    speed: 0,
                    frames: [Utils.clamp(0, 1, framesTotal - 1)]
                },
                'inactive': {
                    speed: 0,
                    frames: [Utils.clamp(0, 2, framesTotal - 1)]
                }
            }
        });
        // add missing functionality
        spriteContainer.setAnimation = function (anim) {
            spriteContainer.sprite.setAnimation(anim);
        };

        return spriteContainer;
    };
    return HdButtons;
});