/**
 * an overlay for the banner
 * @moduleName BannerOverlay
 */
bento.define('gui/overlays/banneroverlay', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/components/fill',
    'bento/components/anchor',
    'bento/components/eventlistener',
    'modules/banner'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Fill,
    Anchor,
    EventListener,
    BannerAd
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var overlaySize = 26;
        var fillSize = new Rectangle(0, 0, viewport.width, overlaySize);
        var origin = new Vector2(viewport.width / 2, overlaySize);
        var fill = new Entity({
            name: 'fill',
            components: [
                new Fill({
                    dimension: fillSize,
                    color: [0, 0, 0, 0.6], // [1, 1, 1, 1] is pure white
                    origin: origin
                })
            ]
        });

        var anchor = new Anchor({
            relative: new Vector2(0.5, 1),
            offset: new Vector2(0, 0),
            onResize: function (isPortrait) {
                var extraHeight = Math.max(0, viewport.height - Anchor.getSafeZone().getY2());
                fillSize.width = viewport.width;
                origin.x = viewport.width / 2;
                fillSize.height = overlaySize + extraHeight;
            }
        });

        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                if (BannerAd.isVisible()) {
                    entity.visible = true;
                } else {
                    entity.visible = false;
                }
            },
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        };

        var entity = new Entity({
            z: 1,
            name: 'BannerOverlay',
            position: settings.position,
            updateWhenPaused: 0,
            components: [
                fill,
                anchor,
                behavior,
                new EventListener({
                    name: 'bannerListener',
                    eventName: 'bannerStatusChanged',
                    ignorePause: false,
                    onEvent: function () {
                        if (BannerAd.isVisible()) {
                            entity.visible = true;
                        } else {
                            entity.visible = false;
                        }
                    },
                }),
            ]
        });
        return entity;
    };
});