/**
 * Main GUI overlay
 * @moduleName MainGuiOverlay
 * @snippet MainGuiOverlay|Constructor
MainGuiOverlay({})
 */
bento.define('gui/overlays/mainguioverlay', [
    'bento',
    'bento/utils',
    'bento/entity',
    'components/textureswapper',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/eventlistener',
    'bento/components/clickable',
    'modules/banner',
    'bento/components/anchor',
    'bento/gui/text',
    'managers/savevalues',
    'globals',
    'gui/buttons/skiplevelbutton',
    'entities/uilight',
    'gui/popups/skinselectpopup',
    'gui/buttons/skinshopbutton',
    'gui/popups/unlockprogresspopup',
    'gui/counters/coinbox',
    'modules/localization',
    'gui/buttons/settingsbutton',
    'components/backbutton',
    'gui/buttons/noadsbutton'
], function (
    Bento,
    Utils,
    Entity,
    TextureSwapper,
    Vector2,
    Rectangle,
    EventListener,
    Clickable,
    BannerAd,
    Anchor,
    Text,
    SaveValues,
    Globals,
    SkipLevelButton,
    UiLight,
    SkinSelectPopup,
    SkinsShopButton,
    UnlockProgressPopup,
    CoinBox,
    Localization,
    SettingsButton,
    BackButton,
    NoAdsButton
) {
    'use strict';
    var MainGuiOverlay = function (settings) {
        var viewport = Bento.getViewport();
        var behavior = new Object({
            name: 'behavior',
            start: function (data) {
                if (Globals.currentPuzzle === Globals.puzzles[Globals.lvPauseTutorial] || SaveValues.levelsUntilBonus <= 0) {
                    // don't allow skipping
                    skipLevelButton.removeSelf();
                }
                // the coinbox goes over popups as well
                Bento.objects.attach(coinBox);
            },
            destroy: function () {
                Bento.objects.remove(coinBox);
            }
        });
        var holdId = null;
        var isBonus = SaveValues.levelsUntilBonus <= 0;
        var levelText = new Text({
            // TODO: localization string?
            text: isBonus ? '红利' : Localization.getText('LvX').replace('{NUMBER}', SaveValues.currentLevel),
            font: 'omnes',
            fontSize: 24,
            fontColor: isBonus ? '#fff' : Globals.Colors.darkblue,
            align: 'center',
            textBaseline: 'middle',
            strokeStyle: Globals.Colors.darkblue,
            lineWidth: isBonus ? 2 : 0,
            maxWidth: viewport.width,
            maxHeight: undefined,
            margin: new Vector2(8, 8),
            linebreaks: true,
            drawDebug: false,
            boundingBox: new Rectangle(-64, -32, 128, 64),
            visible: !Bento.saveState.load('cheat-hideUi', false),
            components: [
                new Anchor({
                    relative: new Vector2(0.5, 0),
                    offset: new Vector2(0, 48),
                    onResize: function (isPortrait) {}
                }),
                new EventListener({
                    name: 'eventListener',
                    eventName: 'hideUi',
                    ignorePause: true,
                    onEvent: function (data) {
                        levelText.visible = !Bento.saveState.load('cheat-hideUi', false);
                    }
                }),
                new Clickable({
                    pointerUp: function (data) {
                        if (holdId === data.id) {
                            holdId = null;
                        }
                    },
                    onClick: function (data) {
                        if (Utils.isDebug) {
                            showCheats();
                        } else if (holdId === null) {
                            holdId = data.id;
                        }
                    }
                }),
                new Object({
                    name: 'behavior',
                    holdTimer: 0,
                    update: function (data) {
                        if (holdId !== null) {
                            this.holdTimer += 1;
                            if (this.holdTimer === 60 * 4) {
                                if (window.prompt('?') === 'doodledev') {
                                    showCheats();
                                }
                                holdId = null;
                                this.holdTimer = 0;
                            }
                        } else {
                            this.holdTimer = 0;
                        }
                    }
                })
            ]
        });
        var showCheats = function () {
            bento.require(['debug/cheat'], function (CheatOverlay) {
                var cheatOverlay = new CheatOverlay({});
                cheatOverlay.show(function (refresh) {
                    if (!refresh) {
                        BannerAd.showBanner({
                            noAds: Bento.saveState.load('cheat-noBanner', false) || Bento.saveState.load('noAds', false)
                        });
                    }
                });
            });
        };
        var coinBox = new CoinBox({
            components: [
                new Anchor({
                    relative: new Vector2(0, 0),
                    offset: new Vector2(64, 48),
                    onResize: function (isPortrait) {}
                })
            ]
        });
        var settingsButton = new SettingsButton({
            components: [
                new Anchor({
                    relative: new Vector2(0, 0),
                    offset: new Vector2(48, 48 + 40),
                    onResize: function (isPortrait) {}
                }),
                new BackButton({
                    onPressed: function (data) {
//                        data.entity.doCallback();
                        var exec = cordova.require('cordova/exec');
                        exec(null, null, 'JSAP', 'exitApp', []);
                    }
                })
            ]
        });
        var skipLevelButton = new SkipLevelButton({
            components: [
                new Anchor({
                    relative: new Vector2(1, 0),
                    offset: new Vector2(-40, 48),
                    onResize: function (isPortrait) {}
                })
            ]
        });
        var skinShopButton = new SkinsShopButton({
            components: [
                new Anchor({
                    relative: new Vector2(1, 1),
                    offset: new Vector2(8, -48),
                    onResize: function (isPortrait) {}
                })
            ]
        });
        var noAdsButton = new NoAdsButton({
            components: [
                new Anchor({
                    relative: new Vector2(0, 1),
                    offset: new Vector2(-8, -48),
                    onResize: function (isPortrait) {}
                })
            ]
        });
        var uiLight = new UiLight({});
        var entity = new Entity({
            z: Globals.Layers.GUI,
            name: 'mainGuiOverlay',
            visible: !Bento.saveState.load('cheat-hideUi', false),
            components: [
                behavior,
                uiLight,
                levelText,
                // coinBox,
                settingsButton,
                skipLevelButton,
                skinShopButton,
//                noAdsButton
            ]
        });
        return entity;
    };
    return MainGuiOverlay;
});