/**
 * Chest screen
 * @moduleName ChestPopup
 * @snippet ChestPopup|Constructor
ChestPopup({
    onComplete: function (didWatchAd) {}
})
 */
bento.define('gui/popups/chestpopup', [
    'bento',
    'bento/utils',
    'bento/entity',
    'bento/math/vector2',
    'bento/math/rectangle',
    'globals',
    'bento/components/spritecontainer',
    'bento/components/anchor',
    'modules/localization',
    'bento/gui/text',
    'gui/iconbutton',
    'gui/hdbuttonsprite',
    'bento/components/modal',
    'bento/components/clickable',
    'bento/gui/centeredlist',
    'onigiri/sprite3d',
    'onigiri/onigiri',
    'components/targetsize',
    'components/materialswapper',
    'bento/tween',
    'onigiri/animator',
    'gui/widgets/coinrain',
    'managers/savevalues',
    'bento/eventsystem',
    'modules/watchad',
    'components/waitingforad',
    'components/backbutton',
    'bento/components/eventlistener'
], function (
    Bento,
    Utils,
    Entity,
    Vector2,
    Rectangle,
    Globals,
    SpriteContainer,
    Anchor,
    Localization,
    Text,
    IconButton,
    HdButtonSprite,
    Modal,
    Clickable,
    CenteredList,
    Sprite3d,
    Onigiri,
    TargetSize,
    MaterialSwapper,
    Tween,
    Animator,
    CoinRain,
    SaveValues,
    EventSystem,
    WatchAd,
    WaitingForAd,
    BackButton,
    EventListener
) {
    'use strict';
    var ChestPopup = function (settings) {
        var onComplete = settings.onComplete;
        var viewport = Bento.getViewport();
        var modal = new Modal({});
        var fill = new Entity({
            name: 'fill',
            components: [
                new SpriteContainer({
                    width: viewport.width + 16,
                    alpha: 0.9,
                    imageName: 'gui/bg-back',
                    originRelative: new Vector2(0.5, 0.5),
                }),
                new Anchor({
                    relative: new Vector2(0.5, 0.5),
                })
            ]
        });
        var unlockedBalls = SaveValues.unlockedBalls;
        var didWatchAd = false;
        var adsWatched = 0;
        var skinName = (function () {
            // randomize skin
            var skin = 'default';
            var skins = [].concat(Globals.balls);
            var removeBall = function (ball) {
                var i = skins.indexOf(ball);
                if (i > -1) {
                    skins.splice(i, 1);
                }
            };
            removeBall(SaveValues.dailyBonusBall);
            while (unlockedBalls.indexOf(skin) >= 0 && skins.length) {
                skin = Utils.pickRandomFrom(skins);
                removeBall(skin);
            }
            return skin;
        })();

        /**
         * Dialog box
         */
        var topBanner = new Entity({
            name: 'topBanner',
            components: [
                new SpriteContainer({
                    width: 336,
                    imageName: 'gui/popup/banner',
                }),
                new Text({
                    position: new Vector2(0, -32),
                    text: Localization.getText('openChests'),
                    font: 'omnes',
                    fontSize: 32,
                    fontColor: '#ffffff',
                    align: 'center',
                    textBaseline: 'middle',
                    maxWidth: 220,
                    linebreaks: false,
                    lineWidth: [3],
                    strokeStyle: ['#03d2ff'],
                    strokeOffset: [new Vector2(0, 1.5)],
                    components: []
                }),
                new Anchor({
                    relative: new Vector2(0.5, 0.5),
                    offset: new Vector2(0, -144)
                })
            ]
        });
        var bestRewardContainer = new Entity({
            name: 'bestRewardContainer',
            position: new Vector2(0, -178),
            components: [
                new SpriteContainer({
                    width: 128,
                    imageName: 'gui/chests/top-prize-box',
                }),
                new Text({
                    position: new Vector2(0, -20),
                    text: Localization.getText('topPrize'),
                    font: 'omnes',
                    fontSize: 14,
                    fontColor: '#ffffff',
                    align: 'center',
                    textBaseline: 'middle',
                    antiAlias: true, // Cocoon only
                    maxWidth: 88,
                    linebreaks: false,
                    drawDebug: false,
                    components: []
                }),
                new SpriteContainer({
                    height: 40,
                    position: new Vector2(0, 8),
                    imageName: 'gui/skins/icons/ball-' + skinName,
                })
            ]
        });
        /**
         * Close button
         */
        var buttonClose = new IconButton({
            name: 'buttonClose',
            position: new Vector2(0, 0),
            sprite: HdButtonSprite({
                imageName: 'gui/buttons/button-close',
                width: 48
            }),
            visible: false,
            active: false,
            // icon options
            icon: new Entity({
                name: 'none'
            }),

            onClick: function () {
                entity.removeSelf();
                if (onComplete) {
                    onComplete(didWatchAd);
                    EventSystem.fire('GameAnalytics-addDesignEvent', {
                        eventId: 'UI:chestPopup:adsWatched',
                        value: adsWatched
                    });
                }
            },
            components: [
                new Anchor({
                    relative: new Vector2(0.5, 0.5),
                    offset: new Vector2(136, -128)
                }),
                new BackButton({
                    onPressed: function (data) {
                        data.entity.doCallback();
                    }
                })
            ]
        });
        /**
         * Keys box
         */
        var keysList = new CenteredList({
            spacing: 36, // in pixels
            useItemDimensions: false, // note: uses last item's dimension
            direction: 'x',
            position: new Vector2(0, 0),
            items: (function () {
                var items = [];
                Utils.repeat(3, function (i, l) {
                    items.push(new Entity({
                        name: 'key-' + i,
                        position: new Vector2(0, -2),
                        components: [
                            new SpriteContainer({
                                height: 38,
                                imageName: 'gui/chests/key',
                            }),
                            new Object({
                                name: 'behavior',
                                update: function (data) {
                                    data.entity.visible = (SaveValues.keys > i);
                                },
                            })
                        ]
                    }));
                });
                return items;
            })()
        });
        var keyBox = new Entity({
            name: 'keyBox',
            position: new Vector2(0, 160),
            components: [
                new SpriteContainer({
                    width: 144,
                    imageName: 'gui/chests/keys-box',
                }),
                keysList
            ]
        });
        /**
         * Buy button
         */
        var buttontext = new Text({
            position: new Vector2(16, -16),
            text: Localization.getText('getKeys'),
            font: 'omnes',
            fontSize: 24,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            antiAlias: true, // Cocoon only
            maxWidth: 88,
            lineWidth: 2,
            strokeStyle: Globals.Colors.coinOutline,
            linebreaks: false,
            drawDebug: false,
            components: []
        });
        var icon = new Entity({
            name: 'icon',
            components: [
                buttontext,
                new CenteredList({
                    spacing: 28, // in pixels
                    useItemDimensions: false, // note: uses last item's dimension
                    direction: 'x',
                    position: new Vector2(16, 12),
                    items: (function () {
                        var items = [];
                        Utils.repeat(3, function (i, l) {
                            items.push(new SpriteContainer({
                                height: 32,
                                imageName: 'gui/chests/key',
                            }));
                        });
                        return items;
                    })()
                })
            ]
        });
        var buttonRv = new IconButton({
            name: 'button',
            position: new Vector2(0, 160),
            sprite: HdButtonSprite({
                imageName: 'gui/buttons/orange-rv',
                width: 168,
            }),
            icon: icon,
            offsetWhenPressed: 2, // push pixels down when onButtonDown
            active: false,
            visible: false,
            onClick: function () {
                EventSystem.fire('GameAnalytics-addDesignEvent', {
                    eventId: 'UI:chestPopup:getKeysButton'
                });
                if (WatchAd.canShowRewarded() || Bento.isDev()) {
                    EventSystem.fire('GameAnalytics-addDesignEvent', {
                        eventId: 'Ads:rv:getKeysAd'
                    });
                    WatchAd.showRewarded({
                        tag: 'getKeys',
                        onReward: function () {
                            SaveValues.keys = 3;
                            adsWatched += 1;

                            // re-hide
                            buttonRv.active = false;
                            buttonRv.visible = false;
                            buttonClose.active = false;
                            buttonClose.visible = false;

                            EventSystem.fire('GameAnalytics-addDesignEvent', {
                                eventId: 'Ads:rv:getKeysAdComplete'
                            });

                            didWatchAd = true;
                        }
                    });
                }
            },
            components: [
                new Object({
                    name: 'behavior',
                    update: function (data) {
                        if (data.entity.ticker % 120 === 0) {
                            new Tween({
                                from: Math.PI / 32,
                                to: 0,
                                in: 45,
                                ease: 'elastic',
                                decay: 5,
                                oscillations: 4,
                                onUpdate: function (v, t) {
                                    buttonRv.rotation = v;
                                }
                            });
                        }
                    }
                }),
                new WaitingForAd({
                    offset: new Vector2(12, 0)
                })
            ]
        });
        /**
         * Backboard container
         */
        var backBoard = new Entity({
            name: 'backboard',
            components: [
                new SpriteContainer({
                    width: 320,
                    imageName: 'gui/skins/backboard-skins',
                }),
                bestRewardContainer,
                keyBox,
                buttonRv,
                new Anchor({
                    relative: new Vector2(0.5, 0.5),
                    offset: new Vector2(0, 64)
                }),
            ]
        });

        /**
         * Slots
         */
        var chestSlots = [];
        var rewardCoins = function (amount) {
            return function (slot) {
                showCoins(slot, amount);
            };
        };
        var rewardSkin = function () {
            return showSkin;
        };

        var showSkin = function (slot) {
            var position = slot.getWorldPosition().add(new Vector2(0, -32));
            var box = new Entity({
                name: 'box',
                position: position,
                components: [
                    new SpriteContainer({
                        width: 128,
                        imageName: 'gui/chests/prize-bubble',
                    }),
                    new SpriteContainer({
                        height: 64,
                        position: new Vector2(0, -16),
                        imageName: 'gui/skins/icons/ball-' + skinName,
                    }),
                    new EventListener({
                        name: 'eventListener',
                        eventName: 'closePopup',
                        ignorePause: false,
                        onEvent: function (data) {
                            box.removeSelf();
                        }
                    })
                ]
            });
            new Tween({
                from: 0,
                to: 0.47,
                in: 30,
                ease: 'elastic',
                decay: 5,
                oscillations: 1,
                onUpdate: function (v, t) {
                    box.scale.x = v;
                    box.scale.y = v;
                }
            });
            entity.attach(box);

            // save
            unlockedBalls = SaveValues.unlockedBalls;
            if (unlockedBalls.indexOf(skinName) < 0) {
                unlockedBalls.push(skinName);
                SaveValues.unlockedBalls = unlockedBalls;
            }

            // set the ball
            SaveValues.currentBall = skinName;
        };
        var showCoins = function (slot, amount) {
            Bento.objects.get('coinBox', function (coinBox) {
                var savedCoins = SaveValues.coins;
                var coinText = new Text({
                    position: new Vector2(8, -16),
                    text: '+' + amount,
                    font: 'omnes',
                    fontSize: 40,
                    fontColor: Globals.Colors.coin,
                    strokeStyle: Globals.Colors.coinOutline,
                    lineWidth: 3,
                    align: 'right',
                    textBaseline: 'middle',
                    maxWidth: 64,
                    linebreaks: false,
                    drawDebug: false,
                    components: []
                });
                var position = slot.getWorldPosition().add(new Vector2(0, -32));
                var box = new Entity({
                    name: 'box',
                    position: position,
                    components: [
                        new SpriteContainer({
                            width: 128,
                            imageName: 'gui/chests/prize-bubble',
                        }),
                        new SpriteContainer({
                            width: 36,
                            imageName: 'gui/icons/big-coin',
                            position: new Vector2(32, -14),
                        }),
                        coinText,
                        new EventListener({
                            name: 'eventListener',
                            eventName: 'closePopup',
                            ignorePause: false,
                            onEvent: function (data) {
                                box.removeSelf();
                            }
                        })
                    ]
                });
                new Tween({
                    from: 0,
                    to: 0.47,
                    in: 30,
                    ease: 'elastic',
                    decay: 5,
                    oscillations: 1,
                    onUpdate: function (v, t) {
                        box.scale.x = v;
                        box.scale.y = v;
                    }
                });
                entity.attach(new CoinRain({
                    z: coinBox.z - 2,
                    targetPosition: coinBox.position.clone(),
                    spawnPosition: position,
                    coinTotal: Math.min(amount, 10),
                    onDestroy: function (coin, completion) {
                        coinBox.bounce();
                        EventSystem.fire('fakeCoins', savedCoins + Math.floor(amount * completion));
                    },
                    onComplete: function () {
                        SaveValues.coins += amount;
                        EventSystem.fire('updateCoins');
                    }
                }));
                entity.attach(box);
            });
        };
        var rewards = [
            rewardCoins(1),
            rewardCoins(5),
            rewardCoins(10),
            rewardCoins(25),
            rewardCoins(25),
            rewardCoins(50),
            rewardCoins(50),
            rewardCoins(75),
            rewardSkin()

        ];
        var addSlot = function (i, l) {
            var innerMesh = Onigiri.getMesh('chest.gltf');
            var chestMesh = new THREE.Object3D().add(innerMesh);
            var chest = new Sprite3d({
                object3D: chestMesh,
            });
            var animator = new Animator({
                object3D: innerMesh,
                rootObject: innerMesh,
                defaultAnimation: 'closed'
            });
            var didOpen = false;
            var spriteBack = new SpriteContainer({
                width: 88,
                imageName: 'gui/chests/chest-locked',
            });
            var slot = new Entity({
                name: 'slot',
                position: new Vector2(
                    96 * Math.floor(i % 3) - 96,
                    84 * Math.floor(i / 3) - 84
                ),
                components: [
                    spriteBack,
                    chest,
                    new TargetSize({
                        width: 64,
                        mesh: innerMesh
                    }),
                    new MaterialSwapper({
                        id: 'chest', // unique id for refcounting
                        type: 'MeshBasicMaterial',
                        deepSwap: true,
                        object3D: innerMesh,
                        materialSettings: {
                            map: Onigiri.getTexture('textures/tex-chest', {
                                wrapS: THREE.RepeatWrapping,
                                wrapT: THREE.RepeatWrapping
                            }),
                            skinning: true
                        }
                    }),
                    animator,
                    new Object({
                        name: 'behavior',
                        start: function (data) {
                            chestMesh.rotation.x = Math.PI / 16;
                            chestMesh.rotation.y = Math.PI / 16;
                            chestMesh.position.y = -4;
                            chestMesh.position.z = -100;
                            Utils.timeout(i * 5, function () {
                                animator.stop();
                                animator.play('spin', {
                                    loop: false
                                }, false);
                            });
                            /*new Tween({
                                from: Math.PI,
                                to: Math.PI / 16,
                                in: 60,
                                ease: 'elastic',
                                decay: 8,
                                oscillations: 1,
                                delay: i * 5,
                                applyOnDelay: true,
                                onStart: function () {
                                },
                                onUpdate: function (v, t) {
                                    chestMesh.rotation.y = v;
                                },
                                onComplete: function () {

                                }
                            });*/
                            new Tween({
                                from: 0,
                                to: chestMesh.scale.x,
                                in: 60,
                                delay: i * 5,
                                applyOnDelay: true,
                                ease: 'elastic',
                                decay: 5,
                                oscillations: 1,
                                onUpdate: function (v, t) {
                                    chestMesh.scale.x = v;
                                    chestMesh.scale.y = v;
                                    chestMesh.scale.z = v;
                                },
                                onComplete: function () {

                                }
                            });

                        }
                    }),
                    new Clickable({
                        onClick: function (data) {
                            if (didOpen || SaveValues.keys === 0) {
                                return;
                            }
                            SaveValues.keys -= 1;
                            didOpen = true;
                            spriteBack.setup({
                                width: 88,
                                imageName: 'gui/chests/chest-opened',
                            });
                            animator.stop();
                            animator.play('opening', {
                                loop: false
                            }, false);

                            // pick reward
                            var index = Utils.getRandom(rewards.length);
                            var rewardFn = rewards[index];

                            // first time ball: try again
                            if (!didWatchAd && rewardFn === showSkin && Utils.getRandom(2)) {
                                console.log('re-roll');
                                index = Utils.getRandom(rewards.length);
                                rewardFn = rewards[index];
                            }

                            // remove from array
                            rewards.splice(index, 1);

                            Utils.timeout(50, function () {
                                rewardFn(slot);
                            });

                            // run out of keys? show rv button
                            if (SaveValues.keys <= 0 && rewards.length) {
                                buttonRv.visible = true;
                                buttonRv.active = true;
                                buttonClose.visible = true;
                                buttonClose.active = true;
                            } else if (rewards.length === 0) {
                                // remove self
                                Utils.timeout(120, function () {
                                    EventSystem.fire('closePopup');
                                    new Tween({
                                        to: 0,
                                        from: topBanner.scale.x,
                                        in: 30,
                                        delay: 10,
                                        ease: 'easeInBack',
                                        onUpdate: function (v, t) {
                                            topBanner.scale.x = v;
                                            topBanner.scale.y = v;
                                        }
                                    });
                                    new Tween({
                                        to: 0,
                                        from: backBoard.scale.x,
                                        in: 30,
                                        ease: 'easeInBack',
                                        applyOnDelay: true,
                                        onUpdate: function (v, t) {
                                            backBoard.scale.x = v;
                                            backBoard.scale.y = v;
                                        },
                                        onComplete: function () {
                                            entity.removeSelf();
                                            if (onComplete) {
                                                onComplete(didWatchAd);
                                                EventSystem.fire('GameAnalytics-addDesignEvent', {
                                                    eventId: 'UI:chestPopup:adsWatched',
                                                    value: adsWatched
                                                });
                                            }
                                        }
                                    });
                                });
                            }
                        }
                    })
                ]
            });

            chestSlots.push(slot);
            backBoard.attach(slot);
        };
        var behavior = new Object({
            name: 'behavior',
            start: function (data) {
                new Tween({
                    from: 0,
                    to: topBanner.scale.x,
                    in: 30,
                    ease: 'elastic',
                    decay: 7,
                    onUpdate: function (v, t) {
                        topBanner.scale.x = v;
                        topBanner.scale.y = v;
                    }
                });
                new Tween({
                    from: 0,
                    to: backBoard.scale.x,
                    in: 30,
                    ease: 'elastic',
                    decay: 7,
                    delay: 10,
                    applyOnDelay: true,
                    onStart: function () {
                        Utils.repeat(9, addSlot);
                    },
                    onUpdate: function (v, t) {
                        backBoard.scale.x = v;
                        backBoard.scale.y = v;
                    }
                });
            }
        });
        var entity = new Entity({
            z: Globals.Layers.POPUPS,
            name: 'chestPopup',
            components: [
                modal,
                behavior,
                fill,
                topBanner,
                backBoard,
                buttonClose
            ]
        });

        return entity;
    };
    return ChestPopup;
});