/**
 * Daily gift popup
 * @moduleName DailyGiftPopup
 * @snippet DailyGiftPopup|Constructor
DailyGiftPopup({})
 */
bento.define('gui/popups/dailygiftpopup', [
    'bento',
    'bento/utils',
    'bento/entity',
    'bento/math/vector2',
    'bento/math/rectangle',
    'gui/iconbutton',
    'gui/hdbuttonsprite',
    'bento/gui/scrollinglist',
    'bento/components/spritecontainer',
    'bento/components/anchor',
    'bento/components/modal',
    'globals',
    'bento/components/fill',
    'bento/gui/text',
    'bento/components/clickable',
    'managers/savevalues',
    'bento/tween',
    'bento/eventsystem',
    'modules/localization',
    'gui/counters/coincounter',
    'gui/widgets/confetti',
    'modules/tapticengine',
    'gui/popups/gotcoinspopup',
    'gui/popups/receivepopup'
], function (
    Bento,
    Utils,
    Entity,
    Vector2,
    Rectangle,
    IconButton,
    HdButtonSprite,
    ScrollingList,
    SpriteContainer,
    Anchor,
    Modal,
    Globals,
    Fill,
    Text,
    Clickable,
    SaveValues,
    Tween,
    EventSystem,
    Localization,
    CoinCounter,
    Confetti,
    TapticEngine,
    GotCoinsPopup,
    ReceivePopup
) {
    'use strict';
    var balls;
    var firstBalls = ['americanfootball', 'golf', 'soccer', 'tennisball'];
    var unlockedBalls = SaveValues.unlockedBalls;
    var players = Globals.players;
    var unlockedPlayers = SaveValues.unlockedPlayers;
    var DailyGiftPopup = function (settings) {
        var viewport = Bento.getViewport();
        var animate = settings.animate;
        var ballName = (function () {
            // populate balls list and remove sportsballs from list since player will probably unlock it before third day
            if (!balls) {
                balls = [].concat(Globals.balls);
                firstBalls.forEach(function (b) {
                    balls.splice(balls.indexOf(b), 1);
                });
            }

            // randomize skin
            var skin = SaveValues.dailyBonusBall;
            var skins = [].concat(balls);
            while (unlockedBalls.indexOf(skin) >= 0 && skins.length) {
                skin = Utils.pickRandomFrom(skins);
                skins.splice(skins.indexOf(skin), 1);
            }
            SaveValues.dailyBonusBall = skin;
            return skin;
        })();

        var characterName = (function () {
            // randomize skin
            var skin = SaveValues.dailyBonusCharacter;
            var skins = [].concat(players);
            var removeCharacter = function (character) {
                var i = skins.indexOf(character);
                if (i > -1) {
                    skins.splice(i, 1);
                }
            };
            removeCharacter(SaveValues.nextCharacter);
            while (unlockedPlayers.indexOf(skin) >= 0 && skins.length) {
                skin = Utils.pickRandomFrom(skins);
                removeCharacter(skin);
            }
            SaveValues.dailyBonusCharacter = skin;
            return skin;
        })();

        var fill = new Entity({
            name: 'fill',
            components: [
                // new Fill({
                //     name: 'fill',
                //     dimension: viewport.getSize().grow(16),
                //     color: [0, 0, 0, 0.5], // [1, 1, 1, 1] is pure white
                //     origin: new Vector2(0, 8)
                // })
                new SpriteContainer({
                    width: viewport.width + 16,
                    alpha: 0.9,
                    imageName: 'gui/bg-back',
                    originRelative: new Vector2(0.5, 0.5),
                }),
            ]
        });
        var isAnimating = false;
        var hapticCooldown = 0;
        /**
         * Dialog box
         */
        var topBanner = new Entity({
            name: 'topBanner',
            position: new Vector2(0, -174),
            components: [
                new SpriteContainer({
                    width: 370,
                    imageName: 'gui/popup/banner',
                }),
                new Text({
                    position: new Vector2(0, -32),
                    text: Localization.getText('dailyBonus').toLowerCase(),
                    font: 'omnes',
                    fontSize: 36,
                    fontColor: '#ffffff',
                    align: 'center',
                    textBaseline: 'middle',
                    maxWidth: 245,
                    linebreaks: true,
                    lineWidth: [3],
                    strokeStyle: ['#03d2ff'],
                    strokeOffset: [new Vector2(0, 1.5)],
                    margin: new Vector2(2, 2),
                    components: []
                }),
            ]
        });

        var offsetBoard = 48;
        var backBoard = new Entity({
            name: 'backboard',
            position: new Vector2(0, offsetBoard),
            components: [
                new SpriteContainer({
                    width: 334,
                    imageName: 'gui/dailybonus/bac',
                }),
            ]
        });

        var contentContainer = new Entity({
            name: 'contentContainer',
            position: new Vector2(0, offsetBoard)
        });

        var buttonClose = new IconButton({
            name: 'buttonClose',
            position: new Vector2(136, -160),
            sprite: HdButtonSprite({
                imageName: 'gui/buttons/button-close',
                width: 48
            }),
            // icon options
            icon: new Entity({
                name: 'none'
            }),

            onClick: function () {
                if (isAnimating) {
                    return;
                }
                entity.removeSelf();
            },
        });
        /**
         * Contents
         */
        var currentDay = SaveValues.dailyBonusDay % 7;
        var addDay = function (daySettings) {
            var dayType = Utils.sign(daySettings.day - currentDay);
            var boxType;
            var prizeBoxType;
            switch (dayType) {
                case -1:
                    boxType = 'daybox-done';
                    prizeBoxType = 'prizebox';
                    break;
                case 0:
                    boxType = 'daybox-current';
                    prizeBoxType = 'prizebox-current';
                    break;
                case 1:
                    boxType = 'daybox-after';
                    prizeBoxType = 'prizebox';
                    break;
            }

            var box = new SpriteContainer({
                width: 282,
                height: null,
                imageName: 'gui/dailybonus/' + boxType,
                position: new Vector2(0, 0),
                originRelative: new Vector2(0.5, 0.5),
                animations: {
                    default: {
                        speed: 0,
                        frames: [0]
                    }
                }
            });

            var day = new Text({
                position: new Vector2(-120, 0),
                text: Localization.getText('dayNumber').replace('{NUMBER}', (daySettings.day + 1).toString()),
                font: 'omnes',
                fontSize: 16,
                fontColor: '#ffffff',
                align: 'left',
                textBaseline: 'middle',
                // maxWidth: 200,
                lineWidth: 3,
                strokeStyle: dayType < 1 ? '#217eff' : '#b2b3c1',
                components: []
            });

            var option;

            var getCheckmark = function () {
                return new SpriteContainer({
                    width: 38,
                    height: null,
                    imageName: 'gui/dailybonus/tick',
                    position: new Vector2(86, 0),
                    originRelative: new Vector2(0.5, 0.5),
                });
            };
            var getButton = function () {
                var claimText = new Text({
                    position: new Vector2(0, 0),
                    text: Localization.getText('btn_claim'),
                    font: 'omnes',
                    fontSize: 22,
                    fontColor: '#ffffff',
                    align: 'center',
                    textBaseline: 'middle',
                    strokeStyle: '#ff592c',
                    ySpacing: 4,
                    lineWidth: 4,
                    maxWidth: 80,
                    maxHeight: 64,
                    linebreaks: false,
                    drawDebug: false,
                    components: []
                });
                var btn = new IconButton({
                    position: new Vector2(86, 0),
                    sprite: HdButtonSprite({
                        imageName: 'gui/dailybonus/claim-button',
                        width: 102,
                    }),
                    icon: claimText,
                    offset: new Vector2(-2, -2),
                    offsetWhenPressed: 2, // push pixels down when onButtonDown
                    iconAlpha: 0.5,
                    onClick: function () {
                        if (daySettings.getGift) {
                            SaveValues.dailyBonusDay += 1;
                            SaveValues.dailyBonusKey = settings.dayKey;
                            daySettings.getGift(daySettings.amount);
                            btn.active = false;
                        }
                    }
                });

                var prize = getPrize();
                prize.attach(new Object({
                    name: 'modulate',
                    timer: 0,
                    update: function (data) {
                        this.timer += data.speed;
                        var sine = Math.sin(this.timer / 32);
                        prize.scale.x = 1 + sine * 0.15;
                        prize.scale.y = 1 + sine * 0.15;
                    }
                }));

                var claimContainer = new Entity({
                    name: 'claimContainer',
                    position: new Vector2(0, 0),
                    components: [
                        btn,
                        prize
                    ]
                });
                return claimContainer;
            };
            var getPrize = function () {
                var prizeBox = new SpriteContainer({
                    width: null,
                    height: 41,
                    imageName: 'gui/dailybonus/' + prizeBoxType,
                    position: new Vector2(0, 0),
                    originRelative: new Vector2(dayType ? 0.5 : 0.44, 0.5),
                });

                var prizeIcon = new Entity({
                    name: 'prizeIcon',
                    position: new Vector2(0, 0),
                    components: []
                });
                switch (daySettings.prize) {
                    case 'coins':
                        var coinText = new Text({
                            position: new Vector2(0, 0),
                            text: daySettings.amount || '0',
                            font: 'omnes',
                            fontSize: 22,
                            fontColor: '#fde000',
                            align: 'left',
                            textBaseline: 'middle',
                            strokeStyle: '#ff592c',
                            ySpacing: 4,
                            lineWidth: 4,
                            maxWidth: 80,
                            maxHeight: 64,
                            linebreaks: true,
                            drawDebug: false,
                            components: []
                        });
                        var coinIcon = new SpriteContainer({
                            width: 20,
                            height: null,
                            imageName: 'gui/icons/coin-small',
                            position: new Vector2(0, 0),
                            originRelative: new Vector2(0.5, 0.5),
                        });
                        prizeIcon.attach(coinText);
                        prizeIcon.attach(coinIcon);
                        prizeIcon.attach(new Object({
                            name: 'spacer',
                            start: function () {
                                var totalSize = coinText.dimension.width + 20 - 3;
                                coinText.position.x = totalSize / -2;
                                coinIcon.position.x = totalSize / 2 - 10;
                            }
                        }));
                        break;
                    case 'ball':
                        prizeIcon.attach(new SpriteContainer({
                            imageName: 'gui/skins/icons/ball-' + ballName,
                            scale: new Vector2(0.2, 0.2),
                            originRelative: new Vector2(0.5, 0.5),
                        }));
                        break;
                    case 'character':
                        prizeIcon.attach(new SpriteContainer({
                            imageName: 'gui/skins/icons/profile-baller-' + characterName,
                            scale: new Vector2(0.2, 0.2),
                            originRelative: new Vector2(0.5, 0.5),
                        }));
                        break;
                }

                if (!dayType) {
                    prizeBox.position.x -= 44;
                    prizeIcon.position.x -= 44;
                }

                var prizeContainer = new Entity({
                    name: 'prizeContainer',
                    family: [''],
                    position: new Vector2(dayType ? 86 : 34, 0),
                    components: [
                        prizeBox,
                        prizeIcon
                    ]
                });
                return prizeContainer;
            };

            switch (dayType) {
                case -1:
                    option = getCheckmark();
                    break;
                case 0:
                    option = getButton();
                    break;
                case 1:
                    option = getPrize();
                    break;
            }

            var dayContainer = new Entity({
                name: 'dayContainter' + daySettings.day.toString(),
                position: daySettings.position,
                components: [
                    box,
                    day,
                    option
                ]
            });
            contentContainer.attach(dayContainer);
        };

        var show = function () {
            fill.scale.x = 1 / 0.001;
            fill.scale.y = 1 / 0.001;
            entity.scale.x = 0.001;
            entity.scale.y = 0.001;
            new Tween({
                from: 0.001,
                to: 1,
                in: 40,
                delay: 0,
                ease: 'elastic',
                decay: 6,
                oscillations: 1,
                onUpdate: function (v, t) {
                    if (v > 0) {
                        fill.scale.x = 1 / v;
                        fill.scale.y = 1 / v;
                    }
                    entity.scale.x = v;
                    entity.scale.y = v;
                },
                onComplete: function () {

                }
            });
        };

        var hide = function () {
            new Tween({
                from: 1,
                to: 0,
                in: 12,
                delay: 20,
                ease: 'easeInQuad',
                onUpdate: function (v, t) {
                    fill.alpha = v;
                    if (v > 0) {
                        fill.scale.x = 1 / v;
                        fill.scale.y = 1 / v;
                    }
                    entity.scale.x = v;
                    entity.scale.y = v;
                },
                onComplete: function () {
                    if (settings.onComplete) {
                        settings.onComplete();
                    }
                    entity.removeSelf();
                }
            });
        };

        var behavior = new Object({
            name: 'behavior',
            start: function (data) {
                if (animate) {
                    show();
                }

                buttonClose.visible = false;

                var dayPosition = -175;
                var dayBoxOffset = 58;

                var getCoins = function (amount) {
                    var coinPopup = new GotCoinsPopup({
                        amount: amount, // note: this amount will be added to SaveValues immediately
                        onComplete: function () {
                            hide();
                        }
                    });
                    Bento.objects.attach(coinPopup);
                };

                addDay({
                    day: 0,
                    position: new Vector2(0, dayPosition),
                    prize: 'coins',
                    amount: 50,
                    getGift: getCoins,
                });

                dayPosition += dayBoxOffset;
                addDay({
                    day: 1,
                    position: new Vector2(0, dayPosition),
                    prize: 'coins',
                    amount: 50,
                    getGift: getCoins
                });

                dayPosition += dayBoxOffset;
                addDay({
                    day: 2,
                    position: new Vector2(0, dayPosition),
                    prize: 'ball',
                    getGift: function () {
                        // save
                        if (unlockedBalls.indexOf(ballName) < 0) {
                            unlockedBalls.push(ballName);
                            SaveValues.unlockedBalls = unlockedBalls;
                        }
                        var receivePopup = new ReceivePopup({
                            skinName: ballName,
                            mesh: 'balls/ball-' + ballName + '.gltf',
                            texture: 'textures/balls/ball-' + ballName,
                            onDone: function () {
                                // select it immediately
                                SaveValues.currentBall = ballName;
                                EventSystem.fire('refreshBallSkin');
                            },
                            onComplete: function () {
                                hide();
                            },
                        });
                        Bento.objects.attach(receivePopup);
                    }
                });

                dayPosition += dayBoxOffset;
                addDay({
                    day: 3,
                    position: new Vector2(0, dayPosition),
                    prize: 'coins',
                    amount: 50,
                    getGift: getCoins
                });

                dayPosition += dayBoxOffset;
                addDay({
                    day: 4,
                    position: new Vector2(0, dayPosition),
                    prize: 'coins',
                    amount: 75,
                    getGift: getCoins
                });

                dayPosition += dayBoxOffset;
                addDay({
                    day: 5,
                    position: new Vector2(0, dayPosition),
                    prize: 'coins',
                    amount: 100,
                    getGift: getCoins
                });

                dayPosition += dayBoxOffset;
                addDay({
                    day: 6,
                    position: new Vector2(0, dayPosition),
                    prize: 'character',
                    getGift: function () {
                        // save
                        if (unlockedPlayers.indexOf(characterName) < 0) {
                            unlockedPlayers.push(characterName);
                            SaveValues.unlockedPlayers = unlockedPlayers;
                        }
                        var receivePopup = new ReceivePopup({
                            skinName: characterName,
                            mesh: 'players/baller-' + characterName + '.gltf',
                            texture: 'textures/players/tex-baller-' + characterName,
                            offset: 80,
                            onDone: function () {
                                // select it immediately
                                SaveValues.currentPlayer = characterName;
                                EventSystem.fire('refreshPlayerSkin');
                            },
                            onComplete: function () {
                                hide();
                            },
                        });
                        Bento.objects.attach(receivePopup);
                    }
                });
            },
            destroy: function (data) {},
            update: function (data) {
                if (hapticCooldown) {
                    hapticCooldown -= 1;
                }
            },
            draw: function (data) {}
        });
        var modal = new Modal({});
        var entity = new Entity({
            z: Globals.Layers.POPUPS,
            name: 'skinSelectPopup',
            components: [
                new Anchor({
                    relative: new Vector2(0.5, 0.5),
                    offset: new Vector2(0, 0)
                }),
                modal,
                fill,
                backBoard,
                topBanner,
                contentContainer,
                buttonClose,
                behavior
            ]
        });
        return entity;
    };
    DailyGiftPopup.shouldAnimate = false;
    return DailyGiftPopup;
});