/**
 * Popup asking to double your coins
 * @moduleName DoubleCoinsPopup
 * @snippet DoubleCoinsPopup|Constructor
DoubleCoinsPopup({})
 */
bento.define('gui/popups/doublecoinspopup', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/utils',
    'bento/tween',
    'bento/components/spritecontainer',
    'bento/components/anchor',
    'onigiri/onigiri',
    'onigiri/sprite3d',
    'components/targetsize',
    'components/materialswapper',
    'globals',
    'bento/gui/text',
    'modules/localization',
    'gui/iconbutton',
    'gui/hdbuttonsprite',
    'bento/eventsystem',
    'modules/watchad',
    'bento/components/clickable',
    'components/waitingforad',
    'managers/savevalues',
    'gui/widgets/confetti',
    'gui/popups/gotcoinspopup'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    Utils,
    Tween,
    SpriteContainer,
    Anchor,
    Onigiri,
    Sprite3d,
    TargetSize,
    MaterialSwapper,
    Globals,
    Text,
    Localization,
    IconButton,
    HdButtonSprite,
    EventSystem,
    WatchAd,
    Clickable,
    WaitingForAd,
    SaveValues,
    Confetti,
    GotCoinsPopup
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var onComplete = settings.onComplete;
        var didWatchAd = false;
        var multiplier = 5;

        var fill = new Entity({
            name: 'fill',
            components: [
                new SpriteContainer({
                    width: viewport.width + 16,
                    alpha: 0.75,
                    imageName: 'gui/bg-back',
                    originRelative: new Vector2(0.5, 0.5),
                }),
                new Anchor({
                    relative: new Vector2(0.5, 0.5),
                })
            ]
        });
        /**
         * Coin
         */
        var mesh = Onigiri.getMesh('coin.gltf');
        var coinSprite = new Sprite3d({
            object3D: mesh,
        });
        var shine = new SpriteContainer({
            width: 160,
            imageName: 'fx/shine',
        });
        var coin = new Entity({
            name: 'coin',
            components: [
                new Anchor({
                    relative: new Vector2(0.5, 0.45),
                }),
                shine,
                coinSprite,
                new TargetSize({
                    width: 80,
                    mesh: mesh
                }),
                new MaterialSwapper({
                    id: 'bigCoin', // unique id for refcounting
                    type: 'MeshPhongMaterial',
                    deepSwap: true,
                    object3D: mesh,
                    materialSettings: {
                        map: Onigiri.getTexture('textures/coin', {
                            wrapS: THREE.RepeatWrapping,
                            wrapT: THREE.RepeatWrapping
                        }),
                        color: 0xffff21,
                        specular: 0xffff21,
                        shininess: 50,
                        reflectivity: 0.3,
                    }
                }),
                new Object({
                    name: 'behavior',
                    start: function (data) {},
                    destroy: function (data) {},
                    update: function (data) {
                        shine.rotation += 0.025;
                        mesh.rotation.y += 0.025;
                    }
                })
            ]
        });
        var collectedText = new Text({
            position: new Vector2(0, 0),
            text: '+' + Globals.coinsCollected,
            font: 'omnes',
            fontSize: 32,
            fontColor: Globals.Colors.coin,
            align: 'center',
            textBaseline: 'middle',
            strokeStyle: Globals.Colors.coinOutline,
            lineWidth: 4,
            linebreaks: false,
            drawDebug: false,
            components: [
                new Anchor({
                    relative: new Vector2(0.5, 0.45),
                    offset: new Vector2(0, 40 + 24)
                })
            ]
        });

        /**
         * Button
         */
        var openText = new Text({
            position: new Vector2(16, -2 + 16),
            text: Localization.getText('btn_claim_x_amount').replace('{MULTIPLE}', multiplier),
            // text: 'x' + multiplier,
            font: 'omnes',
            fontSize: 24,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            antiAlias: true, // Cocoon only
            maxWidth: 64,
            lineWidth: 2,
            strokeStyle: Globals.Colors.coinOutline,
            linebreaks: false,
            drawDebug: false,
            components: []
        });
        var icon = new Entity({
            name: 'icon',
            components: [
                new SpriteContainer({
                    width: 64,
                    imageName: 'gui/icons/triple-coin',
                    position: new Vector2(16, -16)
                }),
                openText,
            ]
        });
        var button = new IconButton({
            name: 'button',
            position: new Vector2(0, 160 - 24),
            sprite: HdButtonSprite({
                imageName: 'gui/buttons/orange-rv',
                width: 144,
            }),
            // icon options
            icon: icon,

            // offset of icon
            offset: new Vector2(0, 0),
            offsetWhenPressed: 2, // push pixels down when onButtonDown

            onClick: function () {
                EventSystem.fire('GameAnalytics-addDesignEvent', {
                    eventId: 'UI:doubleCoinsPopup:claimMultiplied'
                });
                if (WatchAd.canShowRewarded() || Bento.isDev()) {
                    EventSystem.fire('GameAnalytics-addDesignEvent', {
                        eventId: 'Ads:rv:bonusMultiplyCoins'
                    });
                    button.active = false;
                    WatchAd.showRewarded({
                        tag: 'bonusMultiplyCoins',
                        onReward: function () {
                            didWatchAd = true;
                            EventSystem.fire('GameAnalytics-addDesignEvent', {
                                eventId: 'Ads:rv:bonusMultiplyCoinsComplete'
                            });

                            // done: just remove everything
                            coin.removeSelf();
                            collectedText.removeSelf();
                            button.removeSelf();
                            loseItText.removeSelf();
                            entity.fadeOut(multiplier);

                            new Confetti({
                                z: entity.z
                            });
                        },
                        onComplete: function (error) {
                            // Video completed (reward state unknown)
                            button.active = true;
                        }
                    });
                }
            },
            components: [
                new WaitingForAd({
                    offset: new Vector2(12, 0)
                })
            ]
        });
        var loseItText = new Text({
            position: new Vector2(0, 64),
            text: Localization.getText('btn_claim'),
            font: 'omnes',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            linebreaks: false,
            drawDebug: false,
            boundingBox: new Rectangle(-64, -24, 128, 48),
            components: [
                new Clickable({
                    onClick: function (data) {
                        coin.removeSelf();
                        collectedText.removeSelf();
                        button.removeSelf();
                        loseItText.removeSelf();

                        entity.fadeOut(1);

                        EventSystem.fire('GameAnalytics-addDesignEvent', {
                            eventId: 'UI:doubleCoinsPopup:claimNormal'
                        });
                    }
                })
            ]
        });
        // buttons container
        var container = new Entity({
            name: 'container',
            scale: new Vector2(1.25, 1.25),
            components: [
                button,
                new Anchor({
                    relative: new Vector2(0.5, 0.5),
                    offset: new Vector2(0, 16)
                }),
            ]
        });
        var behavior = new Object({
            name: 'behavior',
            start: function (data) {

                new Tween({
                    from: 0,
                    to: 1,
                    in: 30,
                    ease: 'elastic',
                    decay: 5,
                    oscillations: 1,
                    onUpdate: function (v, t) {
                        coin.scale.x = v;
                        coin.scale.y = v;
                    }
                });

                new Tween({
                    from: 0,
                    to: 1,
                    in: 30,
                    ease: 'elastic',
                    decay: 8,
                    oscillations: 1,
                    delay: 5,
                    applyOnDelay: true,
                    onUpdate: function (v, t) {
                        collectedText.scale.x = v;
                        collectedText.scale.y = v;
                    }
                });
                new Tween({
                    from: 0,
                    to: button.scale.x,
                    in: 30,
                    ease: 'elastic',
                    decay: 5,
                    oscillations: 1,
                    delay: 10,
                    applyOnDelay: true,
                    onUpdate: function (v, t) {
                        button.scale.x = v;
                        button.scale.y = v;
                    }
                });

                new Tween({
                    from: 0,
                    to: 1,
                    in: 30,
                    ease: 'linear',
                    delay: 180,
                    onStart: function () {
                        button.attach(loseItText);
                    },
                    onUpdate: function (v, t) {
                        loseItText.alpha = v;
                    }
                });
            },
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        });
        var entity = new Entity({
            name: 'doubleCoinsPopup',
            components: [
                fill,
                coin,
                collectedText,
                behavior,
                container
            ]
        }).extend({
            fadeOut: function (multiplication) {
                console.log('fade out');
                // show coins popup and remove popup
                Bento.objects.attach(new GotCoinsPopup({
                    amount: Globals.coinsCollected * multiplication, // note: this amount will be added to SaveValues immediately
                    onComplete: function () {
                        entity.removeSelf();
                        if (onComplete) {
                            onComplete();
                        }
                    }
                }));
                Globals.coinsCollected = 0;

            }
        });
        return entity;
    };
});