/**
 * Fail pop
 * @moduleName FailPopup
 * @snippet FailPopup|Constructor
FailPopup({})
 */
bento.define('gui/popups/failpopup', [
    'bento',
    'bento/utils',
    'bento/entity',
    'bento/math/vector2',
    'globals',
    'bento/components/fill',
    'gui/widgets/confetti',
    'bento/components/anchor',
    'bento/tween',
    'bento/gui/text',
    'bento/eventsystem',
    'managers/savevalues',
    'gui/widgets/fade',
    'bento/components/spritecontainer',
    'modules/localization'
], function (
    Bento,
    Utils,
    Entity,
    Vector2,
    Globals,
    Fill,
    Confetti,
    Anchor,
    Tween,
    Text,
    EventSystem,
    SaveValues,
    Fade,
    SpriteContainer,
    Localization
) {
    'use strict';
    var FailPopup = function (settings) {
        var viewport = Bento.getViewport();
        var failFastMultiplier = Bento.saveState.load('cheat-failFast', false) ? 0.5 : 1;
        var onComplete = settings.onComplete || function () {};
        var fill = new Entity({
            name: 'fill',
            components: [
                // new Fill({
                //     name: 'fill',
                //     dimension: viewport.getSize().grow(16),
                //     color: [0, 0, 0, 0.5], // [1, 1, 1, 1] is pure white
                //     origin: new Vector2(0, 8)
                // })
                new SpriteContainer({
                    width: viewport.width + 16,
                    alpha: 0.5,
                    imageName: 'gui/bg-dark',
                    originRelative: new Vector2(0.5, 0.5),
                }),
                new Anchor({
                    relative: new Vector2(0.5, 0.5),
                })
            ]
        });
        var behavior = new Object({
            name: 'behavior',
            start: function (data) {
                new Tween({
                    from: 0,
                    to: 1,
                    in: 20 * failFastMultiplier,
                    onUpdate: function (v, t) {
                        failText.alpha = v;
                    }
                });
                new Tween({
                    from: 6,
                    to: 3,
                    in: 40 * failFastMultiplier,
                    delay: 0,
                    applyOnDelay: 0,
                    ease: 'quadratic',
                    onStart: function () {
                        entity.attach(failText);
                    },
                    onUpdate: function (v, t) {
                        failText.scale.x = v;
                        failText.scale.y = v;
                    },
                    onComplete: function () {
                        new Tween({
                            from: 3,
                            to: 0,
                            in: 30 * failFastMultiplier,
                            onUpdate: function (v, t) {
                                failText.transform.x = Utils.getRandomRangeFloat(-v, v);
                                failText.transform.y = Utils.getRandomRangeFloat(-v, v);
                            },
                            onComplete: function () {
                                failText.transform.x = 0;
                                failText.transform.y = 0;

                                Utils.timeout(60, function () {
                                    if (entity.isAdded) {
                                        onComplete();
                                    }
                                });
                            }
                        });
                    }
                });
            }
        });
        var failText = new Text({
            text: Localization.getText('fail'),
            font: 'capture-fail',
            fontSize: 32,
            fontColor: '#ff0000',
            align: 'center',
            textBaseline: 'middle',
            lineWidth: 2, // set to add an outline
            strokeStyle: '#000000',
            margin: new Vector2(16, 16),
            rotation: -Math.PI / 12,
            maxWidth: 96,
            linebreaks: false,
            components: [
                new Anchor({
                    relative: new Vector2(0.5, 0.33),
                    offset: new Vector2(0, 0),
                    onResize: function (isPortrait) {}
                })
            ]
        });
        var entity = new Entity({
            z: Globals.Layers.POPUPS,
            name: 'failPopup',
            components: [
                fill,
                behavior
            ]
        });

        return entity;
    };
    return FailPopup;
});