/**
 * Popup that asks user to open a free gift (for an ad)
 * @moduleName FreeGiftPopup
 * @snippet FreeGiftPopup|Constructor
FreeGiftPopup({
    onComplete: function (didWatchAd) {}
})
 */
bento.define('gui/popups/freegiftpopup', [
    'bento',
    'bento/utils',
    'bento/entity',
    'globals',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/spritecontainer',
    'bento/components/anchor',
    'bento/components/fill',
    'bento/gui/text',
    'bento/tween',
    'onigiri/sprite3d',
    'onigiri/onigiri',
    'components/targetsize',
    'components/materialswapper',
    'gui/iconbutton',
    'gui/hdbuttonsprite',
    'bento/components/clickable',
    'bento/eventsystem',
    'modules/watchad',
    'gui/widgets/confetti',
    'managers/savevalues',
    'modules/localization',
    'bento/components/modal',
    'bento/gui/centeredlist',
    'gui/popups/gotcoinspopup',
    'modules/abtest',
    'components/waitingforad'
], function (
    Bento,
    Utils,
    Entity,
    Globals,
    Vector2,
    Rectangle,
    SpriteContainer,
    Anchor,
    Fill,
    Text,
    Tween,
    Sprite3d,
    Onigiri,
    TargetSize,
    MaterialSwapper,
    IconButton,
    HdButtonSprite,
    Clickable,
    EventSystem,
    WatchAd,
    Confetti,
    SaveValues,
    Localization,
    Modal,
    CenteredList,
    GotCoinsPopup,
    ABTest,
    WaitingForAd
) {
    'use strict';
    var availableBalls = Globals.balls;
    var firstBalls = ['americanfootball', 'golf', 'soccer', 'tennisball'];
    var FreeGiftPopup = function (settings) {
        var viewport = Bento.getViewport();
        var onComplete = settings.onComplete;
        var ballKey = (function () {
            // randomize skin
            var skin = 'default';
            var unlockedBalls = SaveValues.unlockedBalls;
            var skins = [].concat(unlockedBalls.length <= 2 ? firstBalls : availableBalls);
            var removeBall = function (ball) {
                var i = skins.indexOf(ball);
                if (i > -1) {
                    skins.splice(i, 1);
                }
            };
            removeBall(SaveValues.dailyBonusBall);
            while (unlockedBalls.indexOf(skin) >= 0 && skins.length) {
                skin = Utils.pickRandomFrom(skins);
                removeBall(skin);
            }
            return skin;
        })();

        var didWatchAd = false;
        var oldGift = ABTest.getOption('B104_FreeGift') === 'old';
        /**
         * Components
         */
        var fill = new Entity({
            name: 'fill',
            components: [
                // new Fill({
                //     name: 'fill',
                //     dimension: viewport.getSize().grow(16),
                //     color: [0, 0, 0, 0.5], // [1, 1, 1, 1] is pure white
                //     origin: new Vector2(0, 8)
                // })
                new SpriteContainer({
                    width: viewport.width + 16,
                    imageName: 'gui/bg-back',
                    alpha: 0.95,
                    originRelative: new Vector2(0.5, 0.5),
                }),
                new Anchor({
                    relative: new Vector2(0.5, 0.5),
                })
            ]
        });
        var behavior = new Object({
            name: 'behavior',
            start: function (data) {
                // already unlocked all balls?
                console.log('Next ball: ' + ballKey);

                new Tween({
                    from: 0,
                    to: 1,
                    in: 60,
                    ease: 'elastic',
                    decay: 5,
                    oscillations: 2,
                    onUpdate: function (v, t) {
                        topBanner.scale.x = v;
                        topBanner.scale.y = v;
                        list.scale.x = v;
                        list.scale.y = v;

                    }
                });

                new Tween({
                    from: 0,
                    to: 1,
                    in: 60,
                    ease: 'elastic',
                    decay: 5,
                    oscillations: 2,
                    delay: 10,
                    applyOnDelay: true,
                    onUpdate: function (v, t) {
                        gift.scale.x = v;
                        gift.scale.y = v;
                    }
                });

                new Tween({
                    from: 0,
                    to: 1,
                    in: 30,
                    ease: 'linear',
                    delay: 180,
                    onStart: function () {
                        button.attach(loseItText);
                    },
                    onUpdate: function (v, t) {
                        loseItText.alpha = v;
                    }
                });

                list.addItem(new Entity({
                    name: 'chance1',
                    rotation: -Math.PI / 20,
                    position: new Vector2(0, 4),
                    components: [
                        new SpriteContainer({
                            width: 56,
                            imageName: 'gui/chests/prize-bubble',
                        }),
                        new SpriteContainer({
                            width: 32,
                            position: new Vector2(0, -10),
                            imageName: 'gui/icons/big-coin',
                        }),
                        new Text({
                            position: new Vector2(0, 8),
                            text: '+75',
                            font: 'omnes',
                            fontSize: 16,
                            fontColor: Globals.Colors.coin,
                            align: 'center',
                            textBaseline: 'middle',
                            lineWidth: [2], // set to add an outline
                            strokeStyle: [Globals.Colors.coinOutline],
                            strokeOffset: [new Vector2(0, 0)],
                            components: []
                        })
                        // new Text({
                        //     position: new Vector2(0, 8),
                        //     text: '50%',
                        //     font: 'omnes',
                        //     fontSize: 16,
                        //     fontColor: '#ffffff',
                        //     align: 'center',
                        //     textBaseline: 'middle',
                        //     lineWidth: [3, 1], // set to add an outline
                        //     strokeStyle: [Globals.Colors.darkblue, Globals.Colors.lightblue],
                        //     strokeOffset: [new Vector2(0, 1.5), new Vector2(0, 0)],
                        //     components: []
                        // })
                    ]
                }));
                list.addItem(new Entity({
                    name: 'chance2',
                    components: [
                        new SpriteContainer({
                            width: 56,
                            imageName: 'gui/chests/prize-bubble',
                        }),
                        new SpriteContainer({
                            width: 48,
                            position: new Vector2(0, -13),
                            imageName: 'gui/icons/triple-coin',
                        }),
                        new Text({
                            position: new Vector2(0, 8),
                            text: '+200',
                            font: 'omnes',
                            fontSize: 16,
                            fontColor: Globals.Colors.coin,
                            align: 'center',
                            textBaseline: 'middle',
                            lineWidth: [2], // set to add an outline
                            strokeStyle: [Globals.Colors.coinOutline],
                            strokeOffset: [new Vector2(0, 0)],
                            components: []
                        })
                        // new Text({
                        //     position: new Vector2(0, 8),
                        //     text: '30%',
                        //     font: 'omnes',
                        //     fontSize: 16,
                        //     fontColor: '#ffffff',
                        //     align: 'center',
                        //     textBaseline: 'middle',
                        //     lineWidth: [3, 1], // set to add an outline
                        //     strokeStyle: [Globals.Colors.darkblue, Globals.Colors.lightblue],
                        //     strokeOffset: [new Vector2(0, 1.5), new Vector2(0, 0)],
                        //     components: []
                        // })
                    ]
                }));
                list.addItem(new Entity({
                    name: 'chance3',
                    rotation: Math.PI / 20,
                    position: new Vector2(0, 4),
                    components: [
                        new SpriteContainer({
                            width: 56,
                            imageName: 'gui/chests/prize-bubble',
                        }),
                        new SpriteContainer({
                            width: 40,
                            position: new Vector2(0, -8),
                            imageName: 'gui/skins/icons/ball-' + ballKey,
                        }),
                        // new Text({
                        //     position: new Vector2(0, 8),
                        //     text: '20%',
                        //     font: 'omnes',
                        //     fontSize: 16,
                        //     fontColor: '#ffffff',
                        //     align: 'center',
                        //     textBaseline: 'middle',
                        //     lineWidth: [3, 1], // set to add an outline
                        //     strokeStyle: [Globals.Colors.darkblue, Globals.Colors.lightblue],
                        //     strokeOffset: [new Vector2(0, 1.5), new Vector2(0, 0)],
                        //     components: []
                        // })
                    ]
                }));
            },
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        });
        /**
         * Top banner
         */
        var bannerText = new Text({
            text: Localization.getText('freeGift'),
            position: new Vector2(0, -16),
            font: 'omnes',
            fontSize: 32,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 200,
            linebreaks: false,
            drawDebug: false,
            components: []
        });
        var topBanner = new Entity({
            name: 'topBanner',
            position: new Vector2(0, -144),
            visible: oldGift,
            components: [
                new SpriteContainer({
                    width: 256,
                    imageName: 'gui/popup/banner',
                }),
                bannerText
            ]
        });
        /**
         * Gift
         */
        var innerMesh = Onigiri.getMesh('present.gltf');
        var giftMesh = new THREE.Object3D().add(innerMesh);
        var giftSprite = new Sprite3d({
            object3D: giftMesh,
        });
        var shine = new SpriteContainer({
            width: 256,
            imageName: 'fx/shine',
        });
        var list = new CenteredList({
            name: 'list',
            visible: !oldGift,
            spacing: 72, // in pixels
            useItemDimensions: false, // note: uses last item's dimension
            direction: 'x',
            position: new Vector2(0, -128),
            items: [

            ]
        });
        var gift = new Entity({
            name: 'gift',
            components: [
                shine,
                giftSprite,
                new TargetSize({
                    width: 80,
                    mesh: giftSprite.object3D
                }),
                new Object({
                    name: 'behavior',
                    start: function () {
                        // apply the flatten manually (clashes with TargetSize)
                        // giftMesh.scale.z = 0.001;
                        giftMesh.position.z = 10;
                        innerMesh.rotation.x = Math.PI / 8;
                    },
                    update: function (data) {
                        innerMesh.rotation.y += 0.02;
                        shine.rotation += 0.01;
                    },
                }),
            ]
        });
        /**
         * Button
         */
        var openText = new Text({
            position: new Vector2(12, -2),
            text: Localization.getText('get'),
            font: 'omnes',
            fontSize: 32,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            antiAlias: true, // Cocoon only
            maxWidth: 80,
            lineWidth: 2,
            strokeStyle: Globals.Colors.coinOutline,
            linebreaks: false,
            drawDebug: false,
            components: []
        });
        var icon = new Entity({
            name: 'icon',
            components: [
                openText,
            ]
        });
        var button = new IconButton({
            position: new Vector2(0, 160 - 24),
            sprite: HdButtonSprite({
                imageName: 'gui/buttons/orange-rv',
                width: 144,
            }),
            // icon options
            icon: icon,

            // offset of icon
            offset: new Vector2(0, 0),
            offsetWhenPressed: 2, // push pixels down when onButtonDown

            onClick: function () {
                EventSystem.fire('GameAnalytics-addDesignEvent', {
                    eventId: 'UI:homeScreen:openGiftButton'
                });
                if (WatchAd.canShowRewarded() || Bento.isDev()) {
                    EventSystem.fire('GameAnalytics-addDesignEvent', {
                        eventId: 'Ads:rv:freeGift'
                    });
                    button.active = false;
                    WatchAd.showRewarded({
                        tag: oldGift ? 'freeGift' : 'randomFreeGift',
                        onReward: function () {
                            didWatchAd = true;
                            button.removeSelf();
                            loseItText.attach(new Object({
                                name: 'suicide',
                                start: function (data) {
                                    loseItText.removeSelf();
                                },
                            }));
                            entity.showReward();
                            EventSystem.fire('GameAnalytics-addDesignEvent', {
                                eventId: 'Ads:rv:freeGiftComplete'
                            });
                        },
                        onComplete: function (error) {
                            // Video completed (reward state unknown)
                            button.active = true;
                        }
                    });
                }
            },
            components: [
                new WaitingForAd({
                    offset: new Vector2(12, 0)
                })
            ]
        });
        var loseItText = new Text({
            position: new Vector2(0, 64),
            text: Localization.getText('loseIt'),
            font: 'omnes',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            linebreaks: false,
            drawDebug: false,
            boundingBox: new Rectangle(-64, -24, 128, 48),
            components: [
                new Clickable({
                    onClick: function (data) {
                        button.removeSelf();
                        loseItText.removeSelf();
                        entity.fadeOut();

                        EventSystem.fire('GameAnalytics-addDesignEvent', {
                            eventId: 'UI:homeScreen:discardGiftButton'
                        });
                    }
                })
            ]
        });
        /**
         * Ball
         */
        var ballMesh = new THREE.Object3D();
        var ballSprite = new Sprite3d({
            object3D: ballMesh,
        });
        var ball = new Entity({
            name: 'ball',
            components: [
                ballSprite,
                new Object({
                    name: 'behavior',
                    start: function () {
                        ballMesh.position.z = 10;
                    },
                    update: function (data) {
                        ballMesh.rotation.y += 0.02;
                    },
                })
            ]
        });
        var ballText = new Text({
            text: Localization.getText('newBall'),
            position: new Vector2(0, 96),
            font: 'omnes',
            fontSize: 24,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 200,
            linebreaks: false,
            drawDebug: false,
            components: []
        });
        /**
         * Entity
         */
        var container = new Entity({
            name: 'container',
            scale: new Vector2(1.25, 1.25),
            components: [
                new Anchor({
                    relative: new Vector2(0.5, 0.5),
                    offset: new Vector2(0, 16)
                }),
                topBanner,
                list,
                gift,
                ball,
                button,
            ]
        });
        var entity = new Entity({
            z: Globals.Layers.POPUPS,
            name: 'freeGiftPopup',
            components: [
                new Modal({}),
                behavior,
                fill,
                container
            ]
        }).extend({
            fadeOut: function () {

                new Tween({
                    from: 1,
                    to: 0,
                    in: 30,
                    ease: 'easeInBack',
                    onUpdate: function (v, t) {
                        topBanner.scale.x = v;
                        topBanner.scale.y = v;
                        list.scale.x = v;
                        list.scale.y = v;
                    }
                });
                new Tween({
                    from: 1,
                    to: 0,
                    in: 30,
                    delay: 10,
                    ease: 'easeInBack',
                    onUpdate: function (v, t) {
                        gift.scale.x = v;
                        gift.scale.y = v;
                        ball.scale.x = v;
                        ball.scale.y = v;
                    },
                    onComplete: function () {
                        entity.removeSelf();
                        if (onComplete) {
                            onComplete(didWatchAd);


                            EventSystem.fire('GameAnalytics-addDesignEvent', {
                                eventId: oldGift ? 'conversion:freeGift' : 'conversion:freeGiftRandom',
                                value: didWatchAd ? 1 : 0
                            });
                        }
                    }
                });

                new Tween({
                    from: 1,
                    to: 0,
                    in: 10,
                    delay: 20,
                    ease: 'easeInBack',
                    onUpdate: function (v, t) {
                        ballText.scale.y = v;
                        ballText.scale.x = v;
                    }
                });
            },
            showReward: function () {
                // change text
                bannerText.setText(Localization.getText('youGot'));
                new Tween({
                    from: gift.scale.x,
                    to: 0,
                    in: 30,
                    ease: 'easeInBack',
                    onUpdate: function (v, t) {
                        gift.scale.x = v;
                        gift.scale.y = v;
                    },
                    onComplete: function () {
                        new Confetti({
                            z: entity.z,
                            life: 1.5
                        });
                        gift.removeSelf();

                        if (oldGift) {
                            getBall();
                        } else {
                            if (SaveValues.unlockedBalls.length === 1) {
                                // first one is always a ball
                                list.get(0).visible = false;
                                list.get(1).visible = false;
                                getBall();
                            } else {
                                Utils.weightedRandom([{
                                    weight: 1,
                                    fn: function () {
                                        list.get(1).visible = false;
                                        list.get(2).visible = false;
                                        getCoins(75);
                                    }
                                }, {
                                    weight: 1,
                                    fn: function () {
                                        list.get(0).visible = false;
                                        list.get(2).visible = false;
                                        getCoins(200);
                                    }
                                }, {
                                    weight: 1,
                                    fn: function () {
                                        list.get(0).visible = false;
                                        list.get(1).visible = false;
                                        getBall();
                                    }
                                }]).fn();
                            }
                        }

                        Utils.timeout(180, function () {
                            entity.fadeOut();
                        });
                    }
                });
            }
        });
        var getBall = function () {
            // save
            var unlockedBalls = SaveValues.unlockedBalls;
            if (unlockedBalls.indexOf(ballKey) < 0) {
                unlockedBalls.push(ballKey);
                SaveValues.unlockedBalls = unlockedBalls;
            }

            // set the ball
            SaveValues.currentBall = ballKey;

            // show new ball
            var newBallMesh = Onigiri.getMesh('balls/ball-' + ballKey + '.gltf');
            ballMesh.add(newBallMesh);
            ball.attach(new TargetSize({
                width: 96,
                mesh: newBallMesh
            }));
            // apply proper material
            ball.attach(new MaterialSwapper({
                id: 'ballGift-' + ballKey, // unique id for refcounting
                type: 'MeshBasicMaterial',
                deepSwap: true,
                object3D: newBallMesh,
                materialSettings: {
                    map: Onigiri.getTexture('textures/balls/ball-' + ballKey, {
                        wrapS: THREE.RepeatWrapping,
                        wrapT: THREE.RepeatWrapping
                    })
                }
            }));
            container.attach(ballText);

            new Tween({
                from: 0,
                to: 1,
                in: 60,
                ease: 'elastic',
                decay: 5,
                oscillations: 3,
                onUpdate: function (v, t) {
                    ball.scale.x = v;
                    ball.scale.y = v;
                }
            });
        };
        var getCoins = function (amount) {
            Bento.objects.attach(new GotCoinsPopup({
                ignoreModal: true,
                amount: amount, // note: this amount will be added to SaveValues immediately
                onComplete: function () {}
            }));
        };

        return entity;
    };
    return FreeGiftPopup;
});