/**
 * Popup to show the user got coins
 * @moduleName GotCoinsPopup
 * @snippet GotCoinsPopup|Constructor
GotCoinsPopup({
    amount: ${1:0}, // note: this amount will be added to SaveValues immediately
    onComplete: function () {}
})
 */
bento.define('gui/popups/gotcoinspopup', [
    'bento',
    'bento/utils',
    'bento/entity',
    'bento/math/rectangle',
    'bento/math/vector2',
    'bento/components/anchor',
    'bento/components/modal',
    'globals',
    'bento/components/spritecontainer',
    'gui/widgets/coinrain',
    'bento/tween',
    'bento/gui/text',
    'managers/savevalues',
    'bento/eventsystem'
], function (
    Bento,
    Utils,
    Entity,
    Rectangle,
    Vector2,
    Anchor,
    Modal,
    Globals,
    SpriteContainer,
    CoinRain,
    Tween,
    Text,
    SaveValues,
    EventSystem
) {
    'use strict';
    var GotCoinsPopup = function (settings) {
        var savedCoins = SaveValues.coins;
        var amount = settings.amount || 0;
        var onComplete = settings.onComplete;
        var viewport = Bento.getViewport();
        var fill = new Entity({
            name: 'fill',
            components: [
                new SpriteContainer({
                    width: viewport.width + 16,
                    imageName: 'gui/bg-back',
                    alpha: 0.5,
                    originRelative: new Vector2(0.5, 0.5),
                }),
                new Anchor({
                    relative: new Vector2(0.5, 0.5),
                })
            ]
        });
        var coinText = new Text({
            position: new Vector2(8, -1),
            text: '+' + amount,
            font: 'omnes',
            fontSize: 32,
            fontColor: Globals.Colors.coin,
            strokeStyle: Globals.Colors.coinOutline,
            lineWidth: 3,
            align: 'right',
            textBaseline: 'middle',
            maxWidth: 64,
            linebreaks: false,
            drawDebug: false,
            components: []
        });
        var shine = new SpriteContainer({
            height: 200,
            imageName: 'fx/shine',
        });
        var box = new Entity({
            name: 'box',
            components: [
                // shine,
                new Anchor({
                    relative: new Vector2(0.5, 0.5),
                    offset: new Vector2(0, 0),
                    onResize: function (isPortrait) {}
                }),
                new SpriteContainer({
                    height: 64,
                    imageName: 'gui/top/currency-box',
                }),
                coinText
            ]
        });
        var behavior = new Object({
            name: 'behavior',
            start: function (data) {
                Bento.objects.get('coinBox', function (coinBox) {
                    rainContainer.attach(new CoinRain({
                        z: coinBox.z - 2,
                        targetPosition: coinBox.position.clone(),
                        onDestroy: function (coin, completion) {
                            coinBox.bounce();
                            EventSystem.fire('fakeCoins', savedCoins + Math.floor(amount * completion));
                        },
                        onComplete: function () {
                            EventSystem.fire('updateCoins');
                        }
                    }));
                });
                new Tween({
                    from: 0,
                    to: shine.scale.x,
                    in: 60,
                    delay: 10,
                    applyOnDelay: true,
                    ease: 'elastic',
                    decay: 5,
                    oscillations: 2,
                    onUpdate: function (v, t) {
                        shine.scale.x = v;
                        shine.scale.y = v;
                    }
                });
                new Tween({
                    from: 0,
                    to: 1,
                    in: 60,
                    ease: 'elastic',
                    decay: 8,
                    oscillations: 4,
                    onStart: function () {},
                    onUpdate: function (v, t) {
                        box.scale.x = v;
                        box.scale.y = v;
                    },
                    onComplete: function () {
                        // tween out
                        new Tween({
                            from: 1,
                            to: 0,
                            in: 30,
                            delay: 30,
                            ease: 'easeInBack',
                            onUpdate: function (v, t) {
                                box.scale.x = v;
                                box.scale.y = v;
                            },
                            onComplete: function () {
                                entity.removeSelf();
                                if (onComplete) {
                                    onComplete();
                                }
                            }
                        });
                    }
                });
            }
        });
        var rainContainer = new Entity({
            name: 'conatainer',
        });
        var entity = new Entity({
            z: settings.z || Globals.Layers.COINS - 1,
            name: 'gotcoinspopup',
            updateWhenPaused: Bento.objects.isPaused(),
            components: [
                settings.ignoreModal ? {
                    name: 'dummy'
                } : new Modal({}),
                fill,
                rainContainer,
                box,
                behavior
            ]
        });

        EventSystem.fire('holdUpdate');
        SaveValues.coins += amount;

        return entity;
    };
    return GotCoinsPopup;
});