/**
 * Ball or character gift from the game
 * @moduleName ReceivePopup
 * @snippet ReceivePopup|Constructor
ReceivePopup({})
 */
bento.define('gui/popups/receivepopup', [
    'bento',
    'bento/utils',
    'bento/gui/text',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/components/anchor',
    'bento/components/spritecontainer',
    'gui/iconbutton',
    'gui/hdbuttonsprite',
    'bento/eventsystem',
    'modules/watchad',
    'managers/savevalues',
    'bento/tween',
    'onigiri/sprite3d',
    'globals',
    'bento/components/clickable',
    'onigiri/onigiri',
    'components/targetsize',
    'components/materialswapper',
    'gui/widgets/confetti',
    'modules/localization',
    'components/waitingforad',
    'bento/components/modal'
], function (
    Bento,
    Utils,
    Text,
    Vector2,
    Rectangle,
    Entity,
    Anchor,
    SpriteContainer,
    IconButton,
    HdButtonSprite,
    EventSystem,
    WatchAd,
    SaveValues,
    Tween,
    Sprite3d,
    Globals,
    Clickable,
    Onigiri,
    TargetSize,
    MaterialSwapper,
    Confetti,
    Localization,
    WaitingForAd,
    Modal
) {
    'use strict';
    var UnlockPopup = function (settings) {
        var viewport = Bento.getViewport();
        var onDone = settings.onDone;
        var onComplete = settings.onComplete;
        var skinKey = settings.skinName || 'default';
        /**
         * Components
         */
        var fill = new Entity({
            name: 'fill',
            components: [
                new SpriteContainer({
                    width: viewport.width + 16,
                    imageName: 'gui/bg-back',
                    alpha: 0.9,
                    originRelative: new Vector2(0.5, 0.5),
                }),
                new Anchor({
                    relative: new Vector2(0.5, 0.5),
                })
            ]
        });
        var behavior = new Object({
            name: 'behavior',
            start: function (data) {
                new Tween({
                    from: 0,
                    to: 1,
                    in: 60,
                    ease: 'elastic',
                    decay: 5,
                    oscillations: 2,
                    onUpdate: function (v, t) {
                        topBanner.scale.x = v;
                        topBanner.scale.y = v;
                    }
                });

                new Tween({
                    from: 0,
                    to: 1,
                    in: 60,
                    ease: 'elastic',
                    decay: 5,
                    oscillations: 2,
                    delay: 10,
                    applyOnDelay: true,
                    onUpdate: function (v, t) {
                        skin.scale.x = v;
                        skin.scale.y = v;
                    }
                });

                Utils.timeout(10, function () {
                    new Confetti({
                        z: entity.z
                    });
                });

                Utils.timeout(140, function () {
                    // showed long enough, fire function and remove popup
                    if (settings.onDone) {
                        settings.onDone();
                    }
                    entity.fadeOut();
                });
            },
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        });
        /**
         * Top banner
         */
        var bannerText = new Text({
            text: Localization.getText('new'),
            position: new Vector2(0, -16),
            font: 'omnes',
            fontSize: 32,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 200,
            linebreaks: false,
            drawDebug: false,
            components: []
        });
        var topBanner = new Entity({
            name: 'topBanner',
            position: new Vector2(0, -138),
            components: [
                new SpriteContainer({
                    width: 256,
                    imageName: 'gui/popup/banner',
                }),
                bannerText
            ]
        });

        /**
         * Player
         */
        var skinMesh = new THREE.Object3D().add(Onigiri.getMesh(settings.mesh));
        var skinSprite = new Sprite3d({
            object3D: skinMesh,
        });
        var rotateSpeed = 0.2;
        var rotateTarget = 0.02;
        var skin = new Entity({
            name: 'skin',
            position: new Vector2(0, settings.offset || 0),
            components: [
                skinSprite,
                new TargetSize({
                    height: 160,
                    mesh: skinMesh
                }),
                // Globals.getPlayerMaterial(playerKey, playerMesh, null),
                new MaterialSwapper({
                    id: 'skinReceive-' + skinKey, // unique id for refcounting
                    type: 'MeshBasicMaterial',
                    deepSwap: true,
                    object3D: skinMesh,
                    materialSettings: {
                        skinning: true,
                        map: Onigiri.getTexture(settings.texture, {
                            wrapS: THREE.RepeatWrapping,
                            wrapT: THREE.RepeatWrapping,
                        })
                    }
                }),
                new Object({
                    name: 'behavior',
                    start: function () {
                        skinMesh.position.z = 10;
                    },
                    update: function (data) {
                        rotateSpeed += (rotateTarget - rotateSpeed) * 0.02 * data.speed;
                        skinMesh.rotation.y += rotateSpeed * data.speed;
                    },
                })
            ]
        });

        var container = new Entity({
            name: 'container',
            scale: new Vector2(1.25, 1.25),
            components: [
                topBanner,
                skin,
                new Anchor({
                    relative: new Vector2(0.5, 0.5),
                    offset: new Vector2(0, 16)
                }),
            ]
        });
        var modal = new Modal({});
        var entity = new Entity({
            z: Globals.Layers.POPUPS,
            name: 'unlockPopup',
            components: [
                modal,
                behavior,
                fill,
                container,
            ]
        }).extend({
            fadeOut: function () {
                new Tween({
                    from: 1,
                    to: 0,
                    in: 30,
                    ease: 'easeInBack',
                    onUpdate: function (v, t) {
                        topBanner.scale.x = v;
                        topBanner.scale.y = v;
                    }
                });
                new Tween({
                    from: 1,
                    to: 0,
                    in: 30,
                    delay: 10,
                    ease: 'easeInBack',
                    onUpdate: function (v, t) {
                        skin.scale.x = v;
                        skin.scale.y = v;
                    },
                    onComplete: function () {
                        entity.removeSelf();
                        if (onComplete) {
                            onComplete();
                        }
                    }
                });
            }
        });
        return entity;
    };
    return UnlockPopup;
});