/**
 * Settings popup
 * @moduleName SettingsPopup
 * @snippet SettingsPopup|Constructor
SettingsPopup({})
 */
bento.define('gui/popups/settingspopup', [
    'bento',
    'bento/utils',
    'bento/entity',
    'bento/math/vector2',
    'bento/math/rectangle',
    'globals',
    'bento/components/modal',
    'bento/components/spritecontainer',
    'bento/components/anchor',
    'gui/iconbutton',
    'gui/hdbuttonsprite',
    'components/backbutton',
    'modules/localization',
    'bento/gui/text',
    'gui/iapoverlay'
], function (
    Bento,
    Utils,
    Entity,
    Vector2,
    Rectangle,
    Globals,
    Modal,
    SpriteContainer,
    Anchor,
    IconButton,
    HdButtonSprite,
    BackButton,
    Localization,
    Text,
    IapOverlay
) {
    'use strict';
    var SettingsPopup = function (settings) {
        var viewport = Bento.getViewport();
        var modal = new Modal({});
        var fill = new Entity({
            name: 'fill',
            components: [
                new SpriteContainer({
                    width: viewport.width + 16,
                    alpha: 0.9,
                    imageName: 'gui/bg-back',
                    originRelative: new Vector2(0.5, 0.5),
                }),
                new Anchor({
                    relative: new Vector2(0.5, 0.5),
                })
            ]
        });
        var playButton = new IconButton({
            sprite: HdButtonSprite({
                imageName: 'gui/buttons/blue',
                width: 128,
            }),
            icon: new Entity({
                name: 'icon',
                components: [
                    new SpriteContainer({
                        width: 32,
                        imageName: 'gui/icons/resume',
                    })
                ]
            }),
            offset: new Vector2(0, 0),
            offsetWhenPressed: 2, // push pixels down when onButtonDown

            // icon transparency when inactive
            iconAlpha: 0.5,

            onClick: function () {
                entity.removeSelf();
            },
            components: [
                new Anchor({
                    relative: new Vector2(0.5, 0.6),
                    offset: new Vector2(0, 0),
                    onResize: function (isPortrait) {}
                }),
                new BackButton({
                    onPressed: function (data) {
                        data.entity.doCallback();
                    }
                })
            ]
        });
        /**
         * This button needs a proper graphic
         */
        var restorePurchaseButton = new IconButton({
            z: 0,
            position: new Vector2(0, 0),
            // TODO: replace with button sprite
            // sprite: HdButtonSprite({
            //     width: 32,
            //     imageName: 'gui/icons/icn-restorepurchase',
            //     frameCountX: 1,
            //     frameCountY: 1,
            // }),
            // TEMP (makes text clickable)
            boundingBox: new Rectangle(-16, -32, 180, 64),

            // icon options
            icon: new Entity({
                name: 'icon',
                components: [
                    new SpriteContainer({
                        width: 32,
                        imageName: 'gui/icons/icn-restorepurchase',
                    }),
                    new Text({
                        position: new Vector2(32, 0),
                        text: Localization.getText('restore'),
                        font: 'omnes',
                        fontSize: 16,
                        fontColor: '#ffffff',
                        align: 'left',
                        textBaseline: 'middle',
                        maxWidth: 160,
                        linebreaks: false,
                        drawDebug: false,
                        components: []
                    })
                ]
            }),

            // offset of icon
            offset: new Vector2(0, 0),
            offsetWhenPressed: 2, // push pixels down when onButtonDown

            // icon transparency when inactive
            iconAlpha: 0.5,

            onClick: function () {
                IapOverlay.restore({
                    z: Globals.Layers.IAP,
                    fontSettings: {
                        font: 'omnes',
                        fontSize: 24
                    },
                })
                // attach working fill
                .attach(new Entity({
                    name: 'fill',
                    components: [
                        new Object({
                            name: 'behavior',
                            start: function (data) {
                                // move self to top
                                data.entity.parent.moveComponentTo(data.entity, 0);
                            }
                        }),
                        new SpriteContainer({
                            width: viewport.width + 16,
                            alpha: 0.7,
                            imageName: 'gui/bg-dark',
                            originRelative: new Vector2(0.5, 0.5),
                        }),
                        new Anchor({
                            relative: new Vector2(0.5, 0.5),
                        })
                    ]
                }));
            },
            components: [
                new Anchor({
                    relative: new Vector2(0.5, 1),
                    offset: new Vector2(-80, -64),
                    onResize: function (isPortrait) {}
                }),
            ]
        });

        /**
         * Main entity
         */
        var entity = new Entity({
            z: Globals.Layers.POPUPS,
            name: 'settings',
            components: [
                modal,
                fill,
                playButton,
//                restorePurchaseButton
            ]
        });
        return entity;
    };
    return SettingsPopup;
});