/**
 * Skin select popup
 * @moduleName SkinSelectPopup
 * @snippet SkinSelectPopup|Constructor
SkinSelectPopup({})
 */
bento.define('gui/popups/skinselectpopup', [
    'bento',
    'bento/utils',
    'bento/entity',
    'bento/math/vector2',
    'bento/math/rectangle',
    'gui/iconbutton',
    'gui/hdbuttonsprite',
    'bento/gui/scrollinglist',
    'bento/components/spritecontainer',
    'bento/components/anchor',
    'bento/components/modal',
    'globals',
    'bento/components/fill',
    'bento/gui/centeredlist',
    'bento/gui/text',
    'bento/components/clickable',
    'managers/savevalues',
    'bento/tween',
    'bento/eventsystem',
    'modules/localization',
    'gui/counters/coincounter',
    'gui/widgets/confetti',
    'modules/tapticengine',
    'modules/watchad',
    'gui/popups/gotcoinspopup',
    'onigiri/sprite3d',
    'components/waitingforad',
    'components/backbutton'
], function (
    Bento,
    Utils,
    Entity,
    Vector2,
    Rectangle,
    IconButton,
    HdButtonSprite,
    ScrollingList,
    SpriteContainer,
    Anchor,
    Modal,
    Globals,
    Fill,
    CenteredList,
    Text,
    Clickable,
    SaveValues,
    Tween,
    EventSystem,
    Localization,
    CoinCounter,
    Confetti,
    TapticEngine,
    WatchAd,
    GotCoinsPopup,
    Sprite3d,
    WaitingForAd,
    BackButton
) {
    'use strict';
    var balls = Globals.balls;
    var players = Globals.players;
    var SkinSelectPopup = function (settings) {
        var viewport = Bento.getViewport();
        var fill = new Entity({
            name: 'fill',
            components: [
                // new Fill({
                //     name: 'fill',
                //     dimension: viewport.getSize().grow(16),
                //     color: [0, 0, 0, 0.5], // [1, 1, 1, 1] is pure white
                //     origin: new Vector2(0, 8)
                // })
                new SpriteContainer({
                    width: viewport.width + 16,
                    alpha: 0.9,
                    imageName: 'gui/bg-back',
                    originRelative: new Vector2(0.5, 0.5),
                }),
                new Anchor({
                    relative: new Vector2(0.5, 0.5),
                })
            ]
        });
        var isAnimating = false;
        var hapticCooldown = 0;
        /**
         * Scrolling list
         */
        var width = 300;
        var height = 300;
        var leftClippingPlane = new THREE.Plane(new THREE.Vector3(1, 0, 0), -(viewport.width / 2 - width / 2));
        var rightClippingPlane = new THREE.Plane(new THREE.Vector3(-1, 0, 0), (viewport.width / 2 + width / 2));
        var dotList = new CenteredList({
            name: 'dotList',
            spacing: 16,
            position: new Vector2(0, 120),
            useItemDimensions: false, // note: uses last item's dimension
            components: []
        });
        var arrowLeft = new Entity({
            name: 'arrowLeft',
            rotation: Math.PI / 2,
            boundingBox: new Rectangle(-8, -8, 16, 16),
            position: new Vector2(0, -16),
            components: [
                new SpriteContainer({
                    width: 16,
                    // position: new Vector2(-16, 152),
                    imageName: 'gui/top/position-marker',
                }),
                new Object({
                    name: 'behavior',
                    update: function (data) {
                        data.entity.position.x = -152 + 2 * Math.sin(data.entity.timer / 10);
                        data.entity.visible = scrollingList.getSelection() !== 0;
                    }
                }),
                new Clickable({
                    onClick: function (data) {
                        console.log('click');
                        if (arrowLeft.visible) {
                            scrollingList.snapTo(scrollingList.getSelection() - 1);
                        }
                    }
                })
            ]
        });
        var arrowRight = new Entity({
            name: 'arrowRight',
            rotation: -Math.PI / 2,
            position: new Vector2(0, -16),
            components: [
                new SpriteContainer({
                    width: 16,
                    // position: new Vector2(16, 152),
                    imageName: 'gui/top/position-marker',
                }),
                new Object({
                    name: 'behavior',
                    update: function (data) {
                        data.entity.position.x = 152 - 2 * Math.sin(data.entity.timer / 10);
                        data.entity.visible = scrollingList.getSelection() !== scrollingList.length - 1;
                    }
                }),
                new Clickable({
                    onClick: function (data) {
                        if (arrowRight.visible) {
                            scrollingList.snapTo(scrollingList.getSelection() + 1);
                        }
                    }
                })
            ]
        });
        var scrollingList = new ScrollingList({
            name: 'scrollingList',
            position: new Vector2(0, 0),
            updateWhenPaused: 0,
            area: new Rectangle(-width / 2, -height / 2, width, height), // area that user can grab
            direction: 'x', // 'x' or 'y'
            spacing: 88, // additional spacing between items
            maxOffset: 0, // additional max scrolling distance
            minOffset: 0, // additional min scrolling distance
            snap: true, // snap items (cancels maxOffset and minOffset)
            snapNextOffset: 16, // distance to scroll to snap to next, in screen pixels. Used with snap:true, best with damping:0
            onSnap: function (item, index) {
                updateButtons();
                dotList.iterate(function (dot, i) {
                    if (i === index) {
                        dot.alpha = 1;
                    } else {
                        dot.alpha = 0.5;
                    }
                });
            }, // called when snap focus changed
            onLoseFocus: function (index) {}, // called when snap focus changed
            onScrollStart: function () {}, // callback when scrolling has started
            snapMinSlideSpeed: 1, // minimum slidespeed before starting to snap
            snapSpeed: 0.3, // speed of snapping
            damping: 0 // speed decrease,
        });
        /**
         * Tabs
         */

        var tabs = new CenteredList({
            spacing: 128, // in pixels
            useItemDimensions: false, // note: uses last item's dimension
            direction: 'x',
            position: new Vector2(0, -184),
            items: [

            ]
        });
        var setTab = function (tabId) {
            tabs.getComponent('tab-' + tabId, function (tab) {
                tab.setTab();
                updateButtons();
                SaveValues.lastTab = tabId;
            });
        };
        var addTab = function (data) {
            var skinsArray = data.skinsArray;
            var iconName = data.icon;
            // var name = data.name; // deprecated
            var tabId = data.id;
            var slotsArray = [];
            var isSelected = false;
            var tabWidth = 112;
            var tabSprite = new SpriteContainer({
                width: tabWidth,
                imageName: isSelected ? 'gui/skins/tab-on' : 'gui/skins/tab-off',
            });
            // var nameText = new Text({
            //     z: 0,
            //     position: new Vector2(0, 0),
            //     text: name,
            //     font: 'omnes',
            //     fontSize: 16,
            //     fontColor: '#000',
            //     align: 'center',
            //     textBaseline: 'middle',
            //     maxWidth: 64,
            //     linebreaks: false,
            //     drawDebug: false,
            //     components: []
            // });
            var tabIcon = new Entity({
                name: 'icon',
                position: new Vector2(0, 0),
                components: [
                    new SpriteContainer({
                        width: 40,
                        imageName: iconName,
                    })
                ]
            });
            var clickable = new Clickable({
                onClick: function () {
                    if (isSelected || isAnimating) {
                        return;
                    }
                    setTab(tabId);
                }
            });
            var tab = new Entity({
                name: 'tab-' + tabId,
                components: [
                    tabSprite,
                    // nameText,
                    tabIcon,
                    clickable
                ]
            }).extend({
                select: function () {
                    // unselect others
                    tabs.iterate(function (item) {
                        item.unselect();
                    });
                    isSelected = true;
                    tabSprite.setup({
                        width: tabWidth,
                        imageName: 'gui/skins/tab-on',
                    });


                    // reset scrollinglist, remove current pages
                    while (scrollingList.length) {
                        scrollingList.removeItem(0);
                    }
                    slotsArray = [];

                    if (skinsArray) {
                        dotList.clear();
                        makePage({
                            tabId: tabId,
                            slotsArray: slotsArray,
                            skinsArray: skinsArray,
                            parent: tab,
                            onSelect: data.onSelect,
                            onIcon: data.onIcon,
                            onUnlock: data.onUnlock,
                            saveKeyCurrent: data.saveKeyCurrent,
                            saveKeyUnlocked: data.saveKeyUnlocked,
                            countX: data.countX
                        });
                    }

                    if (data.onTabSet) {
                        data.onTabSet(tab, slotsArray);
                    }

                    // makePage(slots, balls, 0, 6);
                    // makePage(slots, balls, 6, 6);
                },
                unselect: function () {
                    isSelected = false;
                    tabSprite.setup({
                        width: tabWidth,
                        imageName: 'gui/skins/tab-off',
                    });
                },
                setTab: function () {
                    tab.select();
                }
            });
            tabs.addItem(tab);

            if (data.onTabAdd) {
                data.onTabAdd(tab, slotsArray);
            }
        };
        /**
         * Dialog box
         */
        var topBanner = new Entity({
            name: 'topBanner',
            components: [
                new SpriteContainer({
                    width: 320,
                    imageName: 'gui/popup/banner',
                }),
                new Text({
                    position: new Vector2(0, -32),
                    text: Localization.getText('shop'),
                    font: 'omnes',
                    fontSize: 32,
                    fontColor: '#ffffff',
                    align: 'center',
                    textBaseline: 'middle',
                    maxWidth: 200,
                    linebreaks: true,
                    lineWidth: [3],
                    strokeStyle: ['#03d2ff'],
                    strokeOffset: [new Vector2(0, 1.5)],
                    components: []
                }),
                new Anchor({
                    relative: new Vector2(0.5, 0.5),
                    offset: new Vector2(0, -168)
                })
            ]
        });
        var backBoard = new Entity({
            name: 'backboard',
            components: [
                tabs,
                new SpriteContainer({
                    width: 320,
                    imageName: 'gui/skins/backboard-skins',
                }),
                new Anchor({
                    relative: new Vector2(0.5, 0.5),
                    offset: new Vector2(0, 64)
                }),
                scrollingList,
                dotList,
                arrowLeft,
                arrowRight
            ]
        });
        var buttonClose = new IconButton({
            name: 'buttonClose',
            position: new Vector2(0, 0),
            sprite: HdButtonSprite({
                imageName: 'gui/buttons/button-close',
                width: 48
            }),
            // icon options
            icon: new Entity({
                name: 'none'
            }),

            onClick: function () {
                if (isAnimating) {
                    return;
                }
                entity.removeSelf();
            },
            components: [
                new Anchor({
                    relative: new Vector2(0.5, 0.5),
                    offset: new Vector2(136, -160)
                }),
                new BackButton({
                    onPressed: function (data) {
                        data.entity.doCallback();
                    }
                })
            ]
        });
        /**
         * Contents
         */
        // single page with slots (groups of 9)
        var makePage = function (data) {
            // add scrollable page
            var tabId = data.tabId;
            var countX = data.countX || 4; // 3 or 4
            var saveKeyUnlocked = data.saveKeyUnlocked;
            var saveKeyCurrent = data.saveKeyCurrent;
            var onUnlock = data.onUnlock;
            var onIcon = data.onIcon;
            var onSelect = data.onSelect;
            var slots = data.slotsArray;
            var skins = data.skinsArray;
            // var scale = 2;
            var scale = countX === 4 ? 1.5 : 1.75;
            var isPageSelected = false;
            var i, l;
            var slotCount = 0;
            var lockedSlots = [];
            var page = new Entity({
                name: tabId + '-' + scrollingList.length,
                position: new Vector2(0, 0),
                dimension: new Rectangle(-60 * scale, -40 * scale,
                    140 * scale,
                    80 * scale
                ),
                components: [
                    // new SpriteContainer({
                    //     width: 140,
                    //     height: 80,
                    //     imageName: 'gui/bar-red',
                    // })
                ]
            }).extend({
                getLockedSlots: function () {
                    return lockedSlots;
                }
            });
            // add slots in this page
            var addSlot = function (skinName) {
                var index = slotCount;
                var isUnlocked = SaveValues[saveKeyUnlocked].indexOf(skinName) >= 0;
                var isSelected = skinName === SaveValues[saveKeyCurrent];
                var iconBack = new SpriteContainer({
                    width: 44 * scale,
                    imageName: !isUnlocked ? 'gui/skins/skinback-locked' : 'gui/skins/skinback-unlocked',
                });
                var iconSelected = new SpriteContainer({
                    width: 44 * scale,
                    imageName: 'gui/skins/selection',
                    visible: isSelected
                });
                var iconSkin = new Entity({
                    name: 'iconSkin',
                    components: [
                        new SpriteContainer(onIcon ? onIcon(skinName) : {
                            width: 32 * scale,
                            imageName: 'gui/skins/icons/ball-' + skinName,
                            position: new Vector2(0, -2),
                            setDimensions: false
                        })
                    ]
                });

                var onClick = function () {
                    // can not select or already selected
                    if (!isUnlocked || isSelected || isAnimating) {
                        return;
                    }
                    if (!scrollingList.didMove(2)) {
                        // unselect other skins
                        Utils.forEach(slots, function (otherSlot) {
                            otherSlot.unselect();
                        });
                        // select this one
                        slot.select();

                        new Tween({
                            from: 0.8,
                            to: 1,
                            in: 30,
                            ease: 'elastic',
                            decay: 5,
                            oscillations: 2,
                            onUpdate: function (v, t) {
                                slot.scale.x = v;
                                slot.scale.y = v;
                            }
                        });

                        // set skin
                        if (onSelect) {
                            onSelect(skinName);
                        }
                    }
                };
                var clickable = new Clickable({
                    onClick: onClick
                });
                var slot = new Entity({
                    name: 'slot',
                    // place in rows of 3
                    position: countX === 4 ? new Vector2(
                        (-48 * 1.5 + (index % 4) * 48) * scale,
                        (-48 * 1.5 + Math.floor(index / 4) * 48) * scale + 16
                    ) : new Vector2(
                        (-48 + (index % 3) * 48) * scale,
                        (-48 + Math.floor(index / 3) * 48) * scale - 16
                    ),
                    components: [
                        iconBack,
                    ]
                }).extend({
                    click: onClick,
                    select: function () {
                        if (!isUnlocked) {
                            return;
                        }
                        isSelected = true;
                        iconSelected.visible = true;
                    },
                    unselect: function () {
                        if (!isUnlocked) {
                            return;
                        }
                        isSelected = false;
                        iconSelected.visible = false;
                    },
                    unlock: function () {
                        isUnlocked = true;
                        slot.remove(lockSprite);
                        makeClickable();
                        // remove from array
                        lockedSlots.splice(lockedSlots.indexOf(slot), 1);
                        // save it
                        if (onUnlock) {
                            onUnlock(skinName);
                        }

                        // re-apply clipping plane
                        iconSkin.getComponent('spriteContainer', function (spriteContainer) {
                            spriteContainer.sprite.mesh.material.clippingPlanes = [leftClippingPlane, rightClippingPlane];
                        });
                        iconSelected.sprite.mesh.material.clippingPlanes = [leftClippingPlane, rightClippingPlane];

                    },
                    highlight: function () {
                        // highlight back
                        iconBack.setup({
                            width: 44 * scale,
                            imageName: 'gui/skins/skinback-unlocked',
                        });
                        // re-apply clipping plane
                        iconBack.sprite.mesh.material.clippingPlanes = [leftClippingPlane, rightClippingPlane];
                    },
                    unHighlight: function () {
                        // highlight back
                        iconBack.setup({
                            width: 44 * scale,
                            imageName: 'gui/skins/skinback-locked',
                        });
                        // re-apply clipping plane
                        iconBack.sprite.mesh.material.clippingPlanes = [leftClippingPlane, rightClippingPlane];
                    }
                });
                var lockSprite;
                var makeClickable = function () {
                    // make clickable
                    slot.attach(clickable);
                    slot.attach(iconSkin);
                    slot.attach(iconSelected);
                };

                slots.push(slot);
                page.attach(slot);

                if (isUnlocked) {
                    makeClickable();
                } else {
                    // TODO: other unlock methods
                    lockSprite = new SpriteContainer({
                        width: 24 * scale,
                        imageName: 'gui/skins/icn-locked',
                        position: new Vector2(0, 0)
                    });
                    slot.attach(lockSprite);
                    lockedSlots.push(slot);
                }

                slotCount += 1;

                // scroll to this page if it has a selected slot
                if (isSelected) {
                    isPageSelected = true;
                }

                return slot;
            };
            var dot = new Entity({
                name: 'dot',
                alpha: 0.5,
                components: [
                    new SpriteContainer({
                        width: 8,
                        imageName: 'gui/top/stage-finished'
                    })
                ]
            });

            for (i = slots.length, l = i + countX * 3; i < l; ++i) {
                if (skins[i]) {
                    addSlot(skins[i]);
                }
            }
            scrollingList.addItem(page);

            dotList.addItem(dot);

            // scroll to this page if it has a selected slot
            if (isPageSelected) {
                scrollingList.snapTo(scrollingList.length - 1);
                dot.alpha = 1;

            }

            // check if there are more pages to spawn
            if (slots.length !== skins.length) {
                makePage(data);
            }

            // apply clipping on all sprites
            findSprites(page);

            return page;
        };
        var findSprites = function (entity) {
            if (entity.components) {
                Utils.forEach(entity.components, function (component) {
                    if (component.components) {
                        findSprites(component);
                    }
                    if (component.name === 'sprite') {
                        component.mesh.material.clippingPlanes = [leftClippingPlane, rightClippingPlane];
                    }
                });
            }
        };
        var startUnlockAnimation = function (data) {
            // get locked slots from current page
            var page = scrollingList.getItems()[scrollingList.getSelection()];
            var slots = page.getLockedSlots();
            var currentSelection = null;
            var timer = null;
            var openCurrent = function () {
                // open it
                currentSelection.unlock();
                // select it
                isAnimating = false;
                currentSelection.click();

                if (data.onComplete) {
                    data.onComplete();
                }

                new Confetti({
                    z: entity.z,
                    life: 1
                });
            };

            if (slots.length === 1) {
                currentSelection = Utils.pickRandomFrom(slots);
                currentSelection.highlight();
                openCurrent();
                return;
            } else if (slots.length === 0) {
                // nothing to unlock
                if (data.onComplete) {
                    data.onComplete();
                }
                return;
            }

            isAnimating = true;
            new Tween({
                from: 0,
                to: 20,
                in: 180,
                ease: 'squareroot',
                decay: 1,
                oscillations: 1,
                onStart: function () {},
                onUpdate: function (v, t) {
                    var previousSelection;
                    if (timer !== Math.floor(v) && t < 180) {
                        timer = Math.floor(v);
                        if (currentSelection) {
                            currentSelection.unHighlight();
                        }
                        previousSelection = currentSelection;
                        while (currentSelection === previousSelection) {
                            currentSelection = Utils.pickRandomFrom(slots);
                        }
                        currentSelection.highlight();

                        if (hapticCooldown === 0) {
                            TapticEngine.impact({
                                type: "light"
                            });
                            hapticCooldown = 4;
                        }
                    }
                },
                onComplete: function () {
                    openCurrent();
                }
            });
        };
        var getCost = function () {
            // depends on amount unlocked and type of unlock
            var page = scrollingList.getItems()[scrollingList.getSelection()];

            if (page && page.name.startsWith('balls')) {
                return 100 + (SaveValues.unlockedBalls.length - 1) * 50;
            }
            if (page && page.name.startsWith('players')) {
                return 500 + (SaveValues.unlockedPlayers.length - 1) * 100;
            }

            return 1000;
        };
        var updateButtons = function () {
            var canUnlockOnThisPage = function () {
                var page = scrollingList.getItems()[scrollingList.getSelection()];
                if (!page) {
                    return false;
                }
                var slots = page.getLockedSlots();
                return slots.length > 0;
            };
            costText.setText(getCost());
            buttonRandom.active = canUnlockOnThisPage(); // disable when nothing left to unlock
        };
        /**
         * Buy buttons
         */
        var costText = new Text({
            position: new Vector2(-44, 6),
            text: '',
            font: 'omnes',
            fontSize: 14,
            fontColor: Globals.Colors.coin,
            align: 'center',
            textBaseline: 'middle',
            strokeStyle: Globals.Colors.coinOutline,
            lineWidth: 1,
            maxWidth: 32,
            linebreaks: false,
            drawDebug: false,
            components: []
        });
        var iconRandom = new Entity({
            name: 'iconRandom',
            components: [
                new SpriteContainer({
                    width: 24,
                    imageName: 'gui/icons/icon-currency',
                    position: new Vector2(-44, -8),
                }),
                costText,
                new Text({
                    position: new Vector2(16, 0),
                    text: Localization.getText('unlockRandom'),
                    linebreaksOnlyOnSpace: true,
                    font: 'omnes',
                    fontSize: 18,
                    fontColor: '#ffffff',
                    align: 'center',
                    textBaseline: 'middle',
                    strokeStyle: '#2284ff',
                    ySpacing: 4,
                    lineWidth: 2,
                    maxWidth: 80,
                    maxHeight: 64,
                    linebreaks: true,
                    drawDebug: false,
                    components: []
                })
            ]
        });
        var buttonRandom = new IconButton({
            position: new Vector2(-80, 160),
            sprite: HdButtonSprite({
                imageName: 'gui/buttons/blue-mid-sink',
                width: 136,
            }),
            icon: iconRandom,
            offsetWhenPressed: 2, // push pixels down when onButtonDown
            iconAlpha: 0.5,
            onClick: function () {
                var cost = getCost();
                if (scrollingList.didMove(1) || isAnimating) {
                    return;
                }
                if (SaveValues.coins >= cost) {
                    SaveValues.coins -= cost;
                    buttonRandom.active = false;
                    buttonCoins.active = false;
                    scrollingList.active = false;
                    startUnlockAnimation({
                        onComplete: function () {
                            // reactivate
                            buttonCoins.active = true;
                            scrollingList.active = true;
                            updateButtons();
                        }
                    });

                    var page = scrollingList.getItems()[scrollingList.getSelection()];
                    if (page && page.name) {
                        if (page.name.startsWith('balls')) {
                            EventSystem.fire('GameAnalytics-addDesignEvent', {
                                eventId: 'UI:skinScreen:unlockBall'
                            });
                        } else if (page.name.startsWith('players')) {
                            EventSystem.fire('GameAnalytics-addDesignEvent', {
                                eventId: 'UI:skinScreen:unlockCharacter'
                            });
                        }
                    }
                } else {
                    EventSystem.fire('notEnoughCoins');
                }
            }
        });
        var iconCoins = new Entity({
            name: 'iconCoins',
            components: [
                new CoinCounter({
                    coins: '+75',
                    position: new Vector2(16, -2),
                    imageName: 'gui/icons/big-coin',
                    fontSize: 24,
                    coinSize: 24,
                    color: Globals.Colors.coin,
                    strokeStyle: Globals.Colors.coinOutline,
                    lineWidth: 2,
                    coinOffset: new Vector2(0, 2),
                    alignment: 'center',
                    coinPos: 'right',
                })
            ]
        });
        var buttonCoins = new IconButton({
            position: new Vector2(80, 160),
            sprite: HdButtonSprite({
                imageName: 'gui/buttons/orange-mid-rv',
                width: 136,
            }),
            icon: iconCoins,
            offsetWhenPressed: 2, // push pixels down when onButtonDown
            iconAlpha: 0.5,
            onClick: function () {
                EventSystem.fire('GameAnalytics-addDesignEvent', {
                    eventId: 'UI:skinScreen:getCoinsButton'
                });

                if (WatchAd.canShowRewarded() || Bento.isDev()) {
                    EventSystem.fire('GameAnalytics-addDesignEvent', {
                        eventId: 'Ads:rv:shopCoins'
                    });
                    buttonCoins.active = false;
                    WatchAd.showRewarded({
                        tag: 'getCoinsShop',
                        onReward: function () {
                            Bento.objects.attach(new GotCoinsPopup({
                                amount: 75
                            }));
                            updateButtons();
                            EventSystem.fire('GameAnalytics-addDesignEvent', {
                                eventId: 'Ads:rv:shopCoinsComplete'
                            });
                        },
                        onComplete: function (error) {
                            // Video completed (reward state unknown)
                            buttonCoins.active = true;
                        }
                    });
                }
            },
            components: [
                new WaitingForAd({
                    offset: new Vector2(12, 0)
                })
            ]
        });
        var behavior = new Object({
            name: 'behavior',
            start: function (data) {
                addTab({
                    name: 'Balls',
                    icon: 'gui/icons/balls',
                    id: 'balls',
                    countX: 4,
                    saveKeyCurrent: 'currentBall',
                    saveKeyUnlocked: 'unlockedBalls',
                    skinsArray: balls,
                    onSelect: function (skinName) {
                        SaveValues.currentBall = skinName;
                        EventSystem.fire('refreshBallSkin');
                    },
                    onTabAdd: function (tab, slots) {
                        // add counter
                        tab.attach(new Text({
                            position: new Vector2(0, 14),
                            text: SaveValues.unlockedBalls.length + '/' + Globals.balls.length,
                            font: 'omnes',
                            fontSize: 8,
                            fontColor: '#ffffff',
                            align: 'center',
                            textBaseline: 'middle',
                            strokeStyle: '#298dff',
                            lineWidth: 2,
                            linebreaks: false,
                            drawDebug: false,
                            components: []
                        }));
                    },
                    onUnlock: function (skinName) {
                        var unlockedBalls = SaveValues.unlockedBalls;
                        if (unlockedBalls.indexOf(skinName) < 0) {
                            unlockedBalls.push(skinName);
                            SaveValues.unlockedBalls = unlockedBalls;
                        }
                    }
                });
                addTab({
                    name: 'Players',
                    icon: 'gui/icons/ballers',
                    id: 'players',
                    countX: 3,
                    saveKeyCurrent: 'currentPlayer',
                    saveKeyUnlocked: 'unlockedPlayers',
                    skinsArray: players,
                    onIcon: function (skinName) {
                        return {
                            height: 80,
                            imageName: 'gui/skins/icons/profile-baller-' + skinName,
                            position: new Vector2(-4, 0),
                            setDimensions: false
                        };
                    },
                    onSelect: function (skinName) {
                        SaveValues.currentPlayer = skinName;
                        EventSystem.fire('refreshPlayerSkin');
                    },
                    onTabAdd: function (tab, slots) {
                        // add counter
                        tab.attach(new Text({
                            position: new Vector2(0, 14),
                            text: SaveValues.unlockedPlayers.length + '/' + Globals.players.length,
                            font: 'omnes',
                            fontSize: 8,
                            fontColor: '#ffffff',
                            align: 'center',
                            textBaseline: 'middle',
                            strokeStyle: '#298dff',
                            lineWidth: 2,
                            linebreaks: false,
                            drawDebug: false,
                            components: []
                        }));
                    },
                    onUnlock: function (skinName) {
                        var unlockedPlayers = SaveValues.unlockedPlayers;
                        if (unlockedPlayers.indexOf(skinName) < 0) {
                            unlockedPlayers.push(skinName);
                            SaveValues.unlockedPlayers = unlockedPlayers;
                        }
                    }
                });

                setTab(SaveValues.lastTab);

                backBoard.attach(buttonRandom);
                backBoard.attach(buttonCoins);


                // requires renderer.localClippingEnabled = true 
                data.renderer.three.renderer.localClippingEnabled = true;
            },
            destroy: function (data) {},
            update: function (data) {
                if (hapticCooldown) {
                    hapticCooldown -= 1;
                }
            },
            draw: function (data) {}
        });
        var modal = new Modal({});
        var entity = new Entity({
            z: Globals.Layers.POPUPS,
            name: 'skinSelectPopup',
            components: [
                modal,
                fill,
                topBanner,
                backBoard,
                buttonClose,
                behavior
            ]
        });

        if (Bento.saveState.load('cheat-shouldUnlockAll', false)) {
            var unlockedBalls = SaveValues.unlockedBalls;
            var unlockedPlayers = SaveValues.unlockedPlayers;
            Utils.forEach(balls, function (ballKey) {
                if (unlockedBalls.indexOf(ballKey) < 0) {
                    unlockedBalls.push(ballKey);
                }
            });
            Utils.forEach(players, function (playerSkin) {
                if (unlockedPlayers.indexOf(playerSkin) < 0) {
                    unlockedPlayers.push(playerSkin);
                }
            });
            SaveValues.unlockedBalls = unlockedBalls;
            SaveValues.unlockedPlayers = unlockedPlayers;
        }


        return entity;
    };
    return SkinSelectPopup;
});