/**
 * Progression meter is full: show unlock
 * @moduleName UnlockPopup
 * @snippet UnlockPopup|Constructor
UnlockPopup({})
 */
bento.define('gui/popups/unlockpopup', [
    'bento',
    'bento/utils',
    'bento/gui/text',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/components/anchor',
    'bento/components/spritecontainer',
    'gui/iconbutton',
    'gui/hdbuttonsprite',
    'bento/eventsystem',
    'modules/watchad',
    'managers/savevalues',
    'bento/tween',
    'onigiri/sprite3d',
    'globals',
    'bento/components/clickable',
    'onigiri/onigiri',
    'components/targetsize',
    'components/materialswapper',
    'gui/widgets/confetti',
    'modules/localization',
    'components/waitingforad'
], function (
    Bento,
    Utils,
    Text,
    Vector2,
    Rectangle,
    Entity,
    Anchor,
    SpriteContainer,
    IconButton,
    HdButtonSprite,
    EventSystem,
    WatchAd,
    SaveValues,
    Tween,
    Sprite3d,
    Globals,
    Clickable,
    Onigiri,
    TargetSize,
    MaterialSwapper,
    Confetti,
    Localization,
    WaitingForAd
) {
    'use strict';
    var UnlockPopup = function (settings) {
        var viewport = Bento.getViewport();
        var onComplete = settings.onComplete;
        var playerKey = settings.skinName || 'default';
        var didWatchAd = false;
        /**
         * Components
         */
        var fill = new Entity({
            name: 'fill',
            components: [
                new SpriteContainer({
                    width: viewport.width + 16,
                    imageName: 'gui/bg-back',
                    alpha: 0.9,
                    originRelative: new Vector2(0.5, 0.5),
                }),
                new Anchor({
                    relative: new Vector2(0.5, 0.5),
                })
            ]
        });
        var behavior = new Object({
            name: 'behavior',
            start: function (data) {

                new Tween({
                    from: 0,
                    to: 1,
                    in: 60,
                    ease: 'elastic',
                    decay: 5,
                    oscillations: 2,
                    onUpdate: function (v, t) {
                        topBanner.scale.x = v;
                        topBanner.scale.y = v;
                    }
                });

                new Tween({
                    from: 0,
                    to: 1,
                    in: 60,
                    ease: 'elastic',
                    decay: 5,
                    oscillations: 2,
                    delay: 10,
                    applyOnDelay: true,
                    onUpdate: function (v, t) {
                        player.scale.x = v;
                        player.scale.y = v;
                    }
                });

                new Tween({
                    from: 0,
                    to: 1,
                    in: 30,
                    ease: 'linear',
                    delay: 180,
                    onStart: function () {
                        button.attach(loseItText);
                    },
                    onUpdate: function (v, t) {
                        loseItText.alpha = v;
                    }
                });
            },
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        });
        /**
         * Top banner
         */
        var bannerText = new Text({
            text: Localization.getText('new'),
            position: new Vector2(0, -16),
            font: 'omnes',
            fontSize: 32,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 200,
            linebreaks: false,
            drawDebug: false,
            components: []
        });
        var topBanner = new Entity({
            name: 'topBanner',
            position: new Vector2(0, -138),
            components: [
                new SpriteContainer({
                    width: 256,
                    imageName: 'gui/popup/banner',
                }),
                bannerText
            ]
        });
        /**
         * Button
         */
        var openText = new Text({
            position: new Vector2(16, -2),
            text: Localization.getText('unlock'),
            font: 'omnes',
            fontSize: 32,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            antiAlias: true, // Cocoon only
            maxWidth: 80,
            lineWidth: 2,
            strokeStyle: Globals.Colors.coinOutline,
            linebreaks: false,
            drawDebug: false,
            components: []
        });
        var icon = new Entity({
            name: 'icon',
            components: [
                openText
            ]
        });
        var button = new IconButton({
            name: 'button',
            position: new Vector2(0, 160 - 24),
            sprite: HdButtonSprite({
                imageName: 'gui/buttons/orange-rv',
                width: 144,
            }),
            // icon options
            icon: icon,

            // offset of icon
            offset: new Vector2(0, 0),
            offsetWhenPressed: 2, // push pixels down when onButtonDown

            onClick: function () {
                EventSystem.fire('GameAnalytics-addDesignEvent', {
                    eventId: 'UI:homeScreen:unlockCharacterButton'
                });
                if (WatchAd.canShowRewarded() || Bento.isDev()) {
                    EventSystem.fire('GameAnalytics-addDesignEvent', {
                        eventId: 'Ads:rv:unlockCharacter'
                    });
                    button.active = false;
                    WatchAd.showRewarded({
                        tag: 'unlockCharacter',
                        onReward: function () {
                            didWatchAd = true;
                            EventSystem.fire('GameAnalytics-addDesignEvent', {
                                eventId: 'Ads:rv:unlockCharacterComplete'
                            });

                            // save
                            var unlockedPlayers = SaveValues.unlockedPlayers;
                            if (unlockedPlayers.indexOf(playerKey) < 0) {
                                unlockedPlayers.push(playerKey);
                                SaveValues.unlockedPlayers = unlockedPlayers;
                            }

                            // select it immediately
                            SaveValues.currentPlayer = playerKey;

                            // done: just remove everything
                            button.removeSelf();
                            loseItText.removeSelf();
                            entity.fadeOut();

                            new Confetti({
                                z: entity.z
                            });
                        },
                        onComplete: function (error) {
                            // Video completed (reward state unknown)
                            button.active = true;
                        }
                    });
                }
            },
            components: [
                new WaitingForAd({
                    offset: new Vector2(12, 0)
                })
            ]
        });
        var loseItText = new Text({
            position: new Vector2(0, 64),
            text: Localization.getText('loseIt'),
            font: 'omnes',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            linebreaks: false,
            drawDebug: false,
            boundingBox: new Rectangle(-64, -24, 128, 48),
            components: [
                new Clickable({
                    onClick: function (data) {
                        button.removeSelf();
                        loseItText.removeSelf();
                        entity.fadeOut();

                        // re-randomize
                        SaveValues.nextCharacter = 'default';

                        EventSystem.fire('GameAnalytics-addDesignEvent', {
                            eventId: 'UI:homeScreen:discardCharacterButton'
                        });
                    }
                })
            ]
        });
        /**
         * Player
         */
        var playerMesh = new THREE.Object3D().add(Onigiri.getMesh('players/baller-' + playerKey + '.gltf'));
        var playerSprite = new Sprite3d({
            object3D: playerMesh,
        });
        var player = new Entity({
            name: 'player',
            position: new Vector2(0, 80),
            components: [
                playerSprite,
                new TargetSize({
                    height: 160,
                    mesh: playerMesh
                }),
                // Globals.getPlayerMaterial(playerKey, playerMesh, null),
                new MaterialSwapper({
                    id: 'playerUnlock-' + playerKey, // unique id for refcounting
                    type: 'MeshBasicMaterial',
                    deepSwap: true,
                    object3D: playerMesh,
                    materialSettings: {
                        skinning: true,
                        map: Onigiri.getTexture('textures/players/tex-baller-' + playerKey, {
                            wrapS: THREE.RepeatWrapping,
                            wrapT: THREE.RepeatWrapping,
                        })
                    }
                }),
                new Object({
                    name: 'behavior',
                    start: function () {
                        playerMesh.position.z = 10;
                    },
                    update: function (data) {
                        playerMesh.rotation.y += 0.02;
                    },
                })
            ]
        });

        var container = new Entity({
            name: 'container',
            scale: new Vector2(1.25, 1.25),
            components: [
                topBanner,
                player,
                button,
                new Anchor({
                    relative: new Vector2(0.5, 0.5),
                    offset: new Vector2(0, 16)
                }),
            ]
        });
        var entity = new Entity({
            z: Globals.Layers.POPUPS,
            name: 'unlockPopup',
            components: [
                behavior,
                fill,
                container,
            ]
        }).extend({
            fadeOut: function () {
                new Tween({
                    from: 1,
                    to: 0,
                    in: 30,
                    ease: 'easeInBack',
                    onUpdate: function (v, t) {
                        topBanner.scale.x = v;
                        topBanner.scale.y = v;
                    }
                });
                new Tween({
                    from: 1,
                    to: 0,
                    in: 30,
                    delay: 10,
                    ease: 'easeInBack',
                    onUpdate: function (v, t) {
                        player.scale.x = v;
                        player.scale.y = v;
                    },
                    onComplete: function () {
                        entity.removeSelf();
                        if (onComplete) {
                            onComplete(didWatchAd);
                        }
                    }
                });
            }
        });
        return entity;
    };
    return UnlockPopup;
});