/**
 * Tiny popup to show progression of character
 * @moduleName UnlockProgressPopup
 * @snippet UnlockProgressPopup|Constructor
UnlockProgressPopup({})
 */
bento.define('gui/popups/unlockprogresspopup', [
    'bento',
    'bento/utils',
    'bento/entity',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/spritecontainer',
    'globals',
    'bento/components/anchor',
    'bento/gui/text',
    'managers/savevalues',
    'bento/tween',
    'modules/tapticengine',
    'gui/popups/unlockpopup',
    'modules/localization'
], function (
    Bento,
    Utils,
    Entity,
    Vector2,
    Rectangle,
    SpriteContainer,
    Globals,
    Anchor,
    Text,
    SaveValues,
    Tween,
    TapticEngine,
    UnlockPopup,
    Localization
) {
    'use strict';
    var UnlockProgressPopup = function (settings) {
        var viewport = Bento.getViewport();
        var onComplete = settings.onComplete;
        var topClippingPlane = new THREE.Plane(new THREE.Vector3(0, 1, 0), 0);
        var height = 104;
        // the more characters you have, the slower it gets
        var addition = Globals.getProgressionAmount();
        var unlockProgress = SaveValues.unlockProgress;
        var unlockProgressFloored = Math.floor(unlockProgress * 100);
        var unlockedPlayers = SaveValues.unlockedPlayers;
        var didPresentUnlock = false;
        var hasNothingToUnlock = false;
        var skinName = (function () {
            // randomize skin
            var skin = SaveValues.nextCharacter;
            var characters = [].concat(Globals.players);
            var removeCharacter = function (character) {
                var i = characters.indexOf(character);
                if (i > -1) {
                    characters.splice(i, 1);
                }
            };
            removeCharacter(SaveValues.dailyBonusCharacter);
            while (unlockedPlayers.indexOf(skin) >= 0 && characters.length) {
                skin = Utils.pickRandomFrom(characters);
                removeCharacter(skin);
            }
            SaveValues.nextCharacter = skin;
            return skin;
        })();
        var percentageText = new Text({
            position: new Vector2(0, 56),
            text: unlockProgressFloored + '%',
            font: 'omnes',
            fontSize: 24,
            fontColor: '#fff',
            align: 'center',
            textBaseline: 'middle',
            lineWidth: [4, 1.5], // set to add an outline
            strokeStyle: [Globals.Colors.darkblue, Globals.Colors.lightblue],
            strokeOffset: [new Vector2(0, 1), new Vector2(0, 0)],
            margin: new Vector2(16, 16),

            maxWidth: 80,
            linebreaks: false,
            drawDebug: false,
            components: []
        });
        var nextText = new Text({
            position: new Vector2(0, -52),
            text: Localization.getText('nextBaller'),
            font: 'omnes',
            fontSize: 16,
            fontColor: Globals.Colors.darkblue,
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 104,
            linebreaks: false,
            drawDebug: false,
            components: []
        });
        var shadowBack = new SpriteContainer({
            height: height,
            imageName: 'gui/skins/icons/baller-' + skinName,
            position: new Vector2(-8, 8)
        });

        var backFront = new SpriteContainer({
            height: 104,
            position: new Vector2(0, 8),
            imageName: 'gui/unlock/progress-filled',
        });
        var shadowFront = new SpriteContainer({
            height: height,
            imageName: 'gui/skins/icons/baller-' + skinName,
            position: new Vector2(-8, 8)
        });
        var behavior = new Object({
            name: 'behavior',
            start: function (data) {
                // complete immediately if no characters left to unlock
                if (unlockedPlayers.length >= Globals.players.length) {
                    entity.removeSelf();
                    hasNothingToUnlock = true;
                    if (onComplete) {
                        onComplete(false);
                    }
                    return;
                }

                // requires renderer.localClippingEnabled = true 
                data.renderer.three.renderer.localClippingEnabled = true;

                shadowFront.sprite.mesh.material.clippingPlanes = [topClippingPlane];
                backFront.sprite.mesh.material.clippingPlanes = [topClippingPlane];

                // move in
                new Tween({
                    from: background.position.x + viewport.width,
                    to: background.position.x,
                    in: 30,
                    ease: 'elastic',
                    decay: 8,
                    oscillations: 1,
                    onStart: function () {},
                    onUpdate: function (v, t) {
                        background.position.x = v;
                    },
                    onComplete: function () {
                        addProgress(addition, onComplete);
                    }
                });
            },
            destroy: function (data) {},
            update: function (data) {
                topClippingPlane.constant = -background.position.y - backFront.position.y - 104 / 2 + 104 * unlockProgress;
            },
            draw: function (data) {}
        });
        var spriteBox = new SpriteContainer({
            width: 128,
            imageName: 'gui/unlock/progress-back'
        });
        var spriteBack = new SpriteContainer({
            height: 104,
            position: new Vector2(0, 8),
            imageName: 'gui/unlock/progress-left'
        });
        var background = new Entity({
            name: 'background',
            components: [
                new Anchor({
                    relative: new Vector2(0.5, 0.5),
                    offset: new Vector2(0, -8),
                    onResize: function (isPortrait) {}
                }),
                spriteBox,
                spriteBack,
                shadowBack,
                new Object({
                    name: 'replaceColor',
                    start: function () {
                        shadowBack.sprite.mesh.material.color.set(0x06133c);
                    }
                }),
                backFront,
                shadowFront,
                percentageText,
                nextText,
                behavior
            ]
        });
        var addProgress = function (amount, onDone) {
            new Tween({
                from: unlockProgress,
                to: Math.min(1, unlockProgress + amount),
                in: 60,
                ease: 'squareroot',
                onStart: function () {},
                onUpdate: function (v, t) {
                    unlockProgress = v;
                    if (unlockProgressFloored !== Math.floor(unlockProgress * 100)) {
                        unlockProgressFloored = Math.floor(unlockProgress * 100);
                        percentageText.setText(unlockProgressFloored + '%');
                        TapticEngine.impact({
                            type: "light"
                        });
                    }
                },
                onComplete: function () {
                    // save progress
                    SaveValues.unlockProgress = unlockProgress;

                    tweenOut(function () {
                        // not 100%? done
                        if (unlockProgress < 1) {
                            if (onDone) {
                                onDone(false);
                            }
                        } else {
                            // 2nd popup: watch ad
                            SaveValues.unlockProgress = 0;
                            var unlockPopup = new UnlockPopup({
                                skinName: skinName,
                                onComplete: onDone
                            });
                            Bento.objects.attach(unlockPopup);
                            didPresentUnlock = true;
                        }
                    });
                }
            });
        };
        var entity = new Entity({
            z: Globals.Layers.POPUPS,
            name: 'unlockProgressPopup',
            components: [
                background
            ]
        }).extend({
            addProgress: addProgress,
            tweenOut: function (tweenDone) {
                new Tween({
                    from: background.position.x,
                    to: background.position.x - viewport.width,
                    in: 30,
                    delay: 30,
                    ease: 'easeInBack',
                    onUpdate: function (v, t) {
                        background.position.x = v;
                    },
                    onComplete: tweenDone
                });
            },
            didPresentUnlock: function () {
                return didPresentUnlock;
            },
            hasNothingToUnlock: function () {
                return hasNothingToUnlock;
            }
        });
        
        var tweenOut = function (callback) {
            

            // tween out deprecated
            callback();
        };
        return entity;
    };
    return UnlockProgressPopup;
});