/**
 * Win pop
 * @moduleName WinPopup
 * @snippet WinPopup|Constructor
WinPopup({})
 */
bento.define('gui/popups/winpopup', [
    'bento',
    'bento/utils',
    'bento/entity',
    'bento/math/vector2',
    'globals',
    'bento/components/fill',
    'gui/widgets/confetti',
    'bento/components/anchor',
    'bento/tween',
    'bento/gui/text',
    'gui/popups/unlockprogresspopup',
    'bento/components/spritecontainer',
    'modules/localization',
    'bento/math/rectangle',
    'gui/iconbutton',
    'gui/hdbuttonsprite',
    'bento/components/clickable',
    'managers/savevalues',
    'modules/watchad',
    'gui/popups/gotcoinspopup',
    'bento/eventsystem',
    'components/waitingforad'
], function (
    Bento,
    Utils,
    Entity,
    Vector2,
    Globals,
    Fill,
    Confetti,
    Anchor,
    Tween,
    Text,
    UnlockProgressPopup,
    SpriteContainer,
    Localization,
    Rectangle,
    IconButton,
    HdButtonSprite,
    Clickable,
    SaveValues,
    WatchAd,
    GotCoinsPopup,
    EventSystem,
    WaitingForAd
) {
    'use strict';
    var WinPopup = function (settings) {
        var viewport = Bento.getViewport();
        var onComplete = settings.onComplete || function () {};
        var fill = new Entity({
            name: 'fill',
            components: [
                new SpriteContainer({
                    width: viewport.width + 16,
                    alpha: 0.75,
                    imageName: 'gui/bg-back',
                    originRelative: new Vector2(0.5, 0.5),
                }),
                new Anchor({
                    relative: new Vector2(0.5, 0.5),
                })
            ]
        });
        var pixel = new Entity({
            name: 'pixel',
            components: [
                // new Fill({
                //     name: 'fill',
                //     dimension: new Rectangle(0, 0, 1, 1),
                //     color: [0xff / 0xff, 0x50 / 0xff, 0x28 / 0xff, 1], // [1, 1, 1, 1] is pure white
                //     originRelative: new Vector2(0, 0.5)
                // }),
                new SpriteContainer({
                    width: 1,
                    imageName: 'pixel',
                    originRelative: new Vector2(0, 0.5),
                    frameCountX: 3,
                    frameCountY: 3,
                    color: 0xff5028,
                    animations: {
                        default: {
                            speed: 0,
                            frames: [5]
                        }
                    }
                }),
                new Anchor({
                    relative: new Vector2(0, 0.20),
                    offset: new Vector2(0, 16),
                    onResize: function (isPortrait) {}
                })
            ]
        });
        var watchedAd;
        var behavior = new Object({
            name: 'behavior',
            start: function (data) {
                // spawn confetti
                new Confetti({
                    z: entity.z,
                    life: 1
                });
                new Tween({
                    from: 0,
                    to: 2,
                    in: 60,
                    ease: 'elastic',
                    decay: 8,
                    oscillations: 3,
                    onStart: function () {
                        entity.attach(winText);
                    },
                    onUpdate: function (v, t) {
                        winText.scale.x = v;
                        winText.scale.y = v;
                    },
                    onComplete: function () {
                        var popup = new UnlockProgressPopup({
                            onComplete: function (didWatchAd) {
                                var random = Utils.getRandom(2); // 50% chance for extra coins
                                watchedAd = didWatchAd;
                                if (watchedAd || SaveValues.currentLevel <= 3) {
                                    showNextButton(popup);
                                } else if (!popup.didPresentUnlock() && !popup.hasNothingToUnlock() && SaveValues.unlockProgress < 1 && random) {
                                    showExtraProgressButton(popup);
                                } else {
                                    // ask for extra coins
                                    showExtraCoins();
                                }
                            }
                        });
                        Bento.objects.attach(popup);
                    }
                });
                pixel.scale.y = 32;

                new Tween({
                    from: 0,
                    to: viewport.width,
                    in: 30,
                    delay: 0,
                    ease: 'exponential',
                    decay: 8,
                    oscillations: 1,
                    onStart: function () {},
                    onUpdate: function (v, t) {
                        pixel.scale.x = v;
                    },
                    onComplete: function () {
                        new Tween({
                            from: pixel.scale.y,
                            to: 80,
                            in: 30,
                            ease: 'exponential',
                            decay: 5,
                            onStart: function () {
                                entity.attach(lowerText);
                            },
                            onUpdate: function (v, t) {
                                pixel.scale.y = v;
                            },
                            onComplete: function () {

                            }
                        });
                    }
                });
            }
        });
        var winText = new Text({
            text: Localization.getText('dunked'),
            font: 'omnes',
            fontSize: 32,
            fontColor: '#ffffff',
            rotation: -Math.PI / 16,
            align: 'center',
            textBaseline: 'middle',
            lineWidth: [4, 1.5], // set to add an outline
            strokeStyle: [Globals.Colors.darkblue, Globals.Colors.lightblue],
            strokeOffset: [new Vector2(0, 1), new Vector2(0, 0)],
            margin: new Vector2(16, 16),
            maxWidth: viewport.width * 3 / 4,

            linebreaks: false,
            components: [
                new Anchor({
                    relative: new Vector2(0.5, 0.2),
                    offset: new Vector2(0, 0),
                    onResize: function (isPortrait) {}
                })
            ]
        });
        var lowerText = new Text({
            text: Localization.getText('dunkText' + Utils.getRandom(2)),
            font: 'omnes',
            fontSize: 24,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            lineWidth: [4, 1.5], // set to add an outline
            strokeStyle: [Globals.Colors.darkblue, Globals.Colors.lightblue],
            strokeOffset: [new Vector2(0, 2), new Vector2(0, 0)],
            margin: new Vector2(16, 16),
            maxWidth: viewport.width * 3 / 4,

            linebreaks: false,
            components: [
                new Anchor({
                    relative: new Vector2(0.5, 0.2),
                    offset: new Vector2(0, 52),
                    onResize: function (isPortrait) {}
                })
            ]
        });
        var entity = new Entity({
            z: Globals.Layers.POPUPS,
            name: 'winPopup',
            components: [
                fill,
                pixel,
                behavior
            ]
        });
        var showNextButton = function (popup) {
            // Deprecated: just go to next
            /*var icon = new Text({
                text: Localization.getText('next'),
                font: 'omnes',
                fontSize: 24,
                fontColor: '#ffffff',
                align: 'center',
                textBaseline: 'middle',
                maxWidth: 100,
                linebreaks: false,
                drawDebug: false,
                components: []
            });
            var buttonNext = new IconButton({
                sprite: HdButtonSprite({
                    imageName: 'gui/buttons/blue',
                    width: 136
                }),
                // icon options
                icon: icon,
                offsetWhenPressed: 2, // push pixels down when onButtonDown        
                // icon transparency when inactive
                iconAlpha: 0.5,
                onClick: function () {
                    onComplete(watchedAd);
                },
                components: [
                    new Anchor({
                        relative: new Vector2(0.5, 0.8),
                        onResize: function (isPortrait) {}
                    })
                ]
            });
            entity.attach(buttonNext);
            new Tween({
                from: buttonNext.position.x + viewport.width,
                to: buttonNext.position.x,
                in: 30,
                ease: 'elastic',
                decay: 8,
                oscillations: 1,
                onStart: function () {},
                onUpdate: function (v, t) {
                    buttonNext.position.x = v;
                }
            });*/

            popup.tweenOut(function () {
                onComplete(watchedAd);
            });
        };
        var showExtraProgressButton = function (popup) {
            var icon = new Entity({
                name: 'icon',
                components: [
                    new Text({
                        text: '+25%',
                        position: new Vector2(12, -8),
                        font: 'omnes',
                        fontSize: 24,
                        fontColor: '#ffffff',
                        align: 'center',
                        textBaseline: 'middle',
                        lineWidth: [4, 1.5], // set to add an outline
                        strokeStyle: [Globals.Colors.darkblue, Globals.Colors.lightblue],
                        strokeOffset: [new Vector2(0, 2), new Vector2(0, 0)],
                        margin: new Vector2(16, 16),
                        maxWidth: 100,
                        linebreaks: false,
                        drawDebug: false,
                        components: []
                    })
                ]
            });
            var loseItText = new Text({
                position: new Vector2(0, 48),
                text: Localization.getText('loseIt'),
                font: 'omnes',
                fontSize: 16,
                fontColor: '#ffffff',
                align: 'center',
                textBaseline: 'middle',
                linebreaks: false,
                drawDebug: false,
                boundingBox: new Rectangle(-64, -24, 128, 48),
                components: [
                    new Clickable({
                        onClick: function (data) {
                            buttonNext.removeSelf();
                            loseItText.removeSelf();
                            onComplete(watchedAd);

                            EventSystem.fire('GameAnalytics-addDesignEvent', {
                                eventId: 'UI:winScreen:skipExtraPercent'
                            });
                        }
                    })
                ]
            });
            var buttonNext = new IconButton({
                sprite: HdButtonSprite({
                    imageName: 'gui/buttons/blue-rv',
                    width: 144
                }),
                scale: new Vector2(1.25, 1.25),
                // icon options
                icon: icon,
                offsetWhenPressed: 2, // push pixels down when onButtonDown        
                // icon transparency when inactive
                iconAlpha: 0.5,
                onClick: function () {
                    EventSystem.fire('GameAnalytics-addDesignEvent', {
                        eventId: 'UI:winScreen:extraPercentButton'
                    });
                    if (WatchAd.canShowRewarded() || Bento.isDev()) {
                        EventSystem.fire('GameAnalytics-addDesignEvent', {
                            eventId: 'Ads:rv:winExtraPercent'
                        });
                        WatchAd.showRewarded({
                            tag: 'winExtraPercent',
                            onReward: function () {
                                watchedAd = true;
                                loseItText.removeSelf();
                                buttonNext.active = false;
                                popup.addProgress(0.25, function () {
                                    new Tween({
                                        from: buttonNext.position.x,
                                        to: buttonNext.position.x - viewport.width,
                                        in: 30,
                                        ease: 'easeInBack',
                                        onStart: function () {},
                                        onUpdate: function (v, t) {
                                            buttonNext.position.x = v;
                                        },
                                        onComplete: function () {
                                            onComplete(watchedAd);
                                        }
                                    });
                                });
                                EventSystem.fire('GameAnalytics-addDesignEvent', {
                                    eventId: 'Ads:rv:winExtraPercentComplete'
                                });
                            },
                            onComplete: function (error) {
                                // Video completed (reward state unknown)

                            }
                        });
                    }

                },
                components: [
                    new Anchor({
                        relative: new Vector2(0.5, 0.5),
                        offset: new Vector2(0, 160),
                        onResize: function (isPortrait) {}
                    }),
                    new WaitingForAd({
                        offset: new Vector2(12, 0)
                    })
                ]
            });
            new Tween({
                from: 0,
                to: 1,
                in: 30,
                ease: 'linear',
                delay: 90,
                onStart: function () {
                    buttonNext.attach(loseItText);
                },
                onUpdate: function (v, t) {
                    loseItText.alpha = v;
                }
            });
            entity.attach(buttonNext);
            new Tween({
                from: buttonNext.position.x + viewport.width,
                to: buttonNext.position.x,
                in: 30,
                ease: 'elastic',
                decay: 8,
                oscillations: 1,
                onStart: function () {},
                onUpdate: function (v, t) {
                    buttonNext.position.x = v;
                }
            });
        };
        var showExtraCoins = function () {
            var coinText = new Text({
                position: new Vector2(8, -1),
                text: '+150',
                font: 'omnes',
                fontSize: 32,
                fontColor: Globals.Colors.coin,
                strokeStyle: Globals.Colors.coinOutline,
                lineWidth: 3,
                align: 'right',
                textBaseline: 'middle',
                maxWidth: 64,
                linebreaks: false,
                drawDebug: false,
                components: []
            });
            var box = new Entity({
                name: 'box',
                scale: new Vector2(0.8, 0.8),
                components: [
                    // shine,
                    new Anchor({
                        relative: new Vector2(0.5, 0.8),
                        offset: new Vector2(0, -64 - 8),
                        onResize: function (isPortrait) {}
                    }),
                    new SpriteContainer({
                        height: 64,
                        imageName: 'gui/top/currency-box',
                    }),
                    coinText
                ]
            });
            var icon = new Entity({
                name: 'icon',
                components: [
                    new Text({
                        text: Localization.getText('coinBonus'),
                        position: new Vector2(14, 8),
                        font: 'omnes',
                        fontSize: 16,
                        fontColor: '#ffffff',
                        align: 'center',
                        textBaseline: 'middle',
                        lineWidth: [2], // set to add an outline
                        strokeStyle: [Globals.Colors.coinOutline],
                        strokeOffset: [new Vector2(0, 0)],
                        margin: new Vector2(16, 16),
                        maxWidth: 80,
                        linebreaks: false,
                        drawDebug: false,
                        components: []
                    }),
                    new Text({
                        text: Localization.getText('get'),
                        position: new Vector2(14, -8),
                        font: 'omnes',
                        fontSize: 24,
                        fontColor: '#ffffff',
                        align: 'center',
                        textBaseline: 'middle',
                        lineWidth: [2], // set to add an outline
                        strokeStyle: [Globals.Colors.coinOutline],
                        strokeOffset: [new Vector2(0, 0)],
                        margin: new Vector2(16, 16),
                        maxWidth: 80,
                        linebreaks: false,
                        drawDebug: false,
                        components: []
                    })
                ]
            });
            var loseItText = new Text({
                position: new Vector2(0, 48),
                text: Localization.getText('loseIt'),
                font: 'omnes',
                fontSize: 16,
                fontColor: '#ffffff',
                align: 'center',
                textBaseline: 'middle',
                linebreaks: false,
                drawDebug: false,
                boundingBox: new Rectangle(-64, -24, 128, 48),
                components: [
                    new Clickable({
                        onClick: function (data) {
                            loseItText.removeSelf();
                            buttonNext.removeSelf();
                            box.removeSelf();
                            onComplete(watchedAd);

                            EventSystem.fire('GameAnalytics-addDesignEvent', {
                                eventId: 'UI:winScreen:skipExtraCoins'
                            });
                        }
                    })
                ]
            });
            var buttonNext = new IconButton({
                sprite: HdButtonSprite({
                    imageName: 'gui/buttons/orange-rv',
                    width: 144
                }),
                scale: new Vector2(1.25, 1.25),
                // icon options
                icon: icon,
                offsetWhenPressed: 2, // push pixels down when onButtonDown        
                // icon transparency when inactive
                iconAlpha: 0.5,
                onClick: function () {
                    EventSystem.fire('GameAnalytics-addDesignEvent', {
                        eventId: 'UI:winScreen:extraCoinsButton'
                    });
                    if (WatchAd.canShowRewarded() || Bento.isDev()) {
                        EventSystem.fire('GameAnalytics-addDesignEvent', {
                            eventId: 'Ads:rv:winExtraCoins'
                        });
                        WatchAd.showRewarded({
                            tag: 'winExtraCoins',
                            onReward: function () {
                                watchedAd = true;
                                loseItText.removeSelf();
                                buttonNext.removeSelf();
                                box.removeSelf();
                                // give coins
                                Bento.objects.attach(new GotCoinsPopup({
                                    amount: 150, // note: this amount will be added to SaveValues immediately
                                    onComplete: function () {
                                        onComplete(watchedAd);
                                    }
                                }));
                                EventSystem.fire('GameAnalytics-addDesignEvent', {
                                    eventId: 'Ads:rv:winExtraCoinsComplete'
                                });

                            }
                        });
                    }

                },
                components: [
                    new Anchor({
                        relative: new Vector2(0.5, 0.8),
                        offset: new Vector2(0, 8),
                        onResize: function (isPortrait) {}
                    }),
                    new WaitingForAd({
                        offset: new Vector2(12, 0)
                    })
                ]
            });
            new Tween({
                from: 0,
                to: 1,
                in: 30,
                ease: 'linear',
                delay: 90,
                onStart: function () {
                    buttonNext.attach(loseItText);
                },
                onUpdate: function (v, t) {
                    loseItText.alpha = v;
                }
            });
            entity.attach(buttonNext);
            new Tween({
                from: buttonNext.position.x + viewport.width,
                to: buttonNext.position.x,
                in: 30,
                ease: 'elastic',
                decay: 8,
                oscillations: 1,
                delay: 10,
                applyOnDelay: true,
                onStart: function () {},
                onUpdate: function (v, t) {
                    buttonNext.position.x = v;
                }
            });

            entity.attach(box);
            new Tween({
                from: box.position.x + viewport.width,
                to: box.position.x,
                in: 30,
                ease: 'elastic',
                decay: 8,
                oscillations: 1,
                onStart: function () {},
                onUpdate: function (v, t) {
                    box.position.x = v;
                }
            });
        };

        return entity;
    };
    return WinPopup;
});