/**
 * Tutorial to teach player to draw
 * @moduleName DrawTutorial
 * @snippet DrawTutorial|Constructor
DrawTutorial({})
 */
bento.define('gui/tutorials/drawtutorial', [
    'bento',
    'bento/utils',
    'bento/entity',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/spritecontainer',
    'bento/tween',
    'onigiri/onigiri',
    'globals'
], function (
    Bento,
    Utils,
    Entity,
    Vector2,
    Rectangle,
    SpriteContainer,
    Tween,
    Onigiri,
    Globals
) {
    'use strict';
    var Dot = function (settings) {
        var entity = new Entity({
            z: Globals.Layers.GUI,
            name: 'dot',
            position: settings.position,
            scale: new Vector2(0, 0),
            rotation: settings.rotation || 0,
            components: [
                new SpriteContainer({
                    width: 16,
                    imageName: 'gui/dash',
                    originRelative: new Vector2(0.5, 0.5),
                    color: Globals.getLineColor()
                }),
                new Object({
                    name: 'behavior',
                    start: function (data) {},
                    destroy: function (data) {},
                    update: function (data) {
                        entity.scale.x = Utils.approach(entity.scale.x, 1, 1 / 4);
                        entity.scale.y = entity.scale.x;

                        if (entity.timer > 60) {
                            entity.alpha = Utils.approach(entity.alpha, 0, 1 / 10);
                            if (entity.timer > 120) {
                                entity.removeSelf();
                            }
                        }
                    },
                    draw: function (data) {}
                })
            ]
        });
        return entity;
    };
    var DrawTutorial = function (settings) {
        var startPos = settings.start || null;
        var targetPos = settings.target || null;
        var arcY = settings.arcY || 128; // 3rd bezier point y offset in pixels
        var curveTarget = settings.curveTarget || 0.9; // end of curve from 0 to 1
        var curve;
        var behavior = new Object({
            name: 'behavior',
            start: function (data) {
                // point from the ball to the hoop
                var start, end;
                var v0, v1, v2, v3;

                Utils.timeout(5, function () {
                    // we have to wait a cycle until the scene is drawn
                    if (startPos && targetPos) {
                        // start and end
                        Bento.objects.draw();
                        start = Onigiri.projectToBento(startPos.clone());
                        end = Onigiri.projectToBento(targetPos.clone());
                        // console.log(start);
                        v0 = new THREE.Vector2(start.x, start.y);
                        v3 = new THREE.Vector2(end.x, end.y);
                        // and anything in between
                        v1 = new THREE.Vector2(v0.x - 32, v3.y - arcY, 0);
                        v2 = new THREE.Vector2(v3.x, v3.y - arcY, 0);

                        curve = new THREE.CubicBezierCurve(v0, v1, v2, v3);

                        // start tweening
                        behavior.startTween();
                    }
                });

            },
            startTween: function () {
                var pos = new THREE.Vector2(0, 0);
                var ticker = 0;

                curve.getPointAt(0, pos);
                new Tween({
                    from: 0,
                    to: 1,
                    in: 30,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        hand.alpha = v;
                    }
                });
                // drop hand down
                new Tween({
                    from: pos.clone().add(new THREE.Vector2(16, 16)),
                    to: pos.clone(),
                    in: 30,
                    ease: 'squareroot',
                    onUpdate: function (v, t) {
                        hand.position = v;
                    },
                    onComplete: function () {}
                });

                // move hand
                new Tween({
                    from: 0,
                    to: curveTarget,
                    in: 120,
                    delay: 50,
                    ease: 'easeInOutQuad',
                    onStart: function () {},
                    onUpdate: function (v, t) {
                        curve.getPointAt(v, pos);
                        hand.position.x = pos.x;
                        hand.position.y = pos.y;

                        ticker += 1;

                        if (Math.floor(ticker) % 4 === 0) {
                            var point2 =  new THREE.Vector2(0, 0);
                            curve.getPointAt(v - 0.01, point2);

                            entity.attach(new Dot({
                                position: new Vector2(pos.x, pos.y),
                                rotation: Math.atan2(pos.y - point2.y, pos.x - point2.x)
                            }));
                            entity.moveComponentTo(hand, entity.components.length - 1);
                        }
                    },
                    onComplete: function () {
                        new Tween({
                            from: 1,
                            to: 0,
                            in: 30,
                            ease: 'linear',
                            onUpdate: function (v, t) {
                                hand.alpha = v;
                            },
                            onComplete: function () {
                                behavior.startTween();
                            }
                        });
                    }
                });
            }
        });
        var hand = new Entity({
            name: 'hand',
            position: new Vector2(0, 0),
            alpha: 0,
            components: [
                new SpriteContainer({
                    width: 64,
                    imageName: 'gui/hand',
                    originRelative: new Vector2(0, 0),
                })
            ]
        });
        var entity = new Entity({
            name: 'drawTutorial',
            position: new Vector2(0, 0),
            components: [
                behavior,
                hand
            ]
        });
        return entity;
    };
    return DrawTutorial;
});