/**
 * Tutorial on passing the ball around
 * @moduleName PassTutorial
 * @snippet PassTutorial|Constructor
PassTutorial({})
 */
bento.define('gui/tutorials/passtutorial', [
    'bento',
    'bento/utils',
    'gui/tutorials/drawtutorial',
    'bento/entity',
    'bento/components/eventlistener'
], function (
    Bento,
    Utils,
    DrawTutorial,
    Entity,
    EventListener
) {
    'use strict';
    var PassTutorial = function (settings) {
        var tutorial1;
        var tutorial2;
        var entity = new Entity({
            name: 'passTutorial',
            components: [
                new Object({
                    name: 'behavior',
                    start: function (data) {
                        // show this tutorial only once
                        if (Bento.saveState.load('passTutorial', false)) {
                            return;
                        }
                        Bento.saveState.save('passTutorial', true);

                        // start with showing draw tutorial between players
                        var players = Bento.objects.getByFamily('players');
                        tutorial1 = new DrawTutorial({
                            start: players[0].position.clone().add(new THREE.Vector3(2, 3, 0)),
                            target: players[1].position.clone().add(new THREE.Vector3(-1, 3, 0)),
                            arcY: -16
                        });
                        Bento.objects.attach(tutorial1);

                        // wait for pass
                        entity.attach(new EventListener({
                            name: 'caughtListener',
                            eventName: 'caughtBall',
                            ignorePause: false,
                            onEvent: function () {
                                tutorial1.removeSelf();
                                // start 2nd tutorial
                                // try to get ball and basket directly
                                Bento.objects.get('ball', function (ball) {
                                    Bento.objects.get('basketHoop', function (hoop) {
                                        tutorial2 = new DrawTutorial({
                                            start: ball.position.clone(),
                                            target: hoop.position.clone(),
                                            curveTarget: 0.5
                                        });
                                        Bento.objects.attach(tutorial2);
                                    });
                                });
                            }
                        }));
                    },
                    destroy: function (data) {
                        if (tutorial1) {
                            tutorial1.removeSelf();
                        }
                        if (tutorial2) {
                            tutorial2.removeSelf();
                        }
                    },
                })
            ]
        });
        return entity;
    };
    return PassTutorial;
});