/**
 * Tutorial 
 * @moduleName PauseTutorial
 * @snippet PauseTutorial|Constructor
PauseTutorial({})
 */
bento.define('gui/tutorials/pausetutorial', [
    'bento',
    'bento/utils',
    'bento/entity',
    'bento/math/vector2',
    'bento/math/rectangle',
    'modules/box2d',
    'onigiri/onigiri',
    'bento/tween',
    'bento/components/eventlistener',
    'gui/widgets/taphand',
    'globals',
    'bento/gui/text',
    'bento/components/spritecontainer',
    'modules/localization'
], function (
    Bento,
    Utils,
    Entity,
    Vector2,
    Rectangle,
    Box2D,
    Onigiri,
    Tween,
    EventListener,
    TapHand,
    Globals,
    Text,
    SpriteContainer,
    Localization
) {
    'use strict';
    var PauseTutorial = function (settings) {
        var position = settings.position.clone() || new Vector2(0, 0);
        var levelScale = settings.levelScale || 1;
        var dimension = settings.dimension.clone();
        var viewport = Bento.getViewport();
        /**
         * Tutorial 1: tap to stop ball
         */
        var tapHand1 = new TapHand({
            imageName: 'gui/hand',
            position: new Vector2(16, -32),
            upPosition: new Vector2(0, -30),
            speed: 1,
            startInUpPosition: false,
            onTap: function () {}
        });
        var text1 = new Text({
            position: new Vector2(0, 0),
            text: Globals.tapStyle === 'tap' ? Localization.getText('tapToStop') : Localization.getText('holdToStop'),
            font: 'omnes',
            fontSize: 24,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 226,
            linebreaks: false,
            drawDebug: false,
            components: []
        });
        var textBox = new SpriteContainer({
            width: 256,
            imageName: 'gui/status-back',
        });
        var tutorial1Container = new Entity({
            z: Globals.Layers.POPUPS,
            name: 'tutorial1',
            // position: new Vector2(viewport.width / 2, viewport.height / 3 * 2),
            position: new Vector2(viewport.width / 2, viewport.height / 3),
            components: [
                textBox,
                tapHand1,
                text1
            ]
        });
        var slomoTween;

        var tut1Listeners = [
            new EventListener({
                name: 'pauseListener',
                eventName: 'pauseBall',
                ignorePause: false,
                onEvent: function (data) {
                    stopTutorial1();

                    // show 2nd tutorial: tap again to resume
                    if (Globals.tapStyle === 'tap') {
                        Utils.timeout(120, function () {
                            Bento.objects.attach(tutorial2Container);
                            Utils.forEach(tut2Listeners, function (listener) {
                                entity.attach(listener);
                            });
                        });
                    }
                }
            }),
            new EventListener({
                name: 'missListener',
                eventName: 'releasedBall',
                ignorePause: false,
                onEvent: function (data) {
                    // oops?
                    console.log('released ball');
                    stopTutorial1();
                }
            }),
            new EventListener({
                name: 'winListener',
                eventName: 'dunkedBall',
                ignorePause: false,
                onEvent: function (data) {
                    stopTutorial1();
                }
            })
        ];

        var stopTutorial1 = function () {
            Bento.setGameSpeed(1);
            if (slomoTween) {
                slomoTween.stop();
                slomoTween.removeSelf();
            }
            // remove hand
            tutorial1Container.removeSelf();

            Utils.forEach(tut1Listeners, function (listener) {
                entity.remove(listener);
            });
        };
        /**
         * Tutorial 2: tap again to resume
         */
        var tapHand2 = new TapHand({
            imageName: 'gui/hand',
            position: new Vector2(16, -32),
            upPosition: new Vector2(0, -30),
            speed: 1,
            startInUpPosition: false,
            onTap: function () {}
        });
        var text2 = new Text({
            position: new Vector2(0, 0),
            text: Localization.getText('tapToResume'),
            font: 'omnes',
            fontSize: 24,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 226,
            linebreaks: false,
            drawDebug: false,
            components: []
        });
        var textBox2 = new SpriteContainer({
            width: 256,
            imageName: 'gui/status-back',
        });
        var tutorial2Container = new Entity({
            z: Globals.Layers.POPUPS,
            name: 'tutorial2',
            position: new Vector2(viewport.width / 2, viewport.height / 3),
            components: [
                textBox2,
                tapHand2,
                text2
            ]
        });

        var tut2Listeners = [
            new EventListener({
                name: 'pauseListener',
                eventName: 'resumeBall',
                ignorePause: false,
                onEvent: function (data) {
                    stopTutorial2();
                }
            }),
            new EventListener({
                name: 'winListener',
                eventName: 'dunkedBall',
                ignorePause: false,
                onEvent: function (data) {
                    stopTutorial2();
                }
            })
        ];
        var stopTutorial2 = function () {
            // remove hand
            tutorial2Container.removeSelf();

            Utils.forEach(tut2Listeners, function (listener) {
                entity.remove(listener);
            });
        };

        /**
         * Box2d trigger: go into slowmo here and show tutorial 1
         */
        var body = new Box2D.Body({
            type: 'static',
            fixture: {
                isSensor: true,
                shape: new Box2D.BoxShape(
                    dimension.width / 2 * Onigiri.pxToThree * levelScale,
                    dimension.height / 2 * Onigiri.pxToThree * levelScale,
                    new Vector2(
                        (dimension.width / 2 + dimension.x) * Onigiri.pxToThree * levelScale,
                        (dimension.height / 2 + dimension.y) * Onigiri.pxToThree * -levelScale
                    )
                ),
                friction: 0,
                restitution: 0,
                density: 0.3,
                onBeginContact: function (contact) {
                    if (
                        contact &&
                        contact.entity &&
                        contact.entity.name === 'ball' &&
                        contact.entity.isFollowingPath
                    ) {
                        // console.log('begin', contact);
                        // remove bounding box
                        Box2D.defer(function () {
                            entity.remove(body);
                        });

                        // start tutorial
                        console.log('Start pause tutorial');
                        slomoTween = new Tween({
                            from: 1,
                            to: 0.05,
                            in: 30,
                            ease: 'easeOutExpo',
                            ignoreGameSpeed: true,
                            onUpdate: function (v, t) {
                                Bento.setGameSpeed(v);
                            },
                            onComplete: function () {
                                slomoTween = null;
                            }
                        });

                        Bento.objects.attach(tutorial1Container);

                        Utils.forEach(tut1Listeners, function (listener) {
                            entity.attach(listener);
                        });
                    }
                },
            }
        });
        var behavior = new Object({
            name: 'behavior',
            start: function (data) {
                // show this tutorial only once
                if (Bento.saveState.load('pauseTutorial', false) && !Bento.isDev()) {
                    entity.removeSelf();
                    return;
                }
            },
            destroy: function (data) {
                Bento.setGameSpeed(1);
            },
            update: function (data) {},
            draw: function (data) {}
        });
        var entity = new Entity({
            name: 'pauseTutorial',
            position: new Vector2(
                position.x * Onigiri.pxToThree * levelScale,
                position.y * Onigiri.pxToThree * levelScale
            ),
            components: [
                body,
                behavior,
                new EventListener({
                    name: 'winListener',
                    eventName: 'dunkedBall',
                    ignorePause: false,
                    onEvent: function (data) {
                        // assume they understand at this point
                        Bento.saveState.save('pauseTutorial', true);
                    }
                })
            ]
        });
        return entity;
    };
    return PauseTutorial;
});