/**
 * Fades out
 * @moduleName Fade
 * @snippet Fade|constructor
Fade({
    z: Globals.Layers.FADE
})
 */
bento.define('gui/widgets/fade', [
    'bento',
    'bento/utils',
    'bento/components/fill',
    'bento/math/vector2',
    'bento/entity',
    'bento/tween'
], function (
    Bento,
    Utils,
    Fill,
    Vector2,
    Entity,
    Tween
) {
    'use strict';
    var Fade = function (settings) {
        var viewport = Bento.getViewport().getSize();
        var entity = new Entity({
            z: settings.z,
            name: 'fade',
            float: true,
            components: [
                new Fill({
                    name: 'fill',
                    dimension: viewport.grow(2),
                    color: [0, 0, 0, 1], // [1, 1, 1, 1] is pure white
                    originRelative: new Vector2(0, 0)
                })
            ]
        }).extend({
            /**
             * @snippet #Fade.fadeOut()|snippet
                fadeOut({
                    onComplete: function () {$1}
                })
             */
            fadeOut: function (data) {
                new Tween({
                    from: entity.alpha,
                    to: 0,
                    in: data.time || 30,
                    ease: 'linear',
                    updateWhenPaused: 10000,
                    ignoreGameSpeed: true,
                    onUpdate: function (v, t) {
                        entity.alpha = v;
                    },
                    onComplete: function () {
                        entity.removeSelf();
                        if (data.onComplete) {
                            data.onComplete();
                        }
                    }
                });
            },
            /**
             * @snippet #Fade.fadeIn()|snippet
                fadeIn({
                    time: 30,
                    onComplete: function () {$1}
                })
             */
            fadeIn: function (data) {
                if (!entity.isAdded) {
                    Bento.objects.attach(entity);
                }
                new Tween({
                    from: 0,
                    to: 1,
                    in: data.time || 30,
                    ease: 'linear',
                    updateWhenPaused: 10000,
                    ignoreGameSpeed: true,
                    onUpdate: function (v, t) {
                        entity.alpha = v;
                    },
                    onComplete: function () {
                        entity.removeSelf();
                        if (data.onComplete) {
                            data.onComplete();
                        }
                    }
                });
            }
        });
        return entity;
    };
    return Fade;
});