/**
 * Circles to indicate loading
 * @moduleName LoadingSpinner
 * @snippet LoadingSpinner|constructor
LoadingSpinner({
    name: 'loadingSpinner',
    radius: ${1:10},
    fadeSpeed: ${2:1 / 60},
    ballsToSpawn: ${3:8},
    rotateBalls: ${4:false},
    imageName: '${5:dot}',
    position: new Vector2(${6:0}, ${7:0}),
    components: [] 
})
 */
bento.define('gui/widgets/loadingspinner', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/utils',
    'bento/components/spritecontainer'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    Utils,
    SpriteContainer
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var radius = settings.radius || 10;
        var fadeSpeed = settings.fadeSpeed || 1 / 60;
        var ballsToSpawn = settings.balls || 8;
        var rotateBalls = settings.rotateBalls || false;
        var imageName = settings.imageName || '';
        var entity = new Entity({
            z: settings.z,
            name: settings.name || 'loadingSpinner',
            scale: settings.scale,
            position: settings.position,
            components: settings.components
        });
        var balls = [];
        var makeBall = function () {
            var sprite = new SpriteContainer({
                imageName: imageName,
                width: 8,
                originRelative: new Vector2(0.5, 0.5)
            });
            var behavior = {
                name: 'alphaBehavior',
                timer: 0,
                fadeSpeed: fadeSpeed,
                start: function () {
                    // count balls, set timer offset
                    var ballCount = balls.length;
                    var index = balls.indexOf(ball);
                    behavior.timer = index / ballCount;
                },
                update: function () {
                    behavior.timer += behavior.fadeSpeed;
                    behavior.timer %= 1;
                    ball.alpha = 1 - behavior.timer;
                }
            };
            var ball = new Entity({
                name: 'ball',
                position: new Vector2(0, 0),
                components: [
                    sprite,
                    behavior
                ]
            });
            balls.push(ball);
            entity.attach(ball);
            return ball;
        };
        var positionBalls = function () {
            var ballCount = balls.length;
            var radialDistance = Math.PI * 2 / ballCount;
            Utils.forEach(balls, function (ball, i, l, breakLoop) {
                var angle = -radialDistance * i;
                ball.position.x = radius * Math.cos(angle);
                ball.position.y = radius * Math.sin(angle);
                if (rotateBalls) {
                    // rotate balls themselves
                    ball.rotation = angle;
                }
            });
        };

        // spawn balls
        Utils.repeat(makeBall, ballsToSpawn);
        // position balls in circle
        positionBalls();

        return entity;
    };
});