/**
 * Save values using getters and setters
 * @moduleName SaveValues
 * @snippet saveSnippet|SaveValueSnippet
/$4* @snippet SaveValues.$1|${2:Number} *$4/
Object.defineProperty(SaveValues, '$1', {
    get: function () {
        if (localValues.$1 === undefined) {
            // default value
            localValues.$1 = Bento.saveState.load('$1', ${3:0});
        }
        return localValues.$1;
    },
    set: function (value) {
        localValues.$1 = value;
        Bento.saveState.save('$1', value);
    }
});
 */

bento.define('managers/savevalues', [
    'bento',
    'bento/utils',
    'bento/eventsystem'
], function (
    Bento,
    Utils,
    EventSystem
) {
    'use strict';
    var localValues = {};
    var SaveValues = {
        // note: this makes "clear" a reserved string for savevalues
        clear: function () {
            Bento.saveState.clear();
            localValues = {};
        }
    };

    /* @snippet SaveValues.coins|Number */
    Object.defineProperty(SaveValues, 'coins', {
        get: function () {
            if (localValues.coins === undefined) {
                // default value
                localValues.coins = Bento.saveState.load('coins', 0);
            }
            return localValues.coins;
        },
        set: function (value) {
            localValues.coins = value;
            Bento.saveState.save('coins', value);
            EventSystem.fire('updateCoins');
        }
    });

    /* @snippet SaveValues.keys|Number */
    Object.defineProperty(SaveValues, 'keys', {
        get: function () {
            if (localValues.keys === undefined) {
                // default value
                localValues.keys = Bento.saveState.load('keys', 0);
            }
            return localValues.keys;
        },
        set: function (value) {
            localValues.keys = value;
            Bento.saveState.save('keys', value);
        }
    });


    /* @snippet SaveValues.dailyBonusKey|String */
    Object.defineProperty(SaveValues, 'dailyBonusKey', {
        get: function () {
            if (localValues.dailyBonusKey === undefined) {
                // default value
                localValues.dailyBonusKey = Bento.saveState.load('dailyBonusKey', 0);
            }
            return localValues.dailyBonusKey;
        },
        set: function (value) {
            localValues.dailyBonusKey = value;
            Bento.saveState.save('dailyBonusKey', value);
        }
    });


    /* @snippet SaveValues.dailyBonusDay|String */
    Object.defineProperty(SaveValues, 'dailyBonusDay', {
        get: function () {
            if (localValues.dailyBonusDay === undefined) {
                // default value
                localValues.dailyBonusDay = Bento.saveState.load('dailyBonusDay', 0);
            }
            return localValues.dailyBonusDay;
        },
        set: function (value) {
            localValues.dailyBonusDay = value;
            Bento.saveState.save('dailyBonusDay', value);
        }
    });

    /* @snippet SaveValues.currentLevel|Number */
    Object.defineProperty(SaveValues, 'currentLevel', {
        get: function () {
            if (localValues.currentLevel === undefined) {
                // default value
                localValues.currentLevel = Bento.saveState.load('currentLevel', 1);
            }
            return localValues.currentLevel;
        },
        set: function (value) {
            localValues.currentLevel = value;
            Bento.saveState.save('currentLevel', value);
        }
    });

    /* @snippet SaveValues.currentBall|String */
    Object.defineProperty(SaveValues, 'currentBall', {
        get: function () {
            if (localValues.currentBall === undefined) {
                // default value
                localValues.currentBall = Bento.saveState.load('currentBall', 'default');
            }
            return localValues.currentBall;
        },
        set: function (value) {
            localValues.currentBall = value;
            Bento.saveState.save('currentBall', value);
        }
    });

    /* @snippet SaveValues.dailyBonusBall|String */
    Object.defineProperty(SaveValues, 'dailyBonusBall', {
        get: function () {
            if (localValues.dailyBonusBall === undefined) {
                // default value
                localValues.dailyBonusBall = Bento.saveState.load('dailyBonusBall', 'default');
            }
            return localValues.dailyBonusBall;
        },
        set: function (value) {
            localValues.dailyBonusBall = value;
            Bento.saveState.save('dailyBonusBall', value);
        }
    });

    /* @snippet SaveValues.unlockedBalls|ArrayOfStrings */
    Object.defineProperty(SaveValues, 'unlockedBalls', {
        get: function () {
            if (localValues.unlockedBalls === undefined) {
                // default value
                localValues.unlockedBalls = Bento.saveState.load('unlockedBalls', ['default']);
            }
            return localValues.unlockedBalls;
        },
        set: function (value) {
            localValues.unlockedBalls = value;
            Bento.saveState.save('unlockedBalls', value);
        }
    });

    /* @snippet SaveValues.currentPlayer|String */
    Object.defineProperty(SaveValues, 'currentPlayer', {
        get: function () {
            if (localValues.currentPlayer === undefined) {
                // default value
                localValues.currentPlayer = Bento.saveState.load('currentPlayer', 'default');
            }
            return localValues.currentPlayer;
        },
        set: function (value) {
            localValues.currentPlayer = value;
            Bento.saveState.save('currentPlayer', value);
        }
    });
    /* @snippet SaveValues.unlockedPlayers|ArrayOfStrings */
    Object.defineProperty(SaveValues, 'unlockedPlayers', {
        get: function () {
            if (localValues.unlockedPlayers === undefined) {
                // default value
                localValues.unlockedPlayers = Bento.saveState.load('unlockedPlayers', ['default']);
            }
            return localValues.unlockedPlayers;
        },
        set: function (value) {
            localValues.unlockedPlayers = value;
            Bento.saveState.save('unlockedPlayers', value);
        }
    });

    /* @snippet SaveValues.unlockProgress|Number */
    Object.defineProperty(SaveValues, 'unlockProgress', {
        get: function () {
            if (localValues.unlockProgress === undefined) {
                // default value
                localValues.unlockProgress = Bento.saveState.load('unlockProgress', 0);
            }
            return localValues.unlockProgress;
        },
        set: function (value) {
            localValues.unlockProgress = value;
            Bento.saveState.save('unlockProgress', value);
        }
    });

    /* @snippet SaveValues.lastTab|String */
    Object.defineProperty(SaveValues, 'lastTab', {
        get: function () {
            if (localValues.lastTab === undefined) {
                // default value
                localValues.lastTab = Bento.saveState.load('lastTab', 'balls');
            }
            return localValues.lastTab;
        },
        set: function (value) {
            localValues.lastTab = value;
            Bento.saveState.save('lastTab', value);
        }
    });

    /* @snippet SaveValues.nextCharacter|String */
    Object.defineProperty(SaveValues, 'nextCharacter', {
        get: function () {
            if (localValues.nextCharacter === undefined) {
                // default value
                localValues.nextCharacter = Bento.saveState.load('nextCharacter', 'default');
            }
            return localValues.nextCharacter;
        },
        set: function (value) {
            localValues.nextCharacter = value;
            Bento.saveState.save('nextCharacter', value);
        }
    });

    /* @snippet SaveValues.dailyBonusCharacter|String */
    Object.defineProperty(SaveValues, 'dailyBonusCharacter', {
        get: function () {
            if (localValues.dailyBonusCharacter === undefined) {
                // default value
                localValues.dailyBonusCharacter = Bento.saveState.load('dailyBonusCharacter', 'default');
            }
            return localValues.dailyBonusCharacter;
        },
        set: function (value) {
            localValues.dailyBonusCharacter = value;
            Bento.saveState.save('dailyBonusCharacter', value);
        }
    });

    /* @snippet SaveValues.levelsUntilBonus|Number */
    Object.defineProperty(SaveValues, 'levelsUntilBonus', {
        get: function () {
            if (localValues.levelsUntilBonus === undefined) {
                // default value
                localValues.levelsUntilBonus = Bento.saveState.load('levelsUntilBonus', 5);
            }
            return localValues.levelsUntilBonus;
        },
        set: function (value) {
            localValues.levelsUntilBonus = value;
            Bento.saveState.save('levelsUntilBonus', value);
        }
    });

    /* @snippet SaveValues.bonusLevelsCleared|Number */
    Object.defineProperty(SaveValues, 'bonusLevelsCleared', {
        get: function () {
            if (localValues.bonusLevelsCleared === undefined) {
                // default value
                localValues.bonusLevelsCleared = Bento.saveState.load('bonusLevelsCleared', 0);
            }
            return localValues.bonusLevelsCleared;
        },
        set: function (value) {
            localValues.bonusLevelsCleared = value;
            Bento.saveState.save('bonusLevelsCleared', value);
        }
    });

    return SaveValues;
});