/**
 * Firebase Analytics module
 * @moduleName FirebaseAnalytics
 * @snippet FirebaseAnalytics.logEvent|Snippet
FirebaseAnalytics.logEvent("${1:event_name}",{
    ${2:property_name}: ${3:"serializable_value"}
});
 * @snippet FirebaseAnalytics.logEvent|Event
EventSystem.fire("FirebaseAnalytics-logEvent", {
    name: "${1:event_name}",
    properties: {
        ${2:property_name}: "${3:serializable_value}"
    }
});
 * @snippet FirebaseAnalytics.setScreenName|Snippet
FirebaseAnalytics.setScreenName("${1:screen_name}");
 * @snippet FirebaseAnalytics.setScreenName|Event
EventSystem.fire("FirebaseAnalytics-setScreenName", "${1:screen_name}");
 * @snippet FirebaseAnalytics.setUserId|Snippet
FirebaseAnalytics.setUserId("${1:user_id}");
 * @snippet FirebaseAnalytics.setUserId|Event
EventSystem.fire("FirebaseAnalytics-setUserId", "${1:user_id}");
 * @snippet FirebaseAnalytics.setUserProperty|Snippet
FirebaseAnalytics.setUserProperty("${1:property_name}", "${2:serializable_value}");
 * @snippet FirebaseAnalytics.setUserProperty|Event
EventSystem.fire("FirebaseAnalytics-setUserProperty", {
    property: "${1:property_name}",
    value: "${2:serializable_value}"
});
 */
bento.define('modules/firebaseanalytics', [
    'bento',
    'bento/eventsystem',
    'bento/utils',
], function (
    Bento,
    EventSystem,
    Utils
) {
    'use strict';
    var nothing = function () {
        //nothing     
    };
    var enabled = true;
    var FirebasePlugin = window.FirebasePlugin;
    //empty module
    var FirebaseAnalytics = {
        logEvent: nothing,
        setScreenName: nothing,
        setUserId: nothing,
        setUserProperty: nothing,
        init: nothing,
        setEnabled: function (newEnabled) {
            enabled = newEnabled;
        }
    };
    //return the empty module if plugin is missing
    if (!FirebasePlugin) {
        return FirebaseAnalytics;
    }
    // replace module methods since we have the plugin
    FirebaseAnalytics.logEvent = function (eventName, eventProperties) {
        FirebasePlugin.logEvent(eventName, eventProperties);
    };
    FirebaseAnalytics.setScreenName = function (screenName) {
        FirebasePlugin.setScreenName(screenName);
    };
    FirebaseAnalytics.setUserId = function (userId) {
        FirebasePlugin.setUserId(userId);
    };
    FirebaseAnalytics.setUserProperty = function (propertyName, propertyValue) {
        FirebasePlugin.setUserProperty(propertyName, propertyValue);
    };
    /**
     * @snippet FirebaseAnalytics.init()|Snippet
    FirebaseAnalytics.init()
     */
    FirebaseAnalytics.init = function () {
        //register events
        EventSystem.on('FirebaseAnalytics-logEvent', function (data) {
            if (enabled) {
                FirebaseAnalytics.logEvent(data.name, data.properties);
            }
        });
        EventSystem.on('FirebaseAnalytics-setScreenName', function (data) {
            if (enabled) {
                FirebaseAnalytics.setScreenName(data);
            }
        });
        EventSystem.on('FirebaseAnalytics-setUserId', function (data) {
            if (enabled) {
                FirebaseAnalytics.setUserId(data);
            }
        });
        EventSystem.on('FirebaseAnalytics-setUserProperty', function (data) {
            if (enabled) {
                FirebaseAnalytics.setUserProperty(data.property, data.value);
            }
        });
        EventSystem.on('FirebaseAnalytics-setEnabled', function (data) {
            if (enabled) {
                FirebaseAnalytics.setEnabled(data);
            }
        });
        // console log
        console.log("Firebase Initialised!");
    };
    return FirebaseAnalytics;
});