/* jshint esversion:6*/
/**
 * Helper module to schedule notifications. Game specific.
 * @moduleName NotificationScheduler 
 * @snippet NotificationScheduler.init()|Snippet
NotificationScheduler.init()
 */
bento.define('modules/notificationscheduler', [
    'bento',
    'bento/eventsystem',
    'bento/utils',
    'modules/notifications'
], function (
    Bento,
    EventSystem,
    Utils,
    Notifications
) {
    'use strict';
    var onSchedule = function (settings) {
        var id = settings.id;
        var time = settings.time;
        var title = settings.title;
        var body = settings.body;
        var savedNotifications = Bento.saveState.load('notifications', {});

        if (!body) {
            Utils.log("ERROR: cannot schedule notification without body text");
            return;
        }
        if (!title) {
            Utils.log("ERROR: cannot schedule notification without title text");
            return;
        }

        // reset badge
        Notifications.resetBadgeNumber();

        // cancel old notification if needed
        var oldId = savedNotifications[id];
        if (oldId) {
            Notifications.cancel(oldId);
        }
        console.log(`scheduling notification: ${id} ${title} - ${body}`);

        // schedule notification
        Notifications.send({
            title: title,
            text: body,
            deltaTime: time
        }, function (notificationId) {
            // save id so it can be cancelled if needed
            savedNotifications[id] = notificationId;
            Bento.saveState.save('notifications', savedNotifications);
            console.log(`Scheduled notification: ${id}, ${notificationId}`);
        });
    };

    return {
        init: function () {
            EventSystem.on('scheduleNotification', onSchedule);
        },

    /**
     * @snippet Notification|schedule
EventSystem.fire('scheduleEvent', {
    id: '$1',
    title: Localization.getText('notifications')['$1Title'],
    body: Localization.getText('notifications')['$1Body'],
    time: ${2:24*60*60*1000}
});
     */
        schedule: onSchedule
    };
});