/**
 * Deterministic randomizer based on seed
 * @moduleName Randomizer
 */
bento.define('modules/randomizer', [
    'bento',
    'bento/utils',
    'modules/mersennetwister'
], function (
    Bento,
    Utils,
    MersenneTwister
) {
    'use strict';
    var Randomizer = function (settings) {
        var seed = Utils.isDefined(settings.seed) ? settings.seed : Date.now();
        var randomizer = new MersenneTwister(seed);

        return {
            seed: seed,
            /* @snippet Randomizer.getSeed()|Number */
            getSeed: function () {
                return this.seed;
            },
            /**
             * @snippet Randomizer.setSeed()|Randomizer
                Randomizer.setSeed(${1:0})
             */
            setSeed: function (value) {
                this.seed = value;
                randomizer = new MersenneTwister(value);
                return randomizer;
            },
            /* @snippet Randomizer.getRandom()|Number */
            getRandom: function (n) {
                return Math.floor(randomizer.random() * n);
            },
            /**
             * @snippet Randomizer.getRandomRange()|Number
            Randomizer.getRandomRange(${1:min}, ${2:max})
             */
            getRandomRange: function (min, max) {
                var diff = max - min;
                return min + Math.floor(randomizer.random() * diff);
            },
            /**
             * @snippet Randomizer.getRandomFloat()|Number
            Randomizer.getRandomFloat(${1:0})
             */
            getRandomFloat: function (n) {
                return randomizer.random() * n;
            },
            /**
             * @snippet Randomizer.getRandomRangeFloat()|Number
            Randomizer.getRandomRangeFloat(${1:min}, ${2:max})
             */
            getRandomRangeFloat: function (min, max) {
                var diff = max - min;
                return min + randomizer.random() * diff;
            }
        };
    };

    return Randomizer;
});