/**
 * Wrapper around IronSource (or other) ad calls
 * @moduleName WatchAd
 */
bento.define('modules/watchad', [
    'bento',
    'bento/utils',
    'modules/ironsource'
], function (
    Bento,
    Utils,
    IronSource
) {
    'use strict';
    var WatchAd = {
        /**
         * @snippet WatchAd.showRewarded()|snippet
            WatchAd.showRewarded({
                tag: '$1',
                onReward: function () {
                    $2
                },
                onComplete: function (error) {
                    // Video completed (reward state unknown)
                    $3
                }
            });
         */
        showRewarded: function (settings, isInterstitial) {
            var didComplete = false;
            var didReward = false;
            var onComplete = function () {
                didComplete = true;
                if (settings.onComplete) {
                    settings.onComplete();
                }

                // was waiting for onComplete
                if (didReward && settings.onReward) {
                    settings.onReward();
                }
            };
            var onReward = function () {
                didReward = true;
                if (didComplete && settings.onReward) {
                    settings.onReward();
                }
                // else wait for onComplete
            };

            if (Bento.isDev()) {
                window.alert('Watching rewarded.');
                if (settings.onReward) {
                    settings.onReward();
                }
                if (settings.onComplete) {
                    settings.onComplete();
                }
                return;
            }
            IronSource.showRewarded({
                tag: settings.tag,
                muteAudio: true,
                onReward: onReward,
                onComplete: onComplete
            });

        },
        /**
         * @snippet WatchAd.showInterstitial()|snippet
            WatchAd.showInterstitial({
                tag: '$1',
                onComplete: function (error) {
                    // Interstitial video completed (or failed)
                    if (error) {
                        console.log('WARNING: interstitial failed to show ' + error);
                    }
                }
            });
         */
        showInterstitial: function (settings) {
            if (Bento.isDev()) {
                console.log('Watching interstitial.');
                settings.onComplete();
                return;
            }
            IronSource.showInterstitial({
                tag: settings.tag,
                muteAudio: true,
                onComplete: settings.onComplete || function () {}
            });
        },
        /* @snippet WatchAd.canShowRewarded()|Boolean */
        canShowRewarded: function () {
            return IronSource.canShowRewarded();
        },
        /* @snippet WatchAd.canShowInterstitial()|Boolean */
        canShowInterstitial: function () {
            return IronSource.canShowInterstitial();
        }
    };
    return WatchAd;
});