/**
 * Main screen
 */
bento.define('screens/main', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/screen',
    'bento/tween',
    'entities/luckykatlogo',
    'globals',
    'components/sun',
    'onigiri/onigiri',
    'onigiri/primitive',
    'managers/levelspawner',
    'modules/box2d',
    'onigiri/entity3d',
    'entities/gamecontroller',
    'entities/drawcontroller',
    'entities/floor',
    'entities/backgroundimage',
    'managers/savevalues',
    'gui/widgets/fade',
    'modules/banner',
    'gui/overlays/mainguioverlay'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Screen,
    Tween,
    LuckyKat,
    Globals,
    Sun,
    Onigiri,
    Primitive,
    LevelSpawner,
    Box2D,
    Entity3D,
    GameController,
    DrawController,
    Floor,
    BackgroundImage,
    SaveValues,
    Fade,
    BannerAd,
    MainGuiOverlay
) {
    'use strict';
    var onShow = function () {
        var onigiri = new Onigiri({
            threeToPx: 32,
            // backgroundColor: '#33ddff',
            camera: {
                cameraStyle: 'perspective',
                perspectiveFieldOfView: 45
            },
            // shadows: true
        });
        var background = new BackgroundImage({
            imageName: Globals.getBackground()
        });
        Box2D.setup({
            meterToPixels: 1,
            gravity: new Vector2(0, -48),
            canSleep: true,
            fixedSpeed: true,
            addNow: true
        });
        var levelSpawner = new LevelSpawner({
            assetName: (function () {
                var puzzle = Utils.getHashParameter('level', GameController.getPuzzle());
                if (!Bento.assets.hasAsset(puzzle, 'json') && Utils.getHashParameter('level')) {
                    // for non-developers: force load the json
                    Bento.assets.loadJsonFromUrl(puzzle, 'assets/main/json/' + puzzle + '.json', function (error) {
                        if (!error) {
                            // try again 
                            Bento.screens.show('screens/main');
                        }
                    });
                }
                return puzzle;
            })(),
            depth: 3,
            onComplete: function () {}
        });
        var sun = new Sun({
            // debug: true,
            color: '#fff',
            directionalIntensity: 0.4,
            ambientIntensity: 0.65,
            position: new THREE.Vector3(0, 100 / 6 + levelSpawner.size.height / 2 * 0.75, 85 / 6),
            targetPosition: new THREE.Vector3(0, levelSpawner.size.height / 2, 0),
            castShadow: true,
            shadowSize: levelSpawner.levelScale * 4
        });
        var floor = new Floor({
            depth: 1
        });
        var gameController = new GameController({});
        var guiOverlay = new MainGuiOverlay({});
        var setupCamera = function () {
            // set up camera (hardcoded in such a way the level always fits on screen in mobile)
            var posY = levelSpawner.size.height / 2 - (Bento.saveState.load('cheat-lowerCamera', false) ? 30 / 6 : 10 / 6) + levelSpawner.cameraYOffset / 6;
            var posZ = 32 * levelSpawner.levelScale;

            // lower camera a bit on iphone X
            if (true || Utils.isIphoneX()) { // always lower to keep some space on the bottom?
                posY -= 20 / 6;
            }

            Onigiri.camera.position.set(0, posY, posZ);
            // draw screen asap to snap camera
            Bento.objects.draw();
        };
        // spawn draw controller
        var drawController = new DrawController({
            drawDistanceStart: 4,
            maxDistance: Utils.getSafe(levelSpawner.json, ['properties', 'maxDistance'], 100)
        });

        Bento.objects.attach(onigiri);
        Bento.objects.attach(sun);
        Bento.objects.attach(floor);

        Bento.objects.attach(levelSpawner);
        Bento.objects.attach(gameController);
        Bento.objects.attach(guiOverlay);
        Bento.objects.attach(drawController); // place after gui overlay to cancel throwing if needed

        if (Bento.saveState.load('cheat-background', 'default') !== 'Gradient...') {
            Bento.objects.attach(background);
        } else {
            //cheat: create gradient using css
            var gradientColor1 = Bento.saveState.load('cheat-gradient1', '#da8bcf');
            var gradientColor2 = Bento.saveState.load('cheat-gradient2', '#8bcbf5');
            document.body.style['background-image'] = 'linear-gradient(to bottom, ' + gradientColor1 + ', ' + gradientColor2 + ')';
        }

        // un-collect key
        Globals.didGetKey = false;
        // un-collect coins
        Globals.coinsCollected = 0;

        /**
         * Testing & debugging
         */
        var debugController = new Entity({
            name: 'debugController',
            components: [
                new Object({
                    name: 'behavior',
                    isDebugging: false,
                    update: function (data) {
                        if (!this.isDebugging && Bento.input.isKeyDown('space')) {
                            this.isDebugging = true;
                            bento.require(['onigiri/orbitcontrols'], function (OrbitControls) {
                                var orbitControls = new OrbitControls({
                                    // target: new THREE.Vector3(0, levelSpawner.size.height / 2 - 10 / 6, 0)
                                    target: Bento.objects.get('player').position.clone().add(new THREE.Vector3(0, 3, 0))
                                });
                                // Bento.objects.pause();
                                Bento.objects.attach(orbitControls);
                                orbitControls.updateWhenPaused = Bento.objects.isPaused();
                            });
                        }
                    },
                })
            ]
        });
        Bento.objects.attach(debugController);

        setupCamera();

        BannerAd.showBanner({
            noAds: Bento.saveState.load('cheat-noBanner', false) || Bento.saveState.load('noAds', false)
        });

        EventSystem.fire('UserStats-levelStart', {
            level: SaveValues.currentLevel
        });

        if (Bento.isDev()) {
            // console cheat
            window.nextLevel = function () {
                SaveValues.currentLevel += 1;
                Bento.screens.show('screens/main');
            };
        }
    };

    return new Screen({
        onShow: onShow
    });
});