/**
 * Screen just showing a model
 */
bento.define('screens/modelviewer', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/screen',
    'bento/tween',
    'onigiri/onigiri',
    'components/sun',
    'debug/html/html',
    'onigiri/orbitcontrols',
    'onigiri/entity3d',
    'onigiri/animationmixer',
    'components/materialswapper',
    'onigiri/animator'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Screen,
    Tween,
    Onigiri,
    Sun,
    Html,
    OrbitControls,
    Entity3D,
    AnimationMixer,
    MaterialSwapper,
    Animator
) {
    'use strict';
    var onShow = function (settings) {
        var listAllModels = function () {
            var dropDown = new Html('select', {
                id: 'modelsDropdown',
                onchange: function (evt) {
                    spawnModel(evt.target.value);
                    // focus canvas
                    var canvas = Html.getById('canvas');
                    if (canvas) {
                        canvas.focus();
                    }
                },
                style: {
                    position: 'absolute',
                    top: '0px',
                    left: '0px'
                }
            });

            var assets3d = Bento.assets.getAssets().meshes;
            var spawnModel = function (name) {
                Bento.objects.get('test', function (test) {
                    test.removeSelf();
                });
                var animationMixer = new Animator({
                    startAnimation: 'idle'
                });
                var mesh = Onigiri.getMesh(name);
                var testEntity = new Entity3D({
                    name: 'test',
                    object3D: mesh,
                    components: [
                        animationMixer,
                        // new MaterialSwapper({
                        //     id: 'modelviewer-' + name, // unique id for refcounting
                        //     deepSwap: true,
                        //     type: 'MeshStandardMaterial',
                        //     materialSettings: {
                        //         map: Onigiri.getTexture('textures/guard', {}),
                        //         skinning: true,
                        //         metalness: 0
                        //     }
                        // })
                    ]
                });
                Bento.objects.attach(testEntity);

                // fill animations

                var dropDownAnimations = Html.getById('animationsDropdown');
                if (dropDownAnimations) {
                    dropDownAnimations.removeSelf();
                }
                dropDownAnimations = new Html('select', {
                    id: 'animationsDropdown',
                    onchange: function (evt) {
                        // get animation mixer
                        // Bento.objects.get('test', function (test) {
                        //     test.getComponent('animationMixer', function (animationMixer) {
                        //     });
                        // });

                        animationMixer.stop();
                        animationMixer.play(evt.target.value);

                        // focus canvas
                        var canvas = Html.getById('canvas');
                        if (canvas) {
                            canvas.focus();
                        }
                    },
                    style: {
                        position: 'absolute',
                        top: '0px',
                        right: '0px'
                    }
                });
                Utils.forEach(animationMixer.getAnimations(), function (animation, animationName) {
                    dropDownAnimations.attach(new Html('option', {
                        value: animationName,
                        innerHtml: animationName
                    }));
                });

                Html.appendToBody(dropDownAnimations);

                window.lastKnownModel = name;
            };
            var lastKnown = window.lastKnownModel;
            var keys = [];
            var temp = {};

            // sort
            Utils.forEach(assets3d, function (model, key) {
                keys.push(key);
            });
            keys.sort();
            Utils.forEach(keys, function (key) {
                temp[key] = assets3d[key];
            });
            assets3d = temp;

            Utils.forEach(assets3d, function (model, name, l, breakLoop) {
                // if (name.endsWith('.gltf')) {
                if (!lastKnown) {
                    lastKnown = name;
                }
                dropDown.attach(new Html('option', {
                    value: name,
                    innerHtml: name
                }));
                // }
            });
            spawnModel(lastKnown);
            Html.appendToBody(dropDown);
        };
        var onigiri = new Onigiri({
            threeToPx: 16,
            backgroundColor: 0x000000,
            cameraStyle: 'perspective',
            perspectiveFieldOfView: 45,
            gammaFactor: 1.1,
        });
        var sun = new Sun({
            color: '#fff',
            directionalIntensity: 0.2,
            ambientIntensity: 1,
            position: new THREE.Vector3(5, 10, 5)
        });

        Bento.objects.attach(onigiri);
        Bento.objects.attach(sun);

        Bento.objects.attach(new OrbitControls({
            target: new THREE.Vector3(0, 1, 0)
        }));
        Onigiri.camera.position.x = 10;
        Onigiri.camera.position.y = 10;
        Onigiri.camera.position.z = 10;
        var divisions = 3;
        var size = Onigiri.pxToThree * 16 * divisions;
        var gridHelper = new THREE.GridHelper(size, divisions);
        var axesHelper = new THREE.AxesHelper(size);
        Onigiri.scene.add(axesHelper);
        Onigiri.scene.add(gridHelper);

        listAllModels();
        document.body.style['background-color'] = '#ccf';

    };

    return new Screen({
        onShow: onShow,
        onHide: function () {
            var dropDown = Html.getById('modelsDropdown');
            if (dropDown) {
                dropDown.removeSelf();
            }
            dropDown = Html.getById('animationsDropdown');
            if (dropDown) {
                dropDown.removeSelf();
            }

        }
    });
});