/**
 * Screen description
 */
bento.define('screens/preloader', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/components/fill',
    'bento/gui/text',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/screen',
    'bento/tween',
    'entities/luckykatlogo',
    'globals',
    'modules/localization',
    'gui/iphonexoverlay',
    'modules/abtest',
    'bento/components/anchor',
    'modules/notificationscheduler',
    'modules/notifications',
    'bento/components/spritecontainer'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Fill,
    Text,
    Entity,
    EventSystem,
    Utils,
    Screen,
    Tween,
    LuckyKat,
    Globals,
    Localization,
    IPhoneXOverlay,
    ABTest,
    Anchor,
    NotificationScheduler,
    Notifications,
    SpriteContainer
) {
    'use strict';
    var initPostPreloader = function () {
        // hide cordova splashscreen if exists
        if (navigator.splashscreen) {
            navigator.splashscreen.hide();
        }

        // Localization language is set after localized json files are loaded
        Localization.setLanguage();

        if (Bento.isDev()) {
            window.setLanguage = Localization.setLanguage;
        }
    };
    var onShow = function () {
        var loaded = false;
        var viewport = Bento.getViewport();
        var background = new Entity({
            z: 0,
            name: 'background',
            components: [
                new Fill({})
            ]
        });
        var luckyKat = new LuckyKat({});
        var loadingBar = new Entity({
            name: 'loadingBar',
            position: new Vector2(-48, 0),
            scale: new Vector2(0, 1),
            components: [
                // new Fill({
                //     name: 'fill',
                //     dimension: new Rectangle(0, 0, 96, 4),
                //     color: [1, 1, 1, 1], // [1, 1, 1, 1] is pure white
                //     originRelative: new Vector2(0, 0.5)
                // }),
                new SpriteContainer({
                    width: 96,
                    height: 4,
                    imageName: 'pixel',
                    originRelative: new Vector2(0, 0.5)
                })
            ]
        });
        var fakeLoad = 0;
        var loadingPercentage = 0;
        var loadDisplay = new Entity({
            z: 0,
            name: 'loadDisplay',
            position: new Vector2(viewport.width / 2, viewport.height / 2 + 128),
            scale: new Vector2(1.5, 1.5),
            components: [
                new Object({
                    name: 'loader',
                    update: function () {
                        if (!loaded) {
                            if (loadingPercentage === 0 && fakeLoad < 0.4) {
                                fakeLoad += 0.05;
                            }
                            loadingBar.scale.x = fakeLoad + (1 - fakeLoad) * loadingPercentage;
                        } else if (loaded === 1) {
                            loaded = 2;
                            new Tween({
                                from: loadingBar.scale.x,
                                to: 1,
                                in: 2 + Math.ceil((1 - loadingBar.scale.x) * 38),
                                delay: 0,
                                applyOnDelay: 0,
                                ease: 'easeInQuad',
                                decay: 1,
                                oscillations: 1,
                                onStart: function () {},
                                onUpdate: function (v, t) {
                                    loadingBar.scale.x = v;
                                },
                                onComplete: function () {
                                    Bento.screens.show('screens/main');
                                }
                            });
                        }
                    }
                }),
                new Sprite({
                    imageName: 'loadingbar',
                    originRelative: new Vector2(0.5, 0.5),
                }),
                loadingBar,
            ]
        });
        var end = function () {
            if (loaded) {
                Bento.screens.show('screens/main');
            }
        };
        var loadFonts = function () {
            var fonts = Bento.assets.getAssetGroups().preloader.fonts;
            var font;
            if (!fonts) {
                return;
            }
            for (font in fonts) {
                if (!fonts.hasOwnProperty(font)) {
                    continue;
                }
                new Text({
                    position: new Vector2(1000, 1000),
                    text: '.',
                    font: font
                });
            }
        };

        initPostPreloader();

        Bento.objects.attach(background);
        Bento.objects.attach(loadDisplay);
        Bento.objects.attach(luckyKat);

        // preload fonts
        loadFonts();

        if (Bento.isDev()) {
            Bento.objects.attach(new IPhoneXOverlay({
                z: Globals.Layers.IPHONE_X_OVERLAY,
                overrideInsets: true,
                landscapeMode: 'left',
            }));
        } else {

            // get extra room for banners (note: hardcoded this also in iPhoneXOverlay during dev)
            Anchor.insetOverrides.bottom = Anchor.insetOverrides.bottom || 0;
            Anchor.insetOverrides.bottom += 50;
        }

        Text.defaultSharpness = 3;

        // set tap style
        Globals.tapStyle = ABTest.getOption('B101_StopMethod') || 'tapAndHold';

        // ask for Notifications on booup
        // Notifications.init(function (success) {
        //     if (success) {
        //         EventSystem.fire('scheduleNotification', {
        //             id: 'notification',
        //             title: Localization.getText('notifications-title'),
        //             body: Localization.getText('notifications-body' + Utils.getRandom(3)),
        //             time: 24 * 60 * 60 * 1000
        //         });
        //     }
        // });

        // if user has allowed notifications initialize notification system otherwise we wait for after daily gift screen
//        if (Bento.saveState.load('notificationsAllowed', false)) {
//            Notifications.init(function (success) {
//                Notifications.resetBadgeNumber();
//                if (success) {
//                    EventSystem.fire('scheduleNotification', {
//                        id: 'notification',
//                        title: Localization.getText('notifications-title'),
//                        body: Localization.getText('notifications-body' + Utils.getRandom(3)),
//                        time: 24 * 60 * 60 * 1000
//                    });
//                }
//            });
//        }

        // we will now load all other assets
        Bento.assets.loadAllAssets({
            exceptions: ['preloader'], // preloader was already loaded
            onLoaded: function (current, total) {
                // show how many assets still to be loaded
                // text.setText('Loading ' + current + '/' + total);
                loadingPercentage = Math.min(current / total, 1);

            },
            onReady: function () {
                loaded = true;
                end();
            }
        });
    };

    return new Screen({
        onShow: onShow
    });
});